# encoding: utf-8

# -------------------------------------------------------------------------------
# Copyright 2023 Open Text.
#
# The only warranties for products and services of Open Text and its
# affiliates and licensors (“Open Text”) are as may be set forth in the
# express warranty statements accompanying such products and services.
# Nothing herein should be construed as constituting an additional
# warranty. Open Text shall not be liable for technical or editorial
# errors or omissions contained herein. The information contained herein
# is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains
# confidential information and a valid license is required for possession,
# use or copying. If this work is provided to the U.S. Government,
# consistent with FAR 12.211 and 12.212, Commercial Computer Software,
# Computer Software Documentation, and Technical Data for Commercial Items
# are licensed to the U.S. Government under vendor's standard commercial
# license.
# -------------------------------------------------------------------------------
# -------------------------------------------------------------------------------
# (C) Copyright 2019 - 2023 Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates
# and licensors (“Micro Focus”) are as may be set forth in the express warranty
# statements accompanying such products and services. Nothing herein should be
# construed as constituting an additional warranty. Micro Focus shall not be
# liable for technical or editorial errors or omissions contained herein. The
# information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential
# information and a valid license is required for possession, use or copying. If
# this work is provided to the U.S. Government, consistent with FAR 12.211 and
# 12.212, Commercial Computer Software, Computer Software Documentation, and
# Technical Data for Commercial Items  are licensed to the U.S. Government under
# vendor's standard commercial license.
# ------------------------------------------------------------------------------

# File:	clients/channel-upgrade.ycp
# Package:	Channel Upgrade Configuration of OES 2 products
# Summary:	Client for channel upgrade
# Authors:	Mark Clark <mclark@novell.com>
#
# $Id: channel-upgrade.ycp,v 1.0.0.0 2009/03/30 10:54:44 mclark Exp $
#
# This is a client for the Channel upgrade option of  OES 2023 SP1. Upgrades OES 2023 to OES 2023 SP1
module Yast
  class ChannelUpgradeOesClient < Client
    def main

      textdomain "novell-base"

      Builtins.y2milestone("----------------------------------------")
      Builtins.y2milestone("ChannelUpgrade:channel-upgrade started")

      Yast.import "Package"
      Yast.import "Popup"
      Yast.import "Service"
      Yast.import "Edirectory"
      Yast.import "NovellLDAP"
      Yast.import "NovellLDAPUI"
      Yast.import "NovellAddonInstall"
      Yast.import "NovellUtils"
      Yast.import "NovellPatterns"
      Yast.import "NovellLum"
      Yast.import "Ncpserver"
      Yast.import "Imanager"
      Yast.import "Iprint"
      Yast.import "Sms"
      Yast.import "Nss"
      Yast.import "NovellDns"
      Yast.import "NovellDhcp"
      Yast.import "Ncs"
      Yast.import "NovellCifs"
      Yast.import "NovellNssad"
      Yast.import "Umc"
      Yast.import "Stage"

      @ret = nil
      @answerSettings = {}

      @xadMode = false
      @oes_version = NovellUtils.OesVersion

      #  Main client driver code starts here
      # -----------------------------------------------------------------------------------------------------------------

      # Do any post config work...

      # Check and delete if /y2update folder exist

      dir = "/y2update"
      if FileUtils.Exists(dir)
	Builtins.y2milestone("/y2update folder exist")
        ret = Convert.to_integer(
          SCR.Execute(path(".target.bash"), Builtins.sformat("rm -rf %1",dir))
        )
        Builtins.y2milestone("Deleting /y2update folder and return value is %1",ret)
      end

      Popup.ShowFeedback(
        Builtins.sformat(_("Doing pre configuration tasks (Channel Upgrade for %1)"),@oes_version),
        _("This may take a while..........")
      )

      NovellUtils.channelUpgrade = true

      @ret = PreConfig()
      Builtins.y2milestone("ChannelUpgrade:ret from pre config = %1", @ret)

      SCR.Write(path(".sysconfig.novell-common.ABORT_OES_INSTALL"), "no")

      # First do an LDAP read to get the admin, etc.
      NovellLDAP.Read

      if Edirectory.xad_packages_installed || Edirectory.xadTestMode == true
        @xadMode = true
      end

      if @xadMode
        Report.Warning(NovellUtils.dsfwMixedModeUpgardeMsg)
      end

      Popup.ClearFeedback

      # Now, see if the answer file exists and if it has an obfuscated password..
      @answerSettings = ReadAnswerFile()
      if @answerSettings == nil || @answerSettings == {}
        Builtins.y2milestone(
          "ChannelUpgrade:answer file not found. Prompting for eDir password."
        )
        PromptForEdirAdminPassword()

        return :abort if NovellUtils.channelUpgrade_abort
      else
        # Now, set the LDAP password that we got from the answer file.
        Builtins.y2milestone("ChannelUpgrade:answer file found.")
        Ops.set(
          NovellLDAP.Settings,
          "CONFIG_LDAP_ADMIN_PASSWORD",
          Ops.get_string(@answerSettings, "CONFIG_LDAP_ADMIN_PASSWORD", "")
        )
      end
      # y2milestone("ChannelUpgrade:LDAP admin password = %1", NovellLDAP::Settings["CONFIG_LDAP_ADMIN_PASSWORD"]:"");
      # y2milestone("ChannelUpgrade:DSFW domain admin password (if dsfw is installed) = %1", answerSettings["CONFIG_DSFW_ADMIN_PASSWORD"]:"");

      #**************
      # eDirectory *
      #**************
      if DoUpgrade("edirectory")
        @statusMsg = Builtins.sformat(_("Upgrading eDirectory (Channel Upgrade %1)"),@oes_version)
        if @xadMode
          Builtins.y2milestone("ChannelUpgrade:doing eDirectory/DSFW upgrade.")
          @statusMsg = Builtins.sformat(
            _(
            "Upgrading eDirectory/DSFW (Channel Upgrade %1)"
            ),
            @oes_version
          )
        else
          Builtins.y2milestone("ChannelUpgrade:doing eDirectory upgrade.")
        end

        Popup.ShowFeedback(@statusMsg, _("This may take a while.........."))

        # Do an eDirectory Read to set up proper upgrade variables.
        Edirectory.channelUpgradeMode = true
        @ret = Edirectory.Read # This will also cause an LDAP read.
        Builtins.y2milestone(
          "ChannelUpgrade:ret from Edirectory::Read = %1",
          @ret
        )

        if @xadMode
          GetSetDsfwTreeAdminAndDomainAdminPasswords()

          return :abort if NovellUtils.channelUpgrade_abort
        end
        # Do the Edirectory::Write() to do the Edirectory upgrade.
        Ops.set(
          Edirectory.EdirSettings,
          "CONFIG_EDIR_OLD_NDSCONF_FILE",
          "/etc/opt/novell/eDirectory/conf/nds.conf"
        )
        @ret = Edirectory.Write
        return :abort if NovellUtils.channelUpgrade_abort
        if Edirectory.ndsconfigWorked == false
          Popup.Message(_("Aborting Channel Upgrade due to ndsconfig error."))
          Builtins.y2milestone(
            "ChannelUpgrade:aborting due to ndsconfig failure."
          )
          Edirectory.ndsconfigWorked = true
          return :next
        end
        Builtins.y2milestone(
          "ChannelUpgrade:ret from Edirectory::Write() = %1",
          @ret
        )

        Popup.ClearFeedback
      else
        Builtins.y2milestone("ChannelUpgrade:no eDir to upgrade.")
      end

      #*******
      # LUM *
      #*******
      if DoUpgrade("novell-lum")
        Builtins.y2milestone("ChannelUpgrade:doing LUM upgrade.")

        Popup.ShowFeedback(
          Builtins.sformat(
            _(
              "Configuring Linux User Management (Channel Upgrade for %1)"
            ),
            @oes_version
          ),
          _("This may take a while..........")
        )

        # Read the LUM settings.
        NovellLum.channelUpgradeMode = true
        @ret = NovellLum.Read
        Builtins.y2milestone(
          "ChannelUpgrade:ret from NovellLum::Read() = %1",
          @ret
        )

        # Do the NovellLum::Write() to perform upgrade
        @ret = NovellLum.Write
        Builtins.y2milestone(
          "ChannelUpgrade:ret from NovellLum::Write() = %1",
          @ret
        )

        Popup.ClearFeedback
      else
        Builtins.y2milestone("ChannelUpgrade:no LUM to upgrade.")
      end

      #**************
      # NCP Server *
      #**************
      if DoUpgrade("ncpserver")
        Builtins.y2milestone("ChannelUpgrade:doing NcpServer upgrade.")

        Popup.ShowFeedback(
          Builtins.sformat(_("Configuring NCP Server (Channel Upgrade for %1)"),@oes_version),
          _("This may take a while..........")
        )

        # Read the NCP Server settings.
        Ncpserver.channelUpgradeMode = true
        @ret = Ncpserver.Read
        Builtins.y2milestone(
          "ChannelUpgrade:ret from Ncpserver::Read() = %1",
          @ret
        )

        # Do the Ncpserver::Write() to perform upgrade
        @ret = Ncpserver.Write
        Builtins.y2milestone(
          "ChannelUpgrade:ret from Ncpserver::Write() = %1",
          @ret
        )

        Popup.ClearFeedback
      else
        Builtins.y2milestone("ChannelUpgrade:no NCP server to upgrade.")
      end

      #************
      # iManager *
      #************
      if DoUpgrade("imanager")
        Builtins.y2milestone("ChannelUpgrade:doing iManager upgrade.")

        Popup.ShowFeedback(
          Builtins.sformat(_("Configuring iManager (Channel Upgrade for %1)"),@oes_version),
          _("This may take a while..........")
        )

        #Make the channelUpgradeMode true
        Imanager.channelUpgradeMode = true

        # Read the iManager settings.
        @ret = Imanager.Read
        Builtins.y2milestone(
          "ChannelUpgrade:ret from Imanager::Read() = %1",
          @ret
        )

        # Do the Imanager::Write() to perform upgrade
        @ret = Imanager.Write
        Builtins.y2milestone(
          "ChannelUpgrade:ret from Imanager::Write() = %1",
          @ret
        )

        Popup.ClearFeedback
      else
        Builtins.y2milestone("ChannelUpgrade:no iManager to upgrade.")
      end

      #**********
      # iPrint *
      #**********
      if DoUpgrade("iprint")
        Builtins.y2milestone("ChannelUpgrade:doing iPrint upgrade.")

        Popup.ShowFeedback(
          Builtins.sformat(_("Configuring iPrint (Channel Upgrade for %1)"),@oes_version),
          _("This may take a while..........")
        )

        # Read the iPrint settings.
        Iprint.channelUpgradeMode = true
        @ret = Iprint.Read
        Builtins.y2milestone(
          "ChannelUpgrade:ret from Iprint::Read() = %1",
          @ret
        )

        # Do the Iprint::Write() to perform upgrade
        @ret = Iprint.Write
        Builtins.y2milestone(
          "ChannelUpgrade:ret from Iprint::Write() = %1",
          @ret
        )

        Popup.ClearFeedback
      else
        Builtins.y2milestone("ChannelUpgrade:no iPrint to upgrade.")
      end

      #*******
      # SMS *
      #*******
      if DoUpgrade("sms")
        Builtins.y2milestone("ChannelUpgrade:doing SMS upgrade.")

        Popup.ShowFeedback(
          Builtins.sformat(_("Configuring SMS (Channel Upgrade for %1)"),@oes_version),
          _("This may take a while..........")
        )

        # Read the SMS settings.
        Sms.channelUpgradeMode = true
        @ret = Sms.Read
        Builtins.y2milestone("ChannelUpgrade:ret from Sms::Read() = %1", @ret)

        # Do the Sms::Write() to perform upgrade
        @ret = Sms.Write
        Builtins.y2milestone("ChannelUpgrade:ret from Sms::Write() = %1", @ret)

        Popup.ClearFeedback
      else
        Builtins.y2milestone("ChannelUpgrade:no SMS to upgrade.")
      end

      #*******
      # NSS *
      #*******
      if DoUpgrade("nss")
        Builtins.y2milestone("ChannelUpgrade:doing NSS upgrade.")

        Popup.ShowFeedback(
          Builtins.sformat(_("Configuring NSS (Channel Upgrade for %1)"),@oes_version),
          _("This may take a while..........")
        )

        # Read the settings.
        Nss.channelUpgradeMode = true
        @ret = Nss.Read
        Builtins.y2milestone("ChannelUpgrade:ret from Nss::Read() = %1", @ret)

        # Do the Write() to perform upgrade
        @ret = Nss.Write
        Builtins.y2milestone("ChannelUpgrade:ret from Nss::Write() = %1", @ret)

        Popup.ClearFeedback
      else
        Builtins.y2milestone("ChannelUpgrade:no NSS to upgrade.")
      end

      #*************
      # NovellDns *
      #*************
      if DoUpgrade("novell-dns")
        Builtins.y2milestone("ChannelUpgrade:doing Novell DNS upgrade.")

        Popup.ShowFeedback(
          Builtins.sformat(_("Configuring OES DNS (Channel Upgrade for %1)"),@oes_version),
          _("This may take a while..........")
        )

        # Read the settings.
        NovellDns.channelUpgradeMode = true
        @ret = NovellDns.Read
        Builtins.y2milestone(
          "ChannelUpgrade:ret from NovellDns::Read() = %1",
          @ret
        )

        # Do the Write() to perform upgrade
        @ret = NovellDns.Write
        Builtins.y2milestone(
          "ChannelUpgrade:ret from NovellDns::Write() = %1",
          @ret
        )

        Popup.ClearFeedback
      else
        Builtins.y2milestone("ChannelUpgrade:no Novell DNS to upgrade.")
      end

      #**************
      # NovellDhcp *
      #**************
      if DoUpgrade("novell-dhcp")
        Builtins.y2milestone("ChannelUpgrade:doing Novell DHCP upgrade.")

        Popup.ShowFeedback(
          Builtins.sformat(_("Configuring OES DHCP (Channel Upgrade for %1)"),@oes_version),
          _("This may take a while..........")
        )

        # Read the settings.
        NovellDhcp.channelUpgradeMode = true
        @ret = NovellDhcp.Read
        Builtins.y2milestone(
          "ChannelUpgrade:ret from NovellDhcp::Read() = %1",
          @ret
        )

        # Do the Write() to perform upgrade
        @ret = NovellDhcp.Write
        Builtins.y2milestone(
          "ChannelUpgrade:ret from NovellDhcp::Write() = %1",
          @ret
        )

        Popup.ClearFeedback
      else
        Builtins.y2milestone("ChannelUpgrade:no Novell DHCP to upgrade.")
      end

      #*******
      # NCS *
      #*******
      if DoUpgrade("ncs")
        Builtins.y2milestone("ChannelUpgrade:doing NCS upgrade.")

        Popup.ShowFeedback(
          Builtins.sformat(_("Configuring NCS (Channel Upgrade for %1)"),@oes_version),
          _("This may take a while..........")
        )

        # Read the settings.
        Ncs.channelUpgradeMode = true
        @ret = Ncs.Read
        Builtins.y2milestone("ChannelUpgrade:ret from Ncs::Read() = %1", @ret)

        # Do the Write() to perform upgrade
        @ret = Ncs.Write
        Builtins.y2milestone("ChannelUpgrade:ret from Ncs::Write() = %1", @ret)

        Popup.ClearFeedback
      else
        Builtins.y2milestone("ChannelUpgrade:no NCS to upgrade.")
      end

      #********
      # CIFS *
      #********
      if DoUpgrade("novell-cifs")
        Builtins.y2milestone("ChannelUpgrade:doing Novell CIFS upgrade.")

        Popup.ShowFeedback(
          Builtins.sformat(_("Configuring OES CIFS (Channel Upgrade for %1)"),@oes_version),
          _("This may take a while..........")
        )

        # Read the settings.
        NovellCifs.channelUpgradeMode = true
        @ret = NovellCifs.Read
        Builtins.y2milestone(
          "ChannelUpgrade:ret from NovellCifs::Read() = %1",
          @ret
        )

        # Do the Write() to perform upgrade
        @ret = NovellCifs.Write
        Builtins.y2milestone(
          "ChannelUpgrade:ret from NovellCifs::Write() = %1",
          @ret
        )

        Popup.ClearFeedback
      else
        Builtins.y2milestone("ChannelUpgrade:no Novell CIFS to upgrade.")
      end

      #********
      # NSSAD *
      #********
      if DoUpgrade("novell-nssad")
        Builtins.y2milestone("ChannelUpgrade:doing Novell NSSAD upgrade.")

        Popup.ShowFeedback(
          Builtins.sformat(_("Configuring OES NSSAD (Channel Upgrade for %1)"),@oes_version),
          _("This may take a while..........")
        )

        # Read the settings.
        NovellNssad.channelUpgradeMode = true
        @ret = NovellNssad.Read
        Builtins.y2milestone(
          "ChannelUpgrade:ret from NovellNssad::Read() = %1",
          @ret
        )

        # Do the Write() to perform upgrade
        @ret = NovellNssad.Write
        Builtins.y2milestone(
          "ChannelUpgrade:ret from NovellNssad::Write() = %1",
          @ret
        )

        Popup.ClearFeedback
      else
        Builtins.y2milestone("ChannelUpgrade:no Novell NSSAD to upgrade.")
      end

      #********
      # UMC *
      #********
      if DoUpgrade("umc")
        Builtins.y2milestone("ChannelUpgrade:doing UMC upgrade.")

        Popup.ShowFeedback(
          Builtins.sformat(_("Configuring UMC (Channel Upgrade for %1)"),@oes_version),
          _("This may take a while..........")
        )

        # Read the settings.
        Umc.channelUpgradeMode = true
        @ret = Umc.Read
        Builtins.y2milestone(
          "ChannelUpgrade:ret from UMC::Read() = %1",
          @ret
        )

        # Do the Write() to perform upgrade
        @ret = Umc.Write
        Builtins.y2milestone(
          "ChannelUpgrade:ret from UMC::Write() = %1",
          @ret
        )

        Popup.ClearFeedback
      else
        Builtins.y2milestone("ChannelUpgrade:no UMC to upgrade.")
      end

      # Do any post config work...
      Popup.ShowFeedback(
        Builtins.sformat(_("Doing post configuration tasks (Channel Upgrade for %1)"),@oes_version),
        _("This may take a while..........")
      )

      @ret = PostConfig()
      Builtins.y2milestone("ChannelUpgrade:ret from post config = %1", @ret)

      @ret = CleanupFirstBoot()
      Builtins.y2milestone(
        "ChannelUpgrade:ret from cleaning up firstboot = %1",
        @ret
      )

      @ret = DeleteAnswerFile()
      Builtins.y2milestone(
        "ChannelUpgrade:ret from deleting answer file = %1",
        @ret
      )

      Popup.ClearFeedback

      Popup.ShowFeedback(
        Builtins.sformat(_("Upgrade Complete (Channel Upgrade for %1)"),@oes_version),
        ""
      )
      Builtins.sleep(5000) #sleep 5 seconds
      Popup.ClearFeedback

      Builtins.y2milestone("ChannelUpgrade:channel-upgrade finished")
      Builtins.y2milestone("----------------------------------------")

      :next

      # EOF
    end

    def CleanupFirstBoot
      ret = true
      cmd = "rm -f /etc/reconfig_system"
      Builtins.y2milestone("ChannelUpgrade.CleanupFirstBoot:executing: %1", cmd)

      ccode = Convert.to_integer(SCR.Execute(path(".target.bash"), cmd))
      if ccode != 0
        ret = false
        Builtins.y2milestone(
          "ChannelUpgrade.CleanupFirstBoot:error deleting /etc/reconfig_system."
        )
      end

      Builtins.y2milestone(
        "ChannelUpgrade.CleanupFirstBoot:disabling 'firstboot' service."
      )
      Service.Disable("firstboot")

      ret
    end

    def DeleteAnswerFile
      ret = true

      cmd = "rm -f /opt/novell/oes-install/answer"
      Builtins.y2milestone("ChannelUpgrade.DeleteAnswerFile:executing: %1", cmd)

      ccode = Convert.to_integer(SCR.Execute(path(".target.bash"), cmd))
      if ccode != 0
        ret = false
        Builtins.y2milestone(
          "ChannelUpgrade.DeleteAnswerFile:error deleting /etc/reconfig_system."
        )
      end

      Builtins.y2milestone(
        "ChannelUpgrade.DeleteAnswerFile:deleting password file."
      )

      ret
    end

    def PreConfig
      ret = true
      Builtins.y2milestone("Executing to set distribution")
      Yast.import "Bootloader"

      NovellUtils.SetupGrubDristributor()

      NovellUtils.ModifyingSfcb

      #Enabling and restarting logrotate.timer post the upgrade

      service = "logrotate.timer"
      if !Service.Enabled(service)
        Builtins.y2milestone("%1 is not eanbled",service)
        Service.Enable(service)
        Builtins.y2milestone("%1 is enabled",service)
        Service.Restart(service)
        Builtins.y2milestone("%1 restarted",service)
        Service.Restart("logrotate.service")
      end

      # Adding TLS_CACERTDIR, TLS_CERT and TLS_KEY to /etc/openldap/ldap.conf file

      NovellUtils.AddtoOpenLdapConfFile

      if Builtins.setenv("LDAPTLS_REQCERT", "never")
        Builtins.y2milestone("LDAPTLS_REQCERT is set")
      else
        Builtins.y2milestone("failed to set LDAPTLS_REQCERT")
      end


      # Starting haveged daemon

      if Service.Status("haveged") != 0
        Service.Enable("haveged")
        Service.Start("haveged")
        Builtins.y2milestone("channelUpgrade: started haveged")
      end

      if Service.Active("novell-tomcat")
        Service.Stop("novell-tomcat")
        Builtins.y2milestone("Stopping novell-tomcat service during channel upgrade to avoid java core, imanager will start novell-tomcat service")
      end
 
      # Temporary fix for oes11sp1, if btrfs file system is selected
      cmd = "/sbin/conf.d/SuSEconfig.permissions"
      rval = Convert.to_integer(SCR.Execute(path(".target.bash"), cmd))
      Builtins.y2milestone(
        "channelUpgrade: executing cmd: %1  rval: %2",
        cmd,
        rval
      )
      ret
    end


    def PostConfig
      ret = true

      # Allow ports 80 (HTTP), 443 (HTTPS), and 5989 (WBEM) through the firewall
      NovellUtils.SetFirewallPort(80, :tcp, true)
      NovellUtils.SetFirewallPort(443, :tcp, true)
      NovellUtils.SetFirewallPort(5989, :tcp, true)

      # Modify the openwbem.conf file to allow admin as well as root to access management data
      #Command executed from command line: sed -i 's/\([\t ]*owcimomd\.allowed_users[\t ]*\=\).*/\1 \*/' /etc/openwbem/openwbem.conf;
      #   string execute_cmd = "sed -i 's/\\([\\t ]*owcimomd\\.allowed_users[\\t ]*\\=\\).*/\\1 \\*/' /etc/openwbem/openwbem.conf";
      #   y2milestone("ChannelUpgrade:PostConfig: Executing: %1", execute_cmd);
      #   integer rval = (integer)SCR::Execute(.target.bash, execute_cmd);

      # Start red-carpet daemon (RCD)
      if Package.Installed("rcd")
        Service.Enable("rcd")
        Service.Start("rcd")
        Builtins.y2milestone(
          "ChannelUpgrade:PostConfig:started and enabled red-carpet daemon (rcd"
        )
      end
      # Start novfsd
      if Package.Installed("novell-novfsd")
        Service.Enable("novfsd")
        Service.Start("novfsd")
        Builtins.y2milestone("ChannelUpgrade:started novfsd")
      end


      # Start CIMOM - always installed
      #  Service::Enable("owcimomd");
      # Service::Stop("owcimomd");
      #  y2milestone("ChannelUpgrade:PostConfig:stopping owcimomd in preparation for restart.");
      #  Service::Start("owcimomd");
      #  y2milestone("ChannelUpgrade:PostConfig:started and enabled owcimomd");

      #Start SFCB - always installed
      Service.Enable("sblim-sfcb")
      Service.Stop("sblim-sfcb")
      Builtins.y2milestone(
        "ChannelUpgrade:PostConfig:stopping sblim-sfcb in preparation for restart."
      )
      Service.Start("sblim-sfcb")
      Builtins.y2milestone("ChannelUpgrade:PostConfig:started and enabled sblim-sfcb")

#      Removing iFolder files during upgrade
      ifolder_files= ["/etc/apache2/conf.d/simias.conf","/etc/apache2/conf.d/ifolder_admin.conf","/etc/apache2/conf.d/ifolder_web.conf","/etc/opt/novell/httpd/conf.d/base.conf","/opt/novell/ifolder3"]
       Builtins.y2milestone("Removing ifolder files during upgrade")
       Builtins.foreach(ifolder_files) do |file|
         if FileUtils.Exists(file)
          ret = Convert.to_integer(
            SCR.Execute(path(".target.bash"), Builtins.sformat("rm -f %1",file))
          )
          Builtins.y2milestone("executed rm %1 and return value is %2",file,ret)
          end
       end
       ret = Convert.to_integer(
          SCR.Execute(path(".target.bash"), Builtins.sformat("rm -f /etc/sysconfig/novell/ifldr3_*"))
          )
       Builtins.y2milestone("executed rm /etc/sysconfig/novell/ifldr3_* and return value is %1",ret)
       ret = Convert.to_integer(
          SCR.Execute(path(".target.bash"), Builtins.sformat("unlink /etc/opt/novell/httpd/conf.d/base.conf"))
          )
       Builtins.y2milestone("executed rm /etc/opt/novell/httpd/conf.d/base.conf and return value is %1",ret)

      # searching and repcaling Order deny,allow, Order allow,deny & Allow from all to Require all granted

      @cmd = "grep 'Allow from all' /etc/opt/novell/httpd/conf.d/iprint_g.conf > /dev/null"
      @rval = Convert.to_integer(SCR.Execute(path(".target.bash"), @cmd))
      Builtins.y2milestone("inst_oes_preconfig: executing cmd: %1  rval: %2",@cmd,@rval)
      if @rval == 0
        @cmd2 = "sed -i 's/Allow from all/Require all granted/' /etc/opt/novell/httpd/conf.d/iprint_g.conf"
        @rval2 = Convert.to_integer(SCR.Execute(path(".target.bash"), @cmd2))
        Builtins.y2milestone("inst_oes_preconfig: executing cmd: %1  rval: %2",@cmd2,@rval2)
      end

      @cmd = "grep  'Order deny\,allow'  /etc/opt/novell/httpd/conf.d/iprint_g.conf > /dev/null"
      @rval = Convert.to_integer(SCR.Execute(path(".target.bash"), @cmd))
      Builtins.y2milestone("inst_oes_preconfig: executing cmd: %1  rval: %2",@cmd,@rval)
      if @rval == 0
        @cmd2 = "sed -i '/Order deny\,allow/d' /etc/opt/novell/httpd/conf.d/iprint_g.conf"
        @rval2 = Convert.to_integer(SCR.Execute(path(".target.bash"), @cmd2))
        Builtins.y2milestone(
          "inst_oes_preconfig: executing cmd: %1  rval: %2",@cmd2,@rval2)
      end
@cmd = "grep  'Order allow\,deny'  /etc/opt/novell/httpd/conf.d/iprint_g.conf > /dev/null"
      Builtins.y2milestone("inst_oes_preconfig: executing cmd: %1  rval: %2",@cmd,@rval)
      @rval = Convert.to_integer(SCR.Execute(path(".target.bash"), @cmd))
      if @rval == 0
        @cmd2 = "sed -i '/Order allow\,deny/d' /etc/opt/novell/httpd/conf.d/iprint_g.conf"
        @rval2 = Convert.to_integer(SCR.Execute(path(".target.bash"), @cmd2))
        Builtins.y2milestone("inst_oes_preconfig: executing cmd: %1  rval: %2",@cmd2,@rval2)
      end

      # Removing Order deny,allow, & Allow from all

      @cmd = "grep  'Order deny,allow'  /etc/opt/novell/httpd/conf.d/iprint_ssl.conf > /dev/null"
      @rval = Convert.to_integer(SCR.Execute(path(".target.bash"), @cmd))
      Builtins.y2milestone("inst_oes_preconfig: executing cmd: %1  rval: %2",@cmd,@rval)
      if @rval == 0
        @cmd2 = "sed -i '/Order deny,allow/d' /etc/opt/novell/httpd/conf.d/iprint_ssl.conf"
        @rval2 = Convert.to_integer(SCR.Execute(path(".target.bash"), @cmd2))
        Builtins.y2milestone(
          "inst_oes_preconfig: executing cmd: %1  rval: %2",@cmd2,@rval2)
      end

      @cmd = "grep  'Allow from all'  /etc/opt/novell/httpd/conf.d/iprint_ssl.conf > /dev/null"
      Builtins.y2milestone("inst_oes_preconfig: executing cmd: %1  rval: %2",@cmd,@rval)
      @rval = Convert.to_integer(SCR.Execute(path(".target.bash"), @cmd))
      if @rval == 0
        @cmd2 = "sed -i '/Allow from all/d' /etc/opt/novell/httpd/conf.d/iprint_ssl.conf"
        @rval2 = Convert.to_integer(SCR.Execute(path(".target.bash"), @cmd2))
        Builtins.y2milestone("inst_oes_preconfig: executing cmd: %1  rval: %2",@cmd2,@rval2)
      end


      # Call the welcome page setup script to register all installed products.
      cmd = "/etc/opt/novell/httpd/scripts/wpincludes.pl rebuild"
      Builtins.y2milestone("ChannelUpgrade:PostConfig: Executing: %1", cmd)
      rval = Convert.to_integer(SCR.Execute(path(".target.bash"), cmd))
      if rval == 0
        Builtins.y2milestone(
          "ChannelUpgrade:PostConfig:welcome page wpincludes.pl script called successfully."
        )
      else
        Builtins.y2milestone(
          "ChannelUpgrade:PostConfig:welcome page wpincludes.pl script returned error: %1",
          rval
        )
      end

      # restart SMS
      Service.Restart("novell-smdrd")
      Builtins.y2milestone(
        "ChannelUpgrade:PostConfig:restarted and enabled novell-smdrd"
      )

      execute_cmd = "/opt/novell/sms/bin/smsconfig -l tsafs"
      rval = Convert.to_integer(SCR.Execute(path(".target.bash"), execute_cmd))
      Builtins.y2milestone(
        "ChannelUpgrade:PostConfig: executed: %1",
        execute_cmd
      )

      # Restart eDirectory
      Service.Stop("ndsd")
      Service.Start("ndsd")
      Builtins.y2milestone("ChannelUpgrade:PostConfig:restarted edir")

      # Restart CIFS
      Service.Stop("novell-cifs")
      Service.Start("novell-cifs")
      Builtins.y2milestone("ChannelUpgrade:PostConfig:restarted cifs")

      # Restart ipsmd
      Service.Stop("novell-ipsmd")
      Service.Start("novell-ipsmd")
      Builtins.y2milestone("ChannelUpgrade:PostConfig:restarted novell-ipsmd")

      cmd = "rm -r /var/opt/novell/tomcat/work/Catalina/localhost/nps/org"
      Builtins.y2milestone("ChannelUpgrade:PostConfig: Executing: %1", cmd)
      rval = Convert.to_integer(SCR.Execute(path(".target.bash"), cmd))


      #Start XTIER
      start_xtier = Builtins.tolower(NovellUtils.ReadOption("START_XTIER"))
      Builtins.y2milestone("channelUpgrade:start xtier is %1", start_xtier)
      if start_xtier == "yes"
        Service.Enable("novell-xregd")
        Service.Enable("novell-xsrvd")
        if Stage.normal
          Service.Restart("novell-xregd")
          Service.Restart("novell-xsrvd")
          Builtins.y2milestone(
            "channelUpgrade:started novell-xregd and novell-xsrvd"
          )
        else
          Service.Start("novell-xregd")
          Service.Start("novell-xsrvd")
          Builtins.y2milestone(
            "channelUpgrade:restarted novell-xregd and novell-xsrvd"
          )
        end
        SCR.Write(path(".sysconfig.novell-common.START_XTIER"), "no")
      end
      #Start DNS
      start_dns = Builtins.tolower(NovellUtils.ReadOption("START_DNS"))
      Builtins.y2milestone("channelUpgrade:start dns is %1", start_dns)
      if start_dns == "yes"
        Service.Enable("novell-named")
        if Stage.normal
          Service.Restart("novell-named")
          Builtins.y2milestone("channelUpgrade:restarted novell-named")
        else
          Service.Start("novell-named")
          Builtins.y2milestone("channelUpgrade:started novell-named")
        end
      end
      if start_dns == "yes" || NovellPatterns.IsPatternInstalled("novell-dhcp")
        SCR.Write(path(".sysconfig.novell-common.START_DNS"), "no")
        if Service.Status("boot.apparmor") == 0
          Service.Enable("boot.apparmor")
          if Stage.normal
            Service.Reload("boot.apparmor")
            Builtins.y2milestone("channelUpgrade:restarted apparmor")
          end
        else
          Builtins.y2milestone(
            "channelUpgrade:apparmor not running,profiles will not be reloaded"
          )
        end
      end
      #Start eDir
      start_edir = Builtins.tolower(NovellUtils.ReadOption("START_EDIR"))
      if start_edir == "yes"
        if Stage.normal
          Service.Stop("ndsd")
          Service.Start("ndsd")
          Service.Stop("novell-cifs")
          Service.Start("novell-cifs")
          Builtins.y2milestone("channelUpgrade:restarted edir")
        else
          Service.Start("ndsd")
          Service.Stop("novell-cifs")
          Service.Start("novell-cifs")
          Builtins.y2milestone("channelUpgrade:started edir")
        end
        SCR.Write(path(".sysconfig.novell-common.START_EDIR"), "no")
      end


      Service.Enable("novell-tomcat")
      Service.Enable("apache2")

      Service.Stop("apache2")
      Service.Stop("novell-tomcat")
      Service.Start("novell-tomcat")
      Service.Start("apache2")

      Builtins.y2milestone(
        "ChannelUpgrade:PostConfig:restarted and enabled apache2 and novell-tomcat"
      )

      Builtins.y2milestone(
        "channelUpgrade: Opening ftp ports, if 'novell-ftp' pattern installed."
      )
      if NovellPatterns.IsPatternInstalled("novell-ftp")
        Service.Enable("pure-ftpd")
        NovellUtils.SetFirewallPort(20, :both, true)
        NovellUtils.SetFirewallPort(21, :both, true)
      end

      # reinitialize the firewall system
      old_mode = Mode.mode
      begin
        Mode.SetMode("normal")
        Builtins.y2milestone("Setting mode to %1",Mode.mode)
        NovellUtils.ReloadFirewall
      ensure
        Mode.SetMode(old_mode)
        Builtins.y2milestone("Setting mode to %1",Mode.mode)
      end

      Builtins.y2milestone("ChannelUpgrade:PostConfig: restarting slpd.")
      cmd = "/usr/sbin/rcslpd restart"
      Builtins.y2milestone("ChannelUpgrade:PostConfig:executing: %1", cmd)
      rval = Convert.to_integer(SCR.Execute(path(".target.bash"), cmd))

      #Start DSFW upgrade script if required
      start_dsfw_upgrade = Builtins.tolower(
        NovellUtils.ReadOption("START_DSFW_UPGRADE")
      )
      Builtins.y2milestone(
        "ChannelUpgrade:PostConfig:start dsfw upg is %1",
        start_dsfw_upgrade
      )
      if start_dsfw_upgrade == "yes"
        cmd2 = "LD_LIBRARY_PATH=/opt/novell/eDirectory/eDir-exclusive/lib64/:$LD_LIBRARY_PATH perl /opt/novell/xad/sbin/upgrade_dsfw.pl"
        adminPassword = Ops.get_string(NovellLDAP.Settings,"CONFIG_LDAP_ADMIN_PASSWORD","")

        Builtins.y2milestone("channel-upgrade-oes:executing %1", cmd2)

        retmap = Convert.to_map(
        SCR.Execute(
          path(".target.bash_output"),
          Ops.add(Ops.add(Ops.add("ADM_PASSWD='", adminPassword), "' "), cmd2)))

        Builtins.y2milestone("channel-upgrade-oes:std is %1", retmap)

        Builtins.y2milestone("channelUpgrade:std is %1", retmap)
        exit = Ops.get_integer(retmap, "exit", -1)
        if exit != 0
          if exit == 2
            Popup.Message(NovellUtils.dsfwUpgradeMsg_2)
          else
            NovellUtils.PopupErrorDetailsWithLogFile(
              NovellUtils.dsfwUpgradeMsg,
              retmap,
              Ops.get_string(retmap, "cmd", ""),
              "/var/opt/novell/xad/log/ndsdcinit.log"
            )
          end
        end
        SCR.Write(path(".sysconfig.novell-common.START_DSFW_UPGRADE"), "no")
      end

      NovellUtils.AddtoOpenLdapConfFile

      ## CIS Data scale pattern activation script
      version_file = "/etc/opt/novell/cis-scale/VERSION"
      if NovellPatterns.IsPatternInstalled("cloud-integrated-storage-data-scale") && !FileUtils::Exists(version_file)
        cmd = Builtins.sformat("/opt/novell/cis/bin/activate_cis_scale.sh")
        Builtins.y2milestone("OesCis-Scale:Running /opt/novell/cis/bin/activate_cis_scale.sh")
        val = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
        exit = Ops.get_integer(val, "exit", -1)
        stdout = Ops.get_string(val, "stdout", "")
        Builtins.y2milestone(
         "OesCis-Scale:Activating CIS scale service returned:%1, with stdout:\n%2",
         val, stdout )
        cmd = Builtins.sformat("/sbin/sysctl -w net.ipv4.ip_forward=1")
        val = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
        Builtins.y2milestone("OesCis-Scale:Setting IPV4 forwarding : %1",val)
      end


      if Builtins.setenv("LDAPTLS_REQCERT", "")
        Builtins.y2milestone("LDAPTLS_REQCERT value is unset")
      else
        Builtins.y2milestone("Failed to unset TLSLDAP_REQCERT")
      end

      ret
    end

    def EdirDependenciesOK
      # verify the eDirectory pattern is installed...
      if Edirectory.xadTestMode == true || Edirectory.upgradeTestMode == true
        return true
      end

      if !NovellPatterns.EnsurePatternInstalled("novell-edirectory")
        return false
      end

      true
    end

    # Assign a valid password character to a random number
    def LoadDeTranslationMap
      ret = {}

      Ops.set(ret, 46, "-") # 1
      Ops.set(ret, 122, "0") # 2
      Ops.set(ret, 18, "1") # 3
      Ops.set(ret, 6, "2") # 4
      Ops.set(ret, 20, "3") # 5
      Ops.set(ret, 32, "4") # 6
      Ops.set(ret, 10, "5") # 7
      Ops.set(ret, 60, "6") # 8
      Ops.set(ret, 70, "7") # 9
      Ops.set(ret, 5, "8") # 10
      Ops.set(ret, 66, "9") # 11
      Ops.set(ret, 209, "a") # 12
      Ops.set(ret, 11, "b") # 13
      Ops.set(ret, 168, "c") # 14
      Ops.set(ret, 102, "d") # 15
      Ops.set(ret, 180, "e") # 16
      Ops.set(ret, 2, "f") # 17
      Ops.set(ret, 152, "g") # 18
      Ops.set(ret, 80, "h") # 19
      Ops.set(ret, 162, "i") # 20
      Ops.set(ret, 187, "j") # 21
      Ops.set(ret, 15, "k") # 22
      Ops.set(ret, 151, "l") # 23
      Ops.set(ret, 21, "m") # 24
      Ops.set(ret, 87, "n") # 25
      Ops.set(ret, 30, "o") # 26
      Ops.set(ret, 1, "p") # 27
      Ops.set(ret, 65, "q") # 28
      Ops.set(ret, 163, "r") # 29
      Ops.set(ret, 81, "s") # 30
      Ops.set(ret, 67, "t") # 31
      Ops.set(ret, 71, "u") # 32
      Ops.set(ret, 17, "v") # 33
      Ops.set(ret, 167, "w") # 34
      Ops.set(ret, 25, "x") # 35
      Ops.set(ret, 185, "y") # 36
      Ops.set(ret, 104, "z") # 37
      Ops.set(ret, 24, "A") # 38
      Ops.set(ret, 61, "B") # 39
      Ops.set(ret, 88, "C") # 40
      Ops.set(ret, 186, "D") # 41
      Ops.set(ret, 33, "E") # 42
      Ops.set(ret, 14, "F") # 43
      Ops.set(ret, 166, "G") # 44
      Ops.set(ret, 72, "H") # 45
      Ops.set(ret, 85, "I") # 46
      Ops.set(ret, 62, "J") # 47
      Ops.set(ret, 36, "K") # 48
      Ops.set(ret, 106, "L") # 49
      Ops.set(ret, 26, "M") # 50
      Ops.set(ret, 181, "N") # 51
      Ops.set(ret, 206, "O") # 52
      Ops.set(ret, 207, "P") # 53
      Ops.set(ret, 3, "Q") # 54
      Ops.set(ret, 74, "R") # 55
      Ops.set(ret, 22, "S") # 56
      Ops.set(ret, 164, "T") # 57
      Ops.set(ret, 86, "U") # 58
      Ops.set(ret, 68, "V") # 59
      Ops.set(ret, 183, "W") # 60
      Ops.set(ret, 103, "X") # 61
      Ops.set(ret, 35, "Y") # 62
      Ops.set(ret, 150, "Z") # 63
      Ops.set(ret, 82, "!") # 64
      Ops.set(ret, 160, "@") # 65
      Ops.set(ret, 31, "#") # 66
      Ops.set(ret, 16, "$") # 67
      Ops.set(ret, 182, "%") # 68
      Ops.set(ret, 63, "^") # 69
      Ops.set(ret, 84, "&") # 70
      Ops.set(ret, 165, "*") # 71
      Ops.set(ret, 23, "~") # 72
      Ops.set(ret, 184, "(") # 73
      Ops.set(ret, 101, ")") # 74
      Ops.set(ret, 161, ",") # 75
      Ops.set(ret, 89, ";") # 76
      Ops.set(ret, 205, ":") # 77
      Ops.set(ret, 64, ".") # 78
      Ops.set(ret, 13, "_") # 79
      Ops.set(ret, 34, "+") # 80
      Ops.set(ret, 83, "/") # 81
      Ops.set(ret, 69, "|") # 82
      Ops.set(ret, 73, "?") # 83
      Ops.set(ret, 4, "{") # 84
      Ops.set(ret, 208, "}") # 85
      Ops.set(ret, 100, "=") # 86
      Ops.set(ret, 105, "[") # 87
      Ops.set(ret, 12, "]") # 88

      deep_copy(ret)
    end

    # How much to bit shift the value based on string position
    def LoadBitshiftList
      ret = []

      Ops.set(ret, 0, 3) # 1
      Ops.set(ret, 1, 1) # 2
      Ops.set(ret, 2, 2) # 3
      Ops.set(ret, 3, 4) # 4
      Ops.set(ret, 4, 3) # 5
      Ops.set(ret, 5, 1) # 6
      Ops.set(ret, 6, 4) # 7
      Ops.set(ret, 7, 2) # 8
      Ops.set(ret, 8, 4) # 9
      Ops.set(ret, 9, 2) # 10
      Ops.set(ret, 10, 1) # 11
      Ops.set(ret, 11, 3) # 12
      Ops.set(ret, 12, 1) # 13
      Ops.set(ret, 13, 2) # 14
      Ops.set(ret, 14, 1) # 15
      Ops.set(ret, 15, 4) # 16
      Ops.set(ret, 16, 3) # 17
      Ops.set(ret, 17, 2) # 18
      Ops.set(ret, 18, 1) # 19
      Ops.set(ret, 19, 3) # 20
      Ops.set(ret, 20, 4) # 21
      Ops.set(ret, 21, 2) # 22
      Ops.set(ret, 22, 1) # 23
      Ops.set(ret, 23, 3) # 24
      Ops.set(ret, 24, 1) # 25
      Ops.set(ret, 25, 4) # 26
      Ops.set(ret, 26, 1) # 27
      Ops.set(ret, 27, 4) # 28
      Ops.set(ret, 28, 2) # 29
      Ops.set(ret, 29, 1) # 30
      Ops.set(ret, 30, 3) # 31
      Ops.set(ret, 31, 2) # 32
      Ops.set(ret, 32, 4) # 33
      Ops.set(ret, 33, 2) # 34
      Ops.set(ret, 34, 3) # 35
      Ops.set(ret, 35, 1) # 36
      Ops.set(ret, 36, 2) # 37
      Ops.set(ret, 37, 4) # 38
      Ops.set(ret, 38, 3) # 39
      Ops.set(ret, 39, 1) # 40
      Ops.set(ret, 40, 2) # 41
      Ops.set(ret, 41, 4) # 42
      Ops.set(ret, 42, 3) # 43
      Ops.set(ret, 43, 4) # 44
      Ops.set(ret, 44, 2) # 45
      Ops.set(ret, 45, 1) # 46
      Ops.set(ret, 46, 3) # 47
      Ops.set(ret, 47, 2) # 48
      Ops.set(ret, 48, 1) # 49
      Ops.set(ret, 49, 4) # 50
      Ops.set(ret, 50, 3) # 51
      Ops.set(ret, 51, 4) # 52
      Ops.set(ret, 52, 3) # 53
      Ops.set(ret, 53, 3) # 54
      Ops.set(ret, 54, 1) # 55
      Ops.set(ret, 55, 2) # 56
      Ops.set(ret, 56, 4) # 57
      Ops.set(ret, 57, 3) # 58
      Ops.set(ret, 58, 1) # 59
      Ops.set(ret, 59, 2) # 60
      Ops.set(ret, 60, 1) # 61
      Ops.set(ret, 61, 4) # 62
      Ops.set(ret, 62, 3) # 63
      Ops.set(ret, 63, 4) # 64
      Ops.set(ret, 64, 1) # 65
      Ops.set(ret, 65, 2) # 66
      Ops.set(ret, 66, 4) # 67
      Ops.set(ret, 67, 3) # 68
      Ops.set(ret, 68, 1) # 69
      Ops.set(ret, 69, 4) # 70
      Ops.set(ret, 70, 2) # 71
      Ops.set(ret, 71, 3) # 72
      Ops.set(ret, 72, 1) # 73
      Ops.set(ret, 73, 4) # 74
      Ops.set(ret, 74, 2) # 75
      Ops.set(ret, 75, 1) # 76
      Ops.set(ret, 76, 3) # 77
      Ops.set(ret, 77, 2) # 78
      Ops.set(ret, 78, 4) # 79
      Ops.set(ret, 79, 3) # 80
      Ops.set(ret, 80, 2) # 81
      Ops.set(ret, 81, 1) # 82
      Ops.set(ret, 82, 3) # 83
      Ops.set(ret, 83, 4) # 84
      Ops.set(ret, 84, 3) # 85
      Ops.set(ret, 85, 1) # 86
      Ops.set(ret, 86, 3) # 87
      Ops.set(ret, 87, 4) # 88

      deep_copy(ret)
    end

    # UnObfuscates a string
    def UnObfuscate(str)
      # Initialize obfuscation tables
      tMap = LoadDeTranslationMap()
      bitshift = LoadBitshiftList()

      split = Builtins.splitstring(str, ",")
      sizePword = Builtins.size(split)
      character = ""
      cnt = 0
      val = 0
      clearstring = ""

      Builtins.foreach(split) do |intVal|
        val = Builtins.tointeger(intVal)
        val = Ops.shift_right(val, Ops.get(bitshift, cnt, 0))
        val = Ops.subtract(val, sizePword)
        character = Ops.get_string(tMap, val, "")
        clearstring = Ops.add(clearstring, character)
        cnt = Ops.add(cnt, 1)
      end

      clearstring
    end

    # This file may contain 1 or 2 passwords. The first is the LDAP admin and the second (optional) is the DSFW Domain Admin.
    def ReadAnswerFile
      ret = {}
      data_file = "/opt/novell/oes-install/answer"
      stmp = []

      # First see if the file exists.
      if SCR.Read(path(".target.size"), data_file) == -1
        return nil # nil indicates empty or no file
      end

      # Read the file into memory
      answerData = Convert.to_string(
        SCR.Read(path(".target.string"), data_file)
      )
      if answerData != ""
        answerData = NovellUtils.StripLeading(answerData, "\n")

        # break up into individual lines
        stmp = Builtins.splitstring(answerData, "\n")
      else
        stmp = nil
      end # empty or no file;


      if stmp != nil
        Ops.set(
          ret,
          "CONFIG_LDAP_ADMIN_PASSWORD",
          UnObfuscate(Ops.get(stmp, 0, ""))
        )
        if Ops.get(stmp, 2, "") != nil && Ops.get(stmp, 2, "") != ""
          Ops.set(
            ret,
            "CONFIG_DSFW_ADMIN_PASSWORD",
            UnObfuscate(Ops.get(stmp, 2, ""))
          )
        end
      end

      deep_copy(ret)
    end


    def PromptForEdirAdminPassword
      useXad = false

      if @xadMode
        xad_tree_admin_context = Ops.get_string(
          NovellLDAP.Settings,
          "XAD_TREE_ADMIN_CONTEXT",
          ""
        )
        if xad_tree_admin_context != nil && xad_tree_admin_context != ""
          useXad = true
        end # if this value is set at all, then it is the eDir admin (in spite of xad name)
      end
      Builtins.y2milestone(
        "ChannelUpgrade:PromptIfNeededForEdirAdminPassword:use xad:%1",
        useXad
      )

      if useXad
        if !NovellLDAPUI.XadTreeAdminPasswordDialogChannelUpgrade
          NovellUtils.channelUpgrade_abort = true
          return :abort
        end
      else
        if !NovellLDAPUI.AdminPasswordDialogChannelUpgrade
          NovellUtils.channelUpgrade_abort = true
          return :abort
        end
      end
      Builtins.y2milestone(
        "ChannelUpgrade:PromptForEdirAdminPassword:useXad %1",
        useXad
      )
      useXad
    end

    def PromptForDomainAdminPassword
      domainPassword = NovellLDAPUI.XadDomainAdminPasswordDialog

      domainPassword
    end

    # See if a product is installed and if it needs to be upgraded
    def DoUpgrade(product)
      #Returns true for any of these upgrades

      NovellAddonInstall.IsProductUpgradeOES(product)
    end


    # If dsfw is involved, password locations may be different
    def GetSetDsfwTreeAdminAndDomainAdminPasswords
      retVal = true

      adminPassword = Ops.get_string(
        NovellLDAP.Settings,
        "CONFIG_LDAP_ADMIN_PASSWORD",
        ""
      )

      treeType = Ops.get_string(
        Edirectory.EdirSettings,
        "CONFIG_EDIR_TREE_TYPE",
        ""
      )
      domainType = Ops.get_string(
        Edirectory.EdirSettings,
        "XAD_DOMAIN_TYPE",
        ""
      )
      adminName = Ops.get_string(
        NovellLDAP.Settings,
        "CONFIG_LDAP_ADMIN_CONTEXT",
        ""
      )

      if @answerSettings != nil && @answerSettings != {}
        if Ops.get_string(@answerSettings, "CONFIG_DSFW_ADMIN_PASSWORD", "") != nil &&
            Ops.get_string(@answerSettings, "CONFIG_DSFW_ADMIN_PASSWORD", "") != ""
          domainAdminPassword = Ops.get_string(
            @answerSettings,
            "CONFIG_DSFW_ADMIN_PASSWORD",
            ""
          )
        end
        if treeType == "existing"
          Ops.set(
            NovellLDAP.Settings,
            "XAD_TREE_ADMIN_PASSWORD",
            Ops.get_string(@answerSettings, "CONFIG_LDAP_ADMIN_PASSWORD", "")
          )
        end
      end

      # We only need the Domain Admin password if we were joining an existing tree
      if treeType == "existing"
        adminName = Ops.get_string(
          NovellLDAP.Settings,
          "XAD_TREE_ADMIN_CONTEXT",
          ""
        )
        adminPassword = Ops.get_string(
          NovellLDAP.Settings,
          "XAD_TREE_ADMIN_PASSWORD",
          ""
        )
        if domainAdminPassword == nil || domainAdminPassword == ""
          domainAdminPassword = PromptForDomainAdminPassword()
        end
        Ops.set(Edirectory.EdirSettings, "XAD_LDAP_ADMIN_CONTEXT", adminName)
        Ops.set(NovellLDAP.Settings, "XAD_TREE_ADMIN_PASSWORD", adminPassword)
        Ops.set(
          Edirectory.EdirSettings,
          "XAD_LDAP_ADMIN_PASSWORD",
          adminPassword
        )

        if Ops.get_string(Edirectory.EdirSettings, "XAD_DOMAIN_TYPE", "") == "forest"
          Ops.set(
            Edirectory.EdirSettings,
            "XAD_ADMIN_PASSWORD",
            domainAdminPassword
          )
        else
          Ops.set(
            NovellLDAP.Settings,
            "CONFIG_LDAP_ADMIN_PASSWORD",
            domainAdminPassword
          )
        end
      else
        Ops.set(NovellLDAP.Settings, "CONFIG_LDAP_ADMIN_CONTEXT", adminName)
        Ops.set(
          NovellLDAP.Settings,
          "CONFIG_LDAP_ADMIN_PASSWORD",
          adminPassword
        )
        Ops.set(Edirectory.EdirSettings, "XAD_LDAP_ADMIN_CONTEXT", adminName)
      end

      retVal
    end
  end
end

Yast::ChannelUpgradeOesClient.new.main
