# encoding: utf-8

# File:        clients/inst_check_cert.ycp
# Package:     novell-common
# Summary:     Check for a certificate and warn user if we can not find one
# Authors:     Stephen T. Bunch <sbunch@novell.com>
#
#
module Yast
  class InstCheckCertClient < Client
    def main

      Yast.import "Service"
      Yast.import "Package"
      Yast.import "NovellUtils"
      Yast.import "Report"
      Yast.import "FileUtils"

      textdomain "novell-base"


      @ret = :auto

      @args = Convert.to_map(WFM.Args(0))
      return @ret if Ops.get_boolean(@args, "going_back", false) == true

      if FileUtils.Exists("/var/lib/YaST2/OES11_AUTOUPGRADE")
        Builtins.y2milestone("inst_check_cert: autoyast upgrade mode")
        return @ret
      end

      @msg = ""
      @error = false
      @cnt = 1

      if Package.Installed("novell-web-base")
        # determine if certificate files exist
        if SCR.Read(path(".target.size"), "/etc/ssl/servercerts/servercert.pem") == -1
          @msg2 = Builtins.sformat(
            _(
              "%1) This machine does not have a certificate.\n" +
                "Validate that your root password is 5 characters or longer and that\n" +
                "you did not skip the certificate creation step."
            ),
            @cnt
          )
          @msg = @msg2
          @error = true
          @cnt = Ops.add(@cnt, 1)
        end
      end

      Report.Error(@msg) if @error == true


      @ret 

      # EOF
    end
  end
end

Yast::InstCheckCertClient.new.main
