# encoding: utf-8

# -------------------------------------------------------------------------------
# Copyright 2023 Open Text.
#
# The only warranties for products and services of Open Text and its
# affiliates and licensors (“Open Text”) are as may be set forth in the
# express warranty statements accompanying such products and services.
# Nothing herein should be construed as constituting an additional
# warranty. Open Text shall not be liable for technical or editorial
# errors or omissions contained herein. The information contained herein
# is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains
# confidential information and a valid license is required for possession,
# use or copying. If this work is provided to the U.S. Government,
# consistent with FAR 12.211 and 12.212, Commercial Computer Software,
# Computer Software Documentation, and Technical Data for Commercial Items
# are licensed to the U.S. Government under vendor's standard commercial
# license.
# -------------------------------------------------------------------------------
# Copyright (c) 2020 Micro Focus or one of its affiliates.
# File:    clients/inst_autosetup.ycp
# Package: Auto-installation
# Summary: Setup and prepare system for auto-installation
# Authors: Anas Nashif <nashif@suse.de>
#          Uwe Gansert <ug@suse.de>
#
# $Id: inst_autosetup.ycp 61521 2010-03-29 09:10:07Z ug $
module Yast
  class OesAutosetupUpgradeClient < Client
    include Yast::Logger

    def main
      Yast.import "Pkg"
      Yast.import "UI"
      Yast.import "NovellUtils"

      textdomain "autoinst"

      @ret = :auto

      Packages.Init(true)
      if true
        PackageCallbacks.SetConvertDBCallbacks

        Pkg.TargetInit(Installation.destdir, false)

      end

      packages_list=[["novell-cifs","novell-cifs"],
	             ["novell-cluster-services","novell-ncs"],
	             ["novell-oes-dhcp-conf","novell-dhcp"],
	             ["novell-bind","novell-dns"],
	             ["novell-xad-framework","novell-domain-services"],
                     ["novell-NDSimon","novell-edirectory"],
                     ["novell-pure-ftpd-config","novell-ftp"],
                     ["novell-imanager","novell-imanager"],
                     ["novell-iprint-server","novell-iprint"],
                     ["novell-lum","novell-lum"],
                     ["novell-ncp2nss","novell-ncp"],
                     ["novell-premigration-server","novell-premigration-server"],
                     ["novell-nrm","novell-nrm"],
                     ["novell-sms","novell-sms"],
                     ["novell-vigil","novell-nss"],
                     ["novell-iprint-mobile","iprint-advanced"],
                     ["novell-business-continuity-cluster","novell-bcc"],
                     ["novell-nurm","novell-nssad"]]

      Builtins.y2milestone("inst_oes_autosetup_upgrade: Selecting patterns")

      Builtins.foreach(packages_list) do |key,value|
        @package_properties = Pkg.ResolvableProperties(key, :package, "")
        Builtins.foreach(@package_properties) do |p|
          if Ops.get(p, "status") == :selected || Ops.get(p, "status") == :installed || Ops.get(p, "status") == :removed
            Pkg.ResolvableInstall(value, :pattern)
            Builtins.y2milestone("%1 pattern selected",value)
          end
        end
      end
    @ret
    end
  end
end

Yast::OesAutosetupUpgradeClient.new.main
