# encoding: utf-8

# File:	installation/general/inst_oes_congratulate.ycp
# Module:	Installation
# Summary:	Display congratulation
# Authors:	Arvin Schnell <arvin@suse.de> Modifications for OES by Mark Clark <mclark@novell.com>
#
# Display a congratulation message for the user. This file is a copy of the SLES inst_congratulate.ycp with OES additions.
#
# $Id: inst_congratulate.ycp 53256 2008-11-13 09:22:56Z ug $
module Yast
  class InstOesCongratulateClient < Client
    def main
      Yast.import "Pkg"
      Yast.import "UI"
      textdomain "installation-oes"

      Yast.import "Mode"
      Yast.import "Wizard"
      Yast.import "Popup"
      Yast.import "Label"
      Yast.import "ProductFeatures"
      Yast.import "GetInstArgs"
      Yast.import "Call"
      Yast.import "Package"
      Yast.import "ProductControl"
      Yast.import "Stage"
      Yast.import "AddOnProduct"
      Yast.import "Service"
      Yast.import "NovellPatterns"
      Yast.import "NovellUtils"
      Yast.import "FileUtils"
      if FileUtils.Exists("/var/lib/YaST2/OES11_AUTOUPGRADE")
        Builtins.y2milestone("inst_oes_congrulate: autoyast upgrade mode")
        @ret2 = :auto
        @cmd = "rm -rf /var/lib/YaST2/OES11_AUTOUPGRADE"
        Builtins.y2milestone("inst_oes_postconfig:executing %1", @cmd)
        @retmap = Convert.to_map(SCR.Execute(path(".target.bash_output"), @cmd))
        Builtins.y2milestone("Removed OES11_AUTOUPGRADE file")
        return deep_copy(@ret2)
      end

      @abortInstall = Builtins.tolower(
        NovellUtils.ReadOption("ABORT_OES_INSTALL")
      )
      if @abortInstall == "yes"
        Builtins.y2milestone(
          "inst_oes_congratulate:setting ABORT_OES_INSTALL is set to 'yes'."
        )
        @ret2 = :next
        return deep_copy(@ret2)
      end

      @argmap = GetInstArgs.argmap

      # show button "clone system"?
      @show_clone_checkbox = !(Stage.firstboot || Mode.live_installation)
      @clone_checkbox_active = ProductFeatures.GetBooleanFeature(
        "globals",
        "enable_clone"
      )
      @clone_enabled = false

      # #302495: Switching ZMD off
      # by default, the checkbox is not visible
      @zmd_service_name = "novell-zmd"
      @zmd_package_name = "zmd"
      @check_box_turnoff_zmd = Empty()
      @turnoff_zmd_help = ""

      # See FATE #302495
      # Param 'show_zmd_turnoff_checkbox' says whether the checkbox for stopping and
      # disabling ZMD should be shown
      @show_zmd_turnoff_checkbox = Ops.get_string(
        @argmap,
        "show_zmd_turnoff_checkbox",
        "no"
      ) == "yes"

      # Param 'zmd_turnoff_default_state' says whether the checkbox is selected or
      # not by default
      @turnoff_zmd_default_state = Ops.get_string(
        @argmap,
        "zmd_turnoff_default_state",
        "no"
      ) == "yes"

      @zmd_installed = Package.Installed(@zmd_package_name)
      # don't check for state a service that is not installed
      @zmd_enabled_or_running = @zmd_installed &&
        (Service.Enabled(@zmd_service_name) ||
          Service.Status(@zmd_service_name) == 0)
      Builtins.y2milestone(
        "ZMD Installed: %1, Enabled/Running: %2",
        @zmd_installed,
        @zmd_enabled_or_running
      )
      Builtins.y2milestone(
        "Show TurnOffZMD checkbox: %1, default state: %2",
        @show_zmd_turnoff_checkbox,
        @turnoff_zmd_default_state
      )

      # + 'show_zmd_turnoff_checkbox'
      # + ZMD package needs to be installed
      # + ZMD service needs to be enabled
      if @show_zmd_turnoff_checkbox && @zmd_installed && @zmd_enabled_or_running
        @check_box_turnoff_zmd = CheckBox(
          Id(:turnoff_zmd),
          # TRANSLATORS: check box, see #ZMD
          _("&Disable ZMD Service"),
          # control_file->software->zmd_turnoff_default_state
          # says whether the checkbox is selected or not by default
          @turnoff_zmd_default_state
        )

        # TRANSLATORS: help text, see #ZMD
        @turnoff_zmd_help = _(
          "<p>Select <b>Disable ZMD Service</b> to stop and disable\nthe ZMD service during the system start.</p>"
        )
      else
        Builtins.y2milestone("ZMD Turnoff check-box will be invisible")
      end

      # Cloning client is installed and there is something to be cloned
      if Package.Installed("autoyast2") &&
          Ops.greater_than(Builtins.size(ProductControl.clone_modules), 0)
        @clone_enabled = true
      end
      if Stage.stage == "normal"
        @clone_checkbox_active = false
      else
        @clone_checkbox_active = true
      end

      Builtins.y2milestone(
        "Cloning Checkbox visible: %1, enabled: %2, pre-selected: %3",
        @show_clone_checkbox,
        @clone_enabled,
        @clone_checkbox_active
      )

      @display = UI.GetDisplayInfo
      @space = Ops.get_boolean(@display, "TextMode", true) ? 1 : 3
      @vendor_url_tmp = ProductFeatures.GetStringFeature(
        "globals",
        "vendor_url"
      )

      # fallback
      @vendor_url = "http://www.novell.com/linux/"
      if ProductFeatures.GetStringFeature("globals", "ui_mode") == "simple"
        @vendor_url = "http://www.openSUSE.org"
      end
      Builtins.y2milestone(
        "UI mode: %1",
        ProductFeatures.GetStringFeature("globals", "ui_mode")
      )

      if @vendor_url_tmp != nil && @vendor_url_tmp != ""
        @vendor_url = @vendor_url_tmp
      end

      @check_box_do_clone = Empty()

      if @show_clone_checkbox
        @check_box_do_clone = CheckBox(
          Id(:do_clone),
          # Check box: start the clone process and store the AutoYaST
          # profile in /root/autoinst.xml
          _("&Clone This System for AutoYaST"),
          @clone_checkbox_active
        )
      end

      # caption for dialog "Congratulation Dialog"
      @caption = _("Installation Completed")

      @text = ProductControl::GetTranslatedText ("congratulate")

      @text = Builtins.dgettext("control-OES", @text)

      while Builtins.issubstring(@text, "&lt;")
        @text = Builtins.regexpsub(@text, "^(.*)&lt;(.*)$", "\\1<\\2")
      end
      while Builtins.issubstring(@text, "&gt;")
        @text = Builtins.regexpsub(@text, "^(.*)&gt;(.*)$", "\\1>\\2")
      end

      if @text == ""
        # congratulation text 1/4
        @text = Ops.add(
          Ops.add(
            _("<p><b>Congratulations!</b></p>") +
              # congratulation text 2/4
              _(
                "<p>The installation of &product; on your machine is complete.\nAfter clicking <b>Finish</b>, you can log in to the system.</p>\n"
              ),
            # congratulation text 3/4
            Builtins.sformat(_("<p>Visit us at %1.</p>"), @vendor_url)
          ),
          # congratulation text 4/4
          _("<p>Have a lot of fun!<br>Your SUSE Development Team</p>")
        )
      else
        @text = Builtins.sformat(@text, @vendor_url)
      end

      @contents = VBox(
        VSpacing(@space),
        HBox(
          HSpacing(Ops.multiply(2, @space)),
          VBox(
            RichText(Id(:text), @text),
            VSpacing(Ops.divide(@space, 2)),
            Left(@check_box_turnoff_zmd),
            Left(@check_box_do_clone)
          ),
          HSpacing(Ops.multiply(2, @space))
        ),
        VSpacing(@space),
        VSpacing(2)
      )

      @help_file = ""
      # help 1/4 for dialog "Congratulation Dialog"
      @help = _("<p>Your system is ready for use.</p>") +
        # help 2/4 for dialog "Congratulation Dialog"
        _(
          "<p><b>Finish</b> will close the YaST installation and continue\nto the login screen.</p>\n"
        ) +
        # help 3/4 for dialog "Congratulation Dialog"
        (DisplayKDEHelp() ?
          _(
            "<p>If you choose the default graphical desktop KDE, you can\n" +
              "adjust some KDE settings to your hardware. Also notice\n" +
              "our SUSE Welcome Dialog.</p>\n"
          ) :
          "") # Show this help only in case of KDE as the default windowmanager

      if @show_clone_checkbox
        @help = Ops.add(
          @help,
          _(
            "<p>Use <b>Clone</b> if you want to create an AutoYaST profile.\n" +
              "AutoYaST is a way to do a complete SUSE Linux installation without user interaction. AutoYaST\n" +
              "needs a profile to know what the installed system should look like. If this option is\n" +
              "selected, a profile of the current system is stored in <tt>/root/autoinst.xml</tt>.</p>"
          )
        )
      end
      @help = Ops.add(@help, @turnoff_zmd_help) if @show_zmd_turnoff_checkbox

      Wizard.SetContents(
        @caption,
        @contents,
        @help,
        GetInstArgs.enable_back,
        GetInstArgs.enable_next
      )
      Wizard.SetTitleIcon("yast-license")

      Wizard.SetNextButton(:next, Label.FinishButton)
      Wizard.RestoreAbortButton
      Wizard.SetFocusToNextButton
      if UI.WidgetExists(Id(:do_clone))
        UI.ChangeWidget(Id(:do_clone), :Enabled, @clone_enabled)
      end

      @ret = nil
      begin
        @ret = Wizard.UserInput

        if @ret == :abort
          break if Popup.ConfirmAbort(:incomplete)
        elsif @ret == :help
          Wizard.ShowHelp(@help)
        end
      end until @ret == :next || @ret == :back


      # If DSFW is installed, then pop up a message to remind the user to run provisioning.
      @xad_pattern = Convert.to_string(
        SCR.Read(path(".sysconfig.novell.xad.PATTERN_NAME"))
      )
      Builtins.y2milestone(
        "inst_oes_congratulate:xad_pattern: %1",
        @xad_pattern
      )
      @xad_packages_installed = NovellPatterns.IsPatternInstalled(@xad_pattern)
      Builtins.y2milestone(
        "inst_oes_congratulate:xad_packages_installed: %1",
        @xad_packages_installed
      )

      @migration_pattern_installed = NovellPatterns.IsPatternInstalled(
        "novell-premigration-server"
      )
      Builtins.y2milestone(
        "inst_oes_congratulate:migration_pattern_installed: %1",
        @migration_pattern_installed
      )

      @dsfwMessage = _(
        "Note : Domain Services for Windows(DSfW) configuration is not yet complete. Restart\nYaST and launch the DSfW Provisioning Wizard in YaST to complete the configuration."
      )
      if Mode.autoinst != true && @xad_packages_installed &&
          XadProvisioned() == false && !@migration_pattern_installed
        Popup.Message(@dsfwMessage)
      end

      # bugzilla #221190
      if @ret == :back
        Wizard.RestoreNextButton
      elsif @ret == :next
        # BNC #441452
        # Remove the congrats dialog
        @zmd = UI.WidgetExists(Id(:turnoff_zmd)) &&
          Convert.to_boolean(UI.QueryWidget(Id(:turnoff_zmd), :Value))
        @clone = UI.WidgetExists(Id(:do_clone)) &&
          Convert.to_boolean(UI.QueryWidget(Id(:do_clone), :Value))

        # Dialog busy message
        Wizard.SetContents(
          @caption,
          Label(_("Finishing the installation...")),
          @help,
          GetInstArgs.enable_back,
          GetInstArgs.enable_next
        )

        StopAndDisableZMD() if @zmd

        CallCloning() if @clone
      end

      # save all sources and finish target
      # bnc #398315
      Pkg.SourceSaveAll
      Pkg.TargetFinish


      deep_copy(@ret)
    end

    # return true or false on whether dsfw is provisioned or not.
    def XadProvisioned
      ret = false
      if Ops.greater_or_equal(
          SCR.Read(
            path(".target.size"),
            "/opt/novell/oes-install/xad_provisioned"
          ),
          0
        )
        ret = true
      end

      ret
    end

    # Function returns true when the default windowmanager is KDE
    # See bug 170880 for more information
    #
    # @return [Boolean] wm is is_kde
    def DisplayKDEHelp
      default_wm = Convert.to_string(
        SCR.Read(path(".sysconfig.windowmanager.\"DEFAULT_WM\""))
      )
      Builtins.y2debug("Default WM: %1", default_wm)

      return true if default_wm != nil && Builtins.tolower(default_wm) == "kde"
      false
    end

    def CallCloning
      # #187558
      # Load Add-On products configured in the fist stage
      AddOnProduct.ReadTmpExportFilename

      # #165860
      # Save sources now because cloning garbles the target
      # Cloning reinitializes sources when it needs them
      Pkg.SourceSaveAll
      Call.Function("clone_system", [])

      nil
    end

    def StopAndDisableZMD
      Builtins.y2milestone(
        "Stopping service: %1 -> %2",
        @zmd_service_name,
        Service.Stop(@zmd_service_name)
      )
      Builtins.y2milestone(
        "Disabling service: %1 -> %2",
        @zmd_service_name,
        Service.Disable(@zmd_service_name)
      )

      nil
    end
  end
end

Yast::InstOesCongratulateClient.new.main
