# encoding: utf-8

# -------------------------------------------------------------------------------
# Copyright 2023 Open Text.
#
# The only warranties for products and services of Open Text and its
# affiliates and licensors (“Open Text”) are as may be set forth in the
# express warranty statements accompanying such products and services.
# Nothing herein should be construed as constituting an additional
# warranty. Open Text shall not be liable for technical or editorial
# errors or omissions contained herein. The information contained herein
# is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains
# confidential information and a valid license is required for possession,
# use or copying. If this work is provided to the U.S. Government,
# consistent with FAR 12.211 and 12.212, Commercial Computer Software,
# Computer Software Documentation, and Technical Data for Commercial Items
# are licensed to the U.S. Government under vendor's standard commercial
# license.
# -------------------------------------------------------------------------------
# -------------------------------------------------------------------------------
# (C) Copyright 2019 - 2023 Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates
# and licensors (“Micro Focus”) are as may be set forth in the express warranty
# statements accompanying such products and services. Nothing herein should be
# construed as constituting an additional warranty. Micro Focus shall not be
# liable for technical or editorial errors or omissions contained herein. The
# information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential
# information and a valid license is required for possession, use or copying. If
# this work is provided to the U.S. Government, consistent with FAR 12.211 and
# 12.212, Commercial Computer Software, Computer Software Documentation, and
# Technical Data for Commercial Items  are licensed to the U.S. Government under
# vendor's standard commercial license.
# -------------------------------------------------------------------------------

# File:        clients/inst_oes_postconfig.ycp
# Package:     novell-common
# Summary:     Configure remaining items
# Authors:     Mark Clark <mclark@novell.com>
#
# $Id: inst_oes_postconfig.ycp,v 1.1.2.18 2005/05/04 22:10:11 mclark Exp $
#
# This code is also known as the grab bag of all things that someone
# else should have done but did not do.
module Yast
  class InstOesPostconfigClient < Client
    def main
      Yast.import "UI"
      Yast.import "Pkg"
      Yast.import "PackageCallbacks"
      Yast.import "Installation"

      textdomain "novell-base"

      Yast.import "Service"
      Yast.import "Package"
      Yast.import "ProductControl"
      Yast.import "Popup"
      Yast.import "Stage"
      Yast.import "NovellUtils"
      Yast.import "NovellPatterns"
      Yast.import "Mode"
      Yast.import "Edirectory"
      Yast.import "NovellLDAP"
      Yast.import "NovellLDAPUI"
      Yast.import "NovellAddonInstall"
      Yast.import "NovellLum"
      Yast.import "Ncpserver"
      Yast.import "Imanager"
      Yast.import "Iprint"
      Yast.import "Sms"
      Yast.import "Nss"
      Yast.import "NovellDns"
      Yast.import "NovellDhcp"
      Yast.import "Ncs"
      Yast.import "NovellCifs"
      Yast.import "NovellNssad"
      Yast.import "FileUtils"
      Yast.import "Label"
      Yast.import "Report"
      Yast.import "IP"
      Yast.import "Address"

      @ret = :auto

      Builtins.y2milestone("inst_oes_postconfig: Starting")

      @args = Convert.to_map(WFM.Args(0))
      return @ret if Ops.get_boolean(@args, "going_back", false) == true

      Yast::PackageCallbacks.InitPackageCallbacks
      # Initialize the target
      Yast::Pkg.TargetInitialize(Installation.destdir)
      # Load the installed packages into the pool.
      Yast::Pkg.TargetLoad
      # Load the repository configuration.
      Yast::Pkg.SourceRestore
      # Load the available packages in the repositories to the pool.
      Yast::Pkg.SourceLoad

      @oes_version = NovellUtils.OesVersion

      if FileUtils.Exists("/var/lib/YaST2/OES11_AUTOUPGRADE")
        Builtins.y2milestone("inst_oes_preconfig: AutoYaST upgrade mode")
        @ret2 = nil
        @answerSettings = {}

        @xadMode = false

        def DeleteAnswerFile
          ret = true

          cmd = "rm -f /opt/novell/oes-install/answer"
          Builtins.y2milestone(
            "AutoYaST Upgrade.DeleteAnswerFile:executing: %1",
            cmd
          )

          ccode = Convert.to_integer(SCR.Execute(path(".target.bash"), cmd))
          if ccode != 0
            ret = false
            Builtins.y2milestone(
              "AutoYaST Upgrade.DeleteAnswerFile:error deleting /etc/reconfig_system."
            )
          end

          Builtins.y2milestone(
            "AutoYaST Upgrade.DeleteAnswerFile:deleting password file."
          )

          ret
        end

        def EdirDependenciesOK
          # verify the eDirectory pattern is installed...
          if Edirectory.xadTestMode == true ||
              Edirectory.upgradeTestMode == true
            return true
          end

          if !NovellPatterns.EnsurePatternInstalled("novell-edirectory")
            return false
          end

          true
        end

        # Assign a valid password character to a random number
        def LoadDeTranslationMap
          ret = {}

          Ops.set(ret, 46, "-") # 1
          Ops.set(ret, 122, "0") # 2
          Ops.set(ret, 18, "1") # 3
          Ops.set(ret, 6, "2") # 4
          Ops.set(ret, 20, "3") # 5
          Ops.set(ret, 32, "4") # 6
          Ops.set(ret, 10, "5") # 7
          Ops.set(ret, 60, "6") # 8
          Ops.set(ret, 70, "7") # 9
          Ops.set(ret, 5, "8") # 10
          Ops.set(ret, 66, "9") # 11
          Ops.set(ret, 209, "a") # 12
          Ops.set(ret, 11, "b") # 13
          Ops.set(ret, 168, "c") # 14
          Ops.set(ret, 102, "d") # 15
          Ops.set(ret, 180, "e") # 16
          Ops.set(ret, 2, "f") # 17
          Ops.set(ret, 152, "g") # 18
          Ops.set(ret, 80, "h") # 19
          Ops.set(ret, 162, "i") # 20
          Ops.set(ret, 187, "j") # 21
          Ops.set(ret, 15, "k") # 22
          Ops.set(ret, 151, "l") # 23
          Ops.set(ret, 21, "m") # 24
          Ops.set(ret, 87, "n") # 25
          Ops.set(ret, 30, "o") # 26
          Ops.set(ret, 1, "p") # 27
          Ops.set(ret, 65, "q") # 28
          Ops.set(ret, 163, "r") # 29
          Ops.set(ret, 81, "s") # 30
          Ops.set(ret, 67, "t") # 31
          Ops.set(ret, 71, "u") # 32
          Ops.set(ret, 17, "v") # 33
          Ops.set(ret, 167, "w") # 34
          Ops.set(ret, 25, "x") # 35
          Ops.set(ret, 185, "y") # 36
          Ops.set(ret, 104, "z") # 37
          Ops.set(ret, 24, "A") # 38
          Ops.set(ret, 61, "B") # 39
          Ops.set(ret, 88, "C") # 40
          Ops.set(ret, 186, "D") # 41
          Ops.set(ret, 33, "E") # 42
          Ops.set(ret, 14, "F") # 43
          Ops.set(ret, 166, "G") # 44
          Ops.set(ret, 72, "H") # 45
          Ops.set(ret, 85, "I") # 46
          Ops.set(ret, 62, "J") # 47
          Ops.set(ret, 36, "K") # 48
          Ops.set(ret, 106, "L") # 49
          Ops.set(ret, 26, "M") # 50
          Ops.set(ret, 181, "N") # 51
          Ops.set(ret, 206, "O") # 52
          Ops.set(ret, 207, "P") # 53
          Ops.set(ret, 3, "Q") # 54
          Ops.set(ret, 74, "R") # 55
          Ops.set(ret, 22, "S") # 56
          Ops.set(ret, 164, "T") # 57
          Ops.set(ret, 86, "U") # 58
          Ops.set(ret, 68, "V") # 59
          Ops.set(ret, 183, "W") # 60
          Ops.set(ret, 103, "X") # 61
          Ops.set(ret, 35, "Y") # 62
          Ops.set(ret, 150, "Z") # 63
          Ops.set(ret, 82, "!") # 64
          Ops.set(ret, 160, "@") # 65
          Ops.set(ret, 31, "#") # 66
          Ops.set(ret, 16, "$") # 67
          Ops.set(ret, 182, "%") # 68
          Ops.set(ret, 63, "^") # 69
          Ops.set(ret, 84, "&") # 70
          Ops.set(ret, 165, "*") # 71
          Ops.set(ret, 23, "~") # 72
          Ops.set(ret, 184, "(") # 73
          Ops.set(ret, 101, ")") # 74
          Ops.set(ret, 161, ",") # 75
          Ops.set(ret, 89, ";") # 76
          Ops.set(ret, 205, ":") # 77
          Ops.set(ret, 64, ".") # 78
          Ops.set(ret, 13, "_") # 79
          Ops.set(ret, 34, "+") # 80
          Ops.set(ret, 83, "/") # 81
          Ops.set(ret, 69, "|") # 82
          Ops.set(ret, 73, "?") # 83
          Ops.set(ret, 4, "{") # 84
          Ops.set(ret, 208, "}") # 85
          Ops.set(ret, 100, "=") # 86
          Ops.set(ret, 105, "[") # 87
          Ops.set(ret, 12, "]") # 88

          deep_copy(ret)
        end

        # How much to bit shift the value based on string position
        def LoadBitshiftList
          ret = []

          Ops.set(ret, 0, 3) # 1
          Ops.set(ret, 1, 1) # 2
          Ops.set(ret, 2, 2) # 3
          Ops.set(ret, 3, 4) # 4
          Ops.set(ret, 4, 3) # 5
          Ops.set(ret, 5, 1) # 6
          Ops.set(ret, 6, 4) # 7
          Ops.set(ret, 7, 2) # 8
          Ops.set(ret, 8, 4) # 9
          Ops.set(ret, 9, 2) # 10
          Ops.set(ret, 10, 1) # 11
          Ops.set(ret, 11, 3) # 12
          Ops.set(ret, 12, 1) # 13
          Ops.set(ret, 13, 2) # 14
          Ops.set(ret, 14, 1) # 15
          Ops.set(ret, 15, 4) # 16
          Ops.set(ret, 16, 3) # 17
          Ops.set(ret, 17, 2) # 18
          Ops.set(ret, 18, 1) # 19
          Ops.set(ret, 19, 3) # 20
          Ops.set(ret, 20, 4) # 21
          Ops.set(ret, 21, 2) # 22
          Ops.set(ret, 22, 1) # 23
          Ops.set(ret, 23, 3) # 24
          Ops.set(ret, 24, 1) # 25
          Ops.set(ret, 25, 4) # 26
          Ops.set(ret, 26, 1) # 27
          Ops.set(ret, 27, 4) # 28
          Ops.set(ret, 28, 2) # 29
          Ops.set(ret, 29, 1) # 30
          Ops.set(ret, 30, 3) # 31
          Ops.set(ret, 31, 2) # 32
          Ops.set(ret, 32, 4) # 33
          Ops.set(ret, 33, 2) # 34
          Ops.set(ret, 34, 3) # 35
          Ops.set(ret, 35, 1) # 36
          Ops.set(ret, 36, 2) # 37
          Ops.set(ret, 37, 4) # 38
          Ops.set(ret, 38, 3) # 39
          Ops.set(ret, 39, 1) # 40
          Ops.set(ret, 40, 2) # 41
          Ops.set(ret, 41, 4) # 42
          Ops.set(ret, 42, 3) # 43
          Ops.set(ret, 43, 4) # 44
          Ops.set(ret, 44, 2) # 45
          Ops.set(ret, 45, 1) # 46
          Ops.set(ret, 46, 3) # 47
          Ops.set(ret, 47, 2) # 48
          Ops.set(ret, 48, 1) # 49
          Ops.set(ret, 49, 4) # 50
          Ops.set(ret, 50, 3) # 51
          Ops.set(ret, 51, 4) # 52
          Ops.set(ret, 52, 3) # 53
          Ops.set(ret, 53, 3) # 54
          Ops.set(ret, 54, 1) # 55
          Ops.set(ret, 55, 2) # 56
          Ops.set(ret, 56, 4) # 57
          Ops.set(ret, 57, 3) # 58
          Ops.set(ret, 58, 1) # 59
          Ops.set(ret, 59, 2) # 60
          Ops.set(ret, 60, 1) # 61
          Ops.set(ret, 61, 4) # 62
          Ops.set(ret, 62, 3) # 63
          Ops.set(ret, 63, 4) # 64
          Ops.set(ret, 64, 1) # 65
          Ops.set(ret, 65, 2) # 66
          Ops.set(ret, 66, 4) # 67
          Ops.set(ret, 67, 3) # 68
          Ops.set(ret, 68, 1) # 69
          Ops.set(ret, 69, 4) # 70
          Ops.set(ret, 70, 2) # 71
          Ops.set(ret, 71, 3) # 72
          Ops.set(ret, 72, 1) # 73
          Ops.set(ret, 73, 4) # 74
          Ops.set(ret, 74, 2) # 75
          Ops.set(ret, 75, 1) # 76
          Ops.set(ret, 76, 3) # 77
          Ops.set(ret, 77, 2) # 78
          Ops.set(ret, 78, 4) # 79
          Ops.set(ret, 79, 3) # 80
          Ops.set(ret, 80, 2) # 81
          Ops.set(ret, 81, 1) # 82
          Ops.set(ret, 82, 3) # 83
          Ops.set(ret, 83, 4) # 84
          Ops.set(ret, 84, 3) # 85
          Ops.set(ret, 85, 1) # 86
          Ops.set(ret, 86, 3) # 87
          Ops.set(ret, 87, 4) # 88

          deep_copy(ret)
        end

        # UnObfuscates a string
        def UnObfuscate(str)
          # Initialize obfuscation tables
          tMap = LoadDeTranslationMap()
          bitshift = LoadBitshiftList()

          split = Builtins.splitstring(str, ",")
          sizePword = Builtins.size(split)
          character = ""
          cnt = 0
          val = 0
          clearstring = ""

          Builtins.foreach(split) do |intVal|
            val = Builtins.tointeger(intVal)
            val = Ops.shift_right(val, Ops.get(bitshift, cnt, 0))
            val = Ops.subtract(val, sizePword)
            character = Ops.get_string(tMap, val, "")
            clearstring = Ops.add(clearstring, character)
            cnt = Ops.add(cnt, 1)
          end

          clearstring
        end

        # This file may contain 1 or 2 passwords. The first is the LDAP admin and the second (optional) is the DSFW Domain Admin.
        def ReadAnswerFile
          ret = {}
          data_file = "/opt/novell/oes-install/answer"
          stmp = []

          # First see if the file exists.
          if SCR.Read(path(".target.size"), data_file) == -1
            return nil # nil indicates empty or no file
          end

          # Read the file into memory
          answerData = Convert.to_string(
            SCR.Read(path(".target.string"), data_file)
          )
          if answerData != ""
            answerData = NovellUtils.StripLeading(answerData, "\n")

            # break up into individual lines
            stmp = Builtins.splitstring(answerData, "\n")
          else
            stmp = nil
          end # empty or no file;


          if stmp != nil
            Ops.set(
              ret,
              "CONFIG_LDAP_ADMIN_PASSWORD",
              UnObfuscate(Ops.get(stmp, 0, ""))
            )
            if Ops.get(stmp, 2, "") != nil && Ops.get(stmp, 2, "") != ""
              Ops.set(
                ret,
                "CONFIG_DSFW_ADMIN_PASSWORD",
                UnObfuscate(Ops.get(stmp, 2, ""))
              )
            end
          end

          deep_copy(ret)
        end


        def PromptForEdirAdminPassword
          useXad = false

          if @xadMode
            xad_tree_admin_context = Ops.get_string(
              NovellLDAP.Settings,
              "XAD_TREE_ADMIN_CONTEXT",
              ""
            )
            if xad_tree_admin_context != nil && xad_tree_admin_context != ""
              useXad = true
            end # if this value is set at all, then it is the eDir admin (in spite of xad name)
          end
          Builtins.y2milestone(
            "AutoYaST Upgrade:PromptIfNeededForEdirAdminPassword:use xad:%1",
            useXad
          )

          if useXad
            NovellLDAPUI.XadTreeAdminPasswordDialog
          else
            NovellLDAPUI.AdminPasswordDialog
          end
          Builtins.y2milestone(
            "AutoYaST Upgrade:PromptForEdirAdminPassword:useXad %1",
            useXad
          )
          useXad
        end

        def PromptForDomainAdminPassword
          domainPassword = NovellLDAPUI.XadDomainAdminPasswordDialog

          domainPassword
        end


        # If dsfw is involved, password locations may be different
        def GetSetDsfwTreeAdminAndDomainAdminPasswords
          retVal = true

          adminPassword = Ops.get_string(
            NovellLDAP.Settings,
            "CONFIG_LDAP_ADMIN_PASSWORD",
            ""
          )
          domainAdminPassword = Ops.get_string(
            NovellLDAP.Settings,
            "CONFIG_DSFW_ADMIN_PASSWORD",
            ""
          )

          treeType = Ops.get_string(
            Edirectory.EdirSettings,
            "CONFIG_EDIR_TREE_TYPE",
            ""
          )
          domainType = Ops.get_string(
            Edirectory.EdirSettings,
            "XAD_DOMAIN_TYPE",
            ""
          )
          adminName = Ops.get_string(
            NovellLDAP.Settings,
            "CONFIG_LDAP_ADMIN_CONTEXT",
            ""
          )

          if @answerSettings != nil && @answerSettings != {}
            if Ops.get_string(@answerSettings, "CONFIG_DSFW_ADMIN_PASSWORD", "") != nil &&
                Ops.get_string(
                  @answerSettings,
                  "CONFIG_DSFW_ADMIN_PASSWORD",
                  ""
                ) != ""
              domainAdminPassword = Ops.get_string(
                @answerSettings,
                "CONFIG_DSFW_ADMIN_PASSWORD",
                ""
              )
            end
            if treeType == "existing"
              Ops.set(
                NovellLDAP.Settings,
                "XAD_TREE_ADMIN_PASSWORD",
                Ops.get_string(
                  @answerSettings,
                  "CONFIG_LDAP_ADMIN_PASSWORD",
                  ""
                )
              )
            end
          end

          # We only need the Domain Admin password if we were joining an existing tree
          if treeType == "existing"
            adminName = Ops.get_string(
              NovellLDAP.Settings,
              "XAD_TREE_ADMIN_CONTEXT",
              ""
            )
            adminPassword = Ops.get_string(
              NovellLDAP.Settings,
              "XAD_TREE_ADMIN_PASSWORD",
              ""
            )
            if domainAdminPassword == nil || domainAdminPassword == ""
              domainAdminPassword = PromptForDomainAdminPassword()
            end
            Ops.set(
              Edirectory.EdirSettings,
              "XAD_LDAP_ADMIN_CONTEXT",
              adminName
            )
            Ops.set(
              NovellLDAP.Settings,
              "XAD_TREE_ADMIN_PASSWORD",
              adminPassword
            )
            Ops.set(
              Edirectory.EdirSettings,
              "XAD_LDAP_ADMIN_PASSWORD",
              adminPassword
            )

            if Ops.get_string(Edirectory.EdirSettings, "XAD_DOMAIN_TYPE", "") == "forest"
              Ops.set(
                Edirectory.EdirSettings,
                "XAD_ADMIN_PASSWORD",
                domainAdminPassword
              )
            else
              Ops.set(
                NovellLDAP.Settings,
                "CONFIG_LDAP_ADMIN_PASSWORD",
                domainAdminPassword
              )
            end
          else
            Ops.set(NovellLDAP.Settings, "CONFIG_LDAP_ADMIN_CONTEXT", adminName)
            Ops.set(
              NovellLDAP.Settings,
              "CONFIG_LDAP_ADMIN_PASSWORD",
              adminPassword
            )
            Ops.set(
              Edirectory.EdirSettings,
              "XAD_LDAP_ADMIN_CONTEXT",
              adminName
            )
          end

          retVal
        end

        #  Main client driver code starts here
        # -----------------------------------------------------------------------------------------------------------------
        def IsServiceConfigured(service)
          oldSettings = {}

          oldSettings = NovellUtils.GetOldSysconfigMap(service)

          if Builtins.tolower(
              Ops.get_string(oldSettings, "SERVICE_CONFIGURED", "")
            ) == "yes"
            Builtins.y2milestone("%1 was configured on %2", service, NovellUtils.UpgradeIsFrom)
            return true
          end

          Builtins.y2milestone("%1 was not configured", service)
          return false
        end

        def MissingReplicaIP
          replicaAddress = Ops.get_string(
            Edirectory.EdirSettings,
            "CONFIG_EDIR_REPLICA_SERVER",
            ""
          )
          tree_type = Ops.get_string(
            Edirectory.EdirSettings,
            "CONFIG_EDIR_TREE_TYPE",
            ""
          )
          missingReplicaIP = false
          if tree_type == "existing"
            if replicaAddress == nil || replicaAddress == ""
              missingReplicaIP = true
              Builtins.y2milestone(
                "Edirectory:UpgradeServerDialog:CONFIG_EDIR_REPLICA_SERVER value was missing. Asking for it in the upgrade dialog."
              )
            elsif !Address.Check4(replicaAddress)
              missingReplicaIP = true
              Builtins.y2milestone("Edirectory:Upgrade : %1", Address.Valid4)
            elsif !NovellUtils.Ping_IP(replicaAddress)
              missingReplicaIP = true
              Builtins.y2milestone(
                "Edirectory::upgrade : The replica server IP address (%1) is not reachable.",
                replicaAddress
              )
            else
              local_addresses = Convert.convert(
                NovellUtils.GetAllStaticIPAddrs,
                :from => "list",
                :to   => "list <string>"
              )
              if Builtins.contains(local_addresses, replicaAddress)
                missingReplicaIP = true
                Builtins.y2milestone(
                  "The replica server IP address / hostname must be a remote (non-local) address."
                )
              end
            end
          else
            Builtins.y2milestone(
              "Edirectory:UpgradeServerDialog:tree type was 'new'."
            )
          end

          contents = HVSquash(
            VBox(
              missingReplicaIP ?
                VBox(
                  TextEntry(
                    Id(:replicaIP),
                    _(
                      "Please enter the &IP address / hostname of the eDirectory server that\n" +
                        "holds a writable replica of the context for the server you are upgrading.\n" +
                        "The upgrade fails on entering an incorrect IP / hostname."
                    ),
                    replicaAddress
                  )
                ) :
                Empty(),
              HBox(Left(PushButton(Id(:ok), Opt(:default), Label.OKButton)))
            )
          )
          UI.OpenDialog(Opt(:decorated), contents)
          UI.SetFocus(Id(:replicaIP))

          ret = nil

          while true
            ret = UI.UserInput
            if ret == :ok
              replicaAddress = Convert.to_string(
                UI.QueryWidget(Id(:replicaIP), :Value)
              )
              if replicaAddress == nil || replicaAddress == ""
                Report.Error(
                  _("The Replica server IP address / hostname cannot be empty.")
                )
                next
              end
              if missingReplicaIP
                if !Address.Check4(replicaAddress)
                  if replicaAddress != ""
                    Popup.Error(Address.Valid4)
                  else
                    Popup.Error(
                      _(
                        "Enter the IP address / hostname of an existing server with a replica in the tree."
                      )
                    )
                  end
                  next
                else
                  local_addresses = Convert.convert(
                    NovellUtils.GetAllStaticIPAddrs,
                    :from => "list",
                    :to   => "list <string>"
                  )
                  if Builtins.contains(local_addresses, replicaAddress)
                    Report.Error(
                      _(
                        "The replica server address must be a remote (non-local) address."
                      )
                    )
                    replicaAddress = ""
                    UI.ChangeWidget(Id(:replicaIP), :Value, replicaAddress)
                    UI.SetFocus(Id(:replicaIP))
                    next
                  end
                end
                if !NovellUtils.Ping_IP(replicaAddress)
                  erMsg = Builtins.sformat(
                    _(
                      "The specified IP address / hostname (%1) is either not reachable or unresolvable.\nDo you want to continue anyway?"
                    ),
                    replicaAddress
                  )

                  next if Popup.ContinueCancel(erMsg) == false
                end
                Builtins.y2milestone(
                  "Edirectory:UpgradeServerDialog:Value from upgrade dialog is %1.",
                  replicaAddress
                )
                Ops.set(
                  Edirectory.EdirSettings,
                  "CONFIG_EDIR_REPLICA_SERVER",
                  replicaAddress
                )
                break
              end
            end
          end
          UI.CloseDialog

          nil
        end
        SCR.Write(path(".sysconfig.novell-common.ABORT_OES_INSTALL"), "no")

        # First do an LDAP read to get the admin, etc.
        NovellLDAP.Read

        if Edirectory.xad_packages_installed || Edirectory.xadTestMode == true
          @xadMode = true
        end

        # Now, see if the answer file exists and if it has an obfuscated password..
        @answerSettings = ReadAnswerFile()
        if @answerSettings == nil || @answerSettings == {}
          Builtins.y2milestone(
            "AutoYaST Upgrade:answer file not found. Prompting for eDir password."
          )
          PromptForEdirAdminPassword()
        else
          # Now, set the LDAP password that we got from the answer file.
          Builtins.y2milestone("AutoYaST Upgrade:answer file found.")
          Ops.set(
            NovellLDAP.Settings,
            "CONFIG_LDAP_ADMIN_PASSWORD",
            Ops.get_string(@answerSettings, "CONFIG_LDAP_ADMIN_PASSWORD", "")
          )
        end
        # y2milestone("AutoYaST Upgrade:LDAP admin password = %1", NovellLDAP::Settings["CONFIG_LDAP_ADMIN_PASSWORD"]:"");
        # y2milestone("AutoYaST Upgrade:DSFW domain admin password (if dsfw is installed) = %1", answerSettings["CONFIG_DSFW_ADMIN_PASSWORD"]:"");

        #**************
        # eDirectory *
        #**************
        if NovellPatterns.IsPatternInstalled("novell-edirectory")
          @statusMsg = Builtins.sformat(
            _(
            "Upgrading eDirectory (AutoYaST Upgrade  %1)"
            ),
            @oes_version
          )
          if @xadMode
            Builtins.y2milestone(
              "AutoYaST Upgrade:doing eDirectory/DSFW upgrade."
            )
            @statusMsg = Builtins.sformat(
              _(
              "Upgrading eDirectory/DSFW (AutoYaST Upgrade  %1)"
              ),
              @oes_version
            )
          else
            Builtins.y2milestone("AutoYaST Upgrade:doing eDirectory upgrade.")
          end

          Popup.ShowFeedback(@statusMsg, _("This may take a while.........."))

          # Do an eDirectory Read to set up proper upgrade variables.
          Edirectory.channelUpgradeMode = true
          @ret2 = Edirectory.Read # This will also cause an LDAP read.
          Builtins.y2milestone(
            "AutoYaST Upgrade:ret from Edirectory::Read = %1",
            @ret2
          )
          @replicaAddress = Ops.get_string(
            Edirectory.EdirSettings,
            "CONFIG_EDIR_REPLICA_SERVER",
            ""
          )
          @tree_type = Ops.get_string(
            Edirectory.EdirSettings,
            "CONFIG_EDIR_TREE_TYPE",
            ""
          )
          if @tree_type == "existing"
            if @replicaAddress == nil || @replicaAddress == ""
              MissingReplicaIP()
            elsif !Address.Check4(@replicaAddress)
              Popup.Error(Address.Valid4)
              MissingReplicaIP()
            elsif !NovellUtils.Ping_IP(@replicaAddress)
              @erMsg = Builtins.sformat(
                _(
                  "The replica server IP address (%1) is not reachable.\nEnter the valid replica server IP address to proceed"
                ),
                @replicaAddress
              )
              Popup.Error(@erMsg)
              MissingReplicaIP()
            else
              @local_addresses = Convert.convert(
                NovellUtils.GetAllStaticIPAddrs,
                :from => "list",
                :to   => "list <string>"
              )
              if Builtins.contains(@local_addresses, @replicaAddress)
                Popup.Error(
                  _(
                    "The replica server IP address must be a remote (non-local) address."
                  )
                )
                MissingReplicaIP()
              end
            end
          end

          GetSetDsfwTreeAdminAndDomainAdminPasswords() if @xadMode

          # Do the Edirectory::Write() to do the Edirectory upgrade.
          Ops.set(
            Edirectory.EdirSettings,
            "CONFIG_EDIR_OLD_NDSCONF_FILE",
            "/etc/opt/novell/eDirectory/conf/nds.conf"
          )
          @ret2 = Edirectory.Write

          while @ret2 == :again_up
            SCR.Write(path(".sysconfig.novell-common.ABORT_OES_INSTALL"), "no")

            PromptForEdirAdminPassword()
            if Edirectory.xad_packages_installed ||
                Edirectory.xadTestMode == true
              @xadMode = true
            end

            @statusMsg2 = Builtins.sformat(
              _(
              "Upgrading eDirectory (AutoYaST Upgrade  %1)"
              ),
              @oes_version
            )
            if @xadMode
              Builtins.y2milestone(
                "AutoYaST Upgrade:doing eDirectory/DSFW upgrade."
              )
              @statusMsg2 = Builtins.sformat(
                _(
                "Upgrading eDirectory/DSFW (AutoYaST Upgrade  %1)"
                ),
                @oes_version
              )
            else
              Builtins.y2milestone("AutoYaST Upgrade:doing eDirectory upgrade.")
            end

            Popup.ShowFeedback(
              @statusMsg2,
              _("This may take a while..........")
            )

            # Now, see if the answer file exists and if it has an obfuscated password..
            Builtins.y2milestone(
              "AutoYaST Upgrade:answer file not found. Prompting for eDir password."
            )
            @ret2 = Edirectory.Write
            Popup.ClearFeedback
          end

          if Edirectory.ndsconfigWorked == false
            Popup.Message(
              _("Aborting AutoYaST Upgrade due to ndsconfig error.")
            )
            Builtins.y2milestone(
              "AutoYaST Upgrade:aborting due to ndsconfig failure."
            )
            Edirectory.ndsconfigWorked = true
            return :next
          end
          Builtins.y2milestone(
            "AutoYaST Upgrade:ret from Edirectory::Write() = %1",
            @ret2
          )

          Popup.ClearFeedback
        else
          Builtins.y2milestone("AutoYaST Upgrade:no eDir to upgrade.")
        end

        #*******
        # LUM *
        #*******
        if NovellPatterns.IsPatternInstalled("novell-lum")
          if IsServiceConfigured("novell-lum")
            Popup.ShowFeedback(
              Builtins.sformat(
                _(
                "Configuring Linux User Management (AutoYaST Upgrade for %1)"
                ),
                @oes_version
              ),
              _("This may take a while..........")
            )

            # Read the LUM settings.
            @ret2 = NovellLum.Read
            Builtins.y2milestone(
              "AutoYaST Upgrade:ret from NovellLum::Read() = %1",
              @ret2
            )

            # Do the NovellLum::Write() to perform upgrade
            @ret2 = NovellLum.Write
            Builtins.y2milestone(
              "AutoYaST Upgrade:ret from NovellLum::Write() = %1",
              @ret2
            )

            Popup.ClearFeedback
          end
        else
          Builtins.y2milestone("AutoYaST Upgrade:no LUM to upgrade.")
        end

        #**************
        # NCP Server *
        #**************
        if NovellPatterns.IsPatternInstalled("novell-ncp")
          if IsServiceConfigured("ncpserver")
            Builtins.y2milestone("AutoYaST Upgrade:doing NcpServer upgrade.")

            Popup.ShowFeedback(
              Builtins.sformat(_("Configuring NCP Server (AutoYaST Upgrade for %1)"),@oes_version),
              _("This may take a while..........")
            )

            # Read the NCP Server settings.
            @ret2 = Ncpserver.Read
            Builtins.y2milestone(
              "AutoYaST Upgrade:ret from Ncpserver::Read() = %1",
              @ret2
            )

            # Do the Ncpserver::Write() to perform upgrade
            @ret2 = Ncpserver.Write
            Builtins.y2milestone(
              "AutoYaST Upgrade:ret from Ncpserver::Write() = %1",
              @ret2
            )

            Popup.ClearFeedback
          end
        else
          Builtins.y2milestone("AutoYaST Upgrade:no NCP server to upgrade.")
        end

        #************
        # iManager *
        #************
        if NovellPatterns.IsPatternInstalled("novell-imanager")
          if IsServiceConfigured("imanager")
            Builtins.y2milestone("AutoYaST Upgrade:doing iManager upgrade.")

            Popup.ShowFeedback(
              Builtins.sformat(_("Configuring iManager (AutoYaST Upgrade for %1)"),@oes_version),
              _("This may take a while..........")
            )

            # Read the iManager settings.
            @ret2 = Imanager.Read
            Builtins.y2milestone(
              "AutoYaST Upgrade:ret from Imanager::Read() = %1",
              @ret2
            )

            # Do the Imanager::Write() to perform upgrade
            @ret2 = Imanager.Write
            Builtins.y2milestone(
              "AutoYaST Upgrade:ret from Imanager::Write() = %1",
              @ret2
            )

            Popup.ClearFeedback
          end
        else
          Builtins.y2milestone("AutoYaST Upgrade:no iManager to upgrade.")
        end

        #**********
        # iPrint *
        #**********
        if NovellPatterns.IsPatternInstalled("novell-iprint")
          if IsServiceConfigured("iprint")
            Builtins.y2milestone("AutoYaST Upgrade:doing iPrint upgrade.")

            Popup.ShowFeedback(
              Builtins.sformat(_("Configuring iPrint (AutoYaST Upgrade for %1)"),@oes_version),
              _("This may take a while..........")
            )

            # Read the iPrint settings.
            @ret2 = Iprint.Read
            Builtins.y2milestone(
              "AutoYaST Upgrade:ret from Iprint::Read() = %1",
              @ret2
            )

            # Do the Iprint::Write() to perform upgrade
            @ret2 = Iprint.Write
            Builtins.y2milestone(
              "AutoYaST Upgrade:ret from Iprint::Write() = %1",
              @ret2
            )

            Popup.ClearFeedback
          end
        else
          Builtins.y2milestone("AutoYaST Upgrade:no iPrint to upgrade.")
        end

        #*******
        # SMS *
        #*******
        if NovellPatterns.IsPatternInstalled("novell-sms")
          if IsServiceConfigured("sms")
            Builtins.y2milestone("AutoYaST Upgrade:doing SMS upgrade.")

            Popup.ShowFeedback(
              Builtins.sformat(_("Configuring SMS (AutoYaST Upgrade for %1)"),@oes_version),
              _("This may take a while..........")
            )

            # Read the SMS settings.
            @ret2 = Sms.Read
            Builtins.y2milestone(
              "AutoYaST Upgrade:ret from Sms::Read() = %1",
              @ret2
            )

            # Do the Sms::Write() to perform upgrade
            @ret2 = Sms.Write
            Builtins.y2milestone(
              "AutoYaST Upgrade:ret from Sms::Write() = %1",
              @ret2
            )

            Popup.ClearFeedback
          end
        else
          Builtins.y2milestone("AutoYaST Upgrade:no SMS to upgrade.")
        end

        #*******
        # NSS *
        #*******
        if NovellPatterns.IsPatternInstalled("novell-nss")
          if IsServiceConfigured("nss")
            Builtins.y2milestone("AutoYaST Upgrade:doing NSS upgrade.")

            Popup.ShowFeedback(
              Builtins.sformat(_("Configuring NSS (AutoYaST Upgrade for %1)"),@oes_version),
              _("This may take a while..........")
            )

            # Read the settings.
            @ret2 = Nss.Read
            Builtins.y2milestone(
              "AutoYaST Upgrade:ret from Nss::Read() = %1",
              @ret2
            )

            # Do the Write() to perform upgrade
            @ret2 = Nss.Write
            Builtins.y2milestone(
              "AutoYaST Upgrade:ret from Nss::Write() = %1",
              @ret2
            )

            Popup.ClearFeedback
          end
        else
          Builtins.y2milestone("AutoYaST Upgrade:no NSS to upgrade.")
        end

        #*************
        # NovellDns *
        #*************
        if NovellPatterns.IsPatternInstalled("novell-dns")
          if IsServiceConfigured("novell-dns")
            Builtins.y2milestone("AutoYaST Upgrade:doing Novell DNS upgrade.")

            Popup.ShowFeedback(
              Builtins.sformat(_("Configuring OES DNS (AutoYaST Upgrade for %1)"),@oes_version),
              _("This may take a while..........")
            )

            # Read the settings.
            @ret2 = NovellDns.Read
            Builtins.y2milestone(
              "AutoYaST Upgrade:ret from NovellDns::Read() = %1",
              @ret2
            )

            # Do the Write() to perform upgrade
            @ret2 = NovellDns.Write
            Builtins.y2milestone(
              "AutoYaST Upgrade:ret from NovellDns::Write() = %1",
              @ret2
            )

            Popup.ClearFeedback
          end
        else
          Builtins.y2milestone("AutoYaST Upgrade:no Novell DNS to upgrade.")
        end


        #**************
        # NovellDhcp *
        #**************
        if NovellPatterns.IsPatternInstalled("novell-dhcp")
          if IsServiceConfigured("novell-dhcp")
            Builtins.y2milestone("AutoYaST Upgrade:doing Novell DHCP upgrade.")

            Popup.ShowFeedback(
              Builtins.sformat(_("Configuring OES DHCP (AutoYaST Upgrade for %1)"),@oes_version),
              _("This may take a while..........")
            )

            # Read the settings.
            @ret2 = NovellDhcp.Read
            Builtins.y2milestone(
              "AutoYaST Upgrade:ret from NovellDhcp::Read() = %1",
              @ret2
            )

            # Do the Write() to perform upgrade
            @ret2 = NovellDhcp.Write
            Builtins.y2milestone(
              "AutoYaST Upgrade:ret from NovellDhcp::Write() = %1",
              @ret2
            )

            Popup.ClearFeedback
          end
        else
          Builtins.y2milestone("AutoYaST Upgrade:no Novell DHCP to upgrade.")
        end

        #*******
        # NCS *
        #*******
        if NovellPatterns.IsPatternInstalled("novell-ncs")
          if IsServiceConfigured("ncs")
            Builtins.y2milestone("AutoYaST Upgrade:doing NCS upgrade.")

            Popup.ShowFeedback(
              Builtins.sformat(_("Configuring NCS (AutoYaST Upgrade for %1)"),@oes_version),
              _("This may take a while..........")
            )

            # Read the settings.
            @ret2 = Ncs.Read
            Builtins.y2milestone(
              "AutoYaST Upgrade:ret from Ncs::Read() = %1",
              @ret2
            )

            # Do the Write() to perform upgrade
            @ret2 = Ncs.Write
            Builtins.y2milestone(
              "AutoYaST Upgrade:ret from Ncs::Write() = %1",
              @ret2
            )

            Popup.ClearFeedback
          end
        else
          Builtins.y2milestone("AutoYaST Upgrade:no NCS to upgrade.")
        end

        #********
        # CIFS *
        #********
        if NovellPatterns.IsPatternInstalled("novell-cifs")
          if IsServiceConfigured("novell-cifs")
            Builtins.y2milestone("AutoYaST Upgrade:doing Novell CIFS upgrade.")

            Popup.ShowFeedback(
              Builtins.sformat(_("Configuring OES CIFS (AutoYaST Upgrade for %1)"),@oes_version),
              _("This may take a while..........")
            )

            # Read the settings.
            @ret2 = NovellCifs.Read
            Builtins.y2milestone(
              "AutoYaST Upgrade:ret from NovellCifs::Read() = %1",
              @ret2
            )

            # Do the Write() to perform upgrade
            @ret2 = NovellCifs.Write
            Builtins.y2milestone(
              "AutoYaST Upgrade:ret from NovellCifs::Write() = %1",
              @ret2
            )

            Popup.ClearFeedback
          end
        else
          Builtins.y2milestone("AutoYaST Upgrade:no Novell CIFS to upgrade.")
        end

        #********
        # NSS AD *
        #********
        if NovellPatterns.IsPatternInstalled("novell-nssad")
          if IsServiceConfigured("novell-nssad")
            Builtins.y2milestone("AutoYaST Upgrade:doing OES NSS AD upgrade.")

            Popup.ShowFeedback(
              Builtins.sformat(_("Configuring OES NSS AD (AutoYaST Upgrade for %1)"),@oes_version),
              _("This may take a while..........")
            )

            # Read the settings.
            @ret2 = NovellNssad.Read
            Builtins.y2milestone(
              "AutoYaST Upgrade:ret from NovellNssad::Read() = %1",
              @ret2
            )

            # Do the Write() to perform upgrade
            @ret2 = NovellNssad.Write
            Builtins.y2milestone(
              "AutoYaST Upgrade:ret from NovellNssad::Write() = %1",
              @ret2
            )

            Popup.ClearFeedback
          end
        else
          Builtins.y2milestone("AutoYaST Upgrade:no OES NSS AD to upgrade.")
        end


        # Do any post config work...
        Popup.ShowFeedback(
          Builtins.sformat(
          _(
            "Doing post configuration tasks (AutoYaST Upgrade for %1)"
          ),
          @oes_version
        ),
          _("This may take a while..........")
        )

        @ret2 = DeleteAnswerFile()
        Builtins.y2milestone(
          "AutoYaST Upgrade:ret from deleting answer file = %1",
          @ret2
        )

        Popup.ClearFeedback

        Popup.ShowFeedback(
          Builtins.sformat(_("Upgrade Complete (AutoYaST Upgrade for %1)"),@oes_version),
          ""
        )
        Builtins.sleep(5000) #sleep 5 seconds
        Popup.ClearFeedback

        Builtins.y2milestone("AutoYaST Upgrade finished")
        Builtins.y2milestone("----------------------------------------")
	
	if FileUtils.Exists("/var/lib/YaST2/OES11_AUTOUPGRADE")
          Builtins.y2milestone("inst_oes_postconfig: autoyast upgrade mode")
          cmd = "rm -rf /var/lib/YaST2/OES11_AUTOUPGRADE"
          Builtins.y2milestone("inst_oes_postconfig:executing %1", cmd)
          retmap = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
          Builtins.y2milestone("Removed OES11_AUTOUPGRADE file and the return value is %1", retmap)
        end

      end

      if NovellUtils.IsSecureBoot
        NovellUtils.ImportOesCertificate
      end

      updateUmcRestConfig = Builtins.sformat(
        "python3 /opt/novell/umc/scripts/umc-config.py update_oes_rest_config"
      )
      Builtins.y2milestone(
        "Executing %1",
        updateUmcRestConfig
      )
      returnValue = Convert.to_map(
        SCR.Execute(
          path(".target.bash_output"),
          updateUmcRestConfig
        )
      )
      Builtins.y2milestone(
        "Return Value for %1 is %2",
        updateUmcRestConfig,
        returnValue
      )
      Service.Restart("microfocus-umc-backend")

      @abortInstall = Builtins.tolower(
        NovellUtils.ReadOption("ABORT_OES_INSTALL")
      )
      if @abortInstall == "yes"
        Builtins.y2milestone(
          "inst_oes_postconfig:setting ABORT_OES_INSTALL is set to 'yes'."
        )
        return @ret
      end

      Popup.ShowFeedback(
        _("Please Wait"),
        _("Finalizing installation of Open Enterprise Server components.")
      )

      SCR.Write(path(".sysconfig.novell-common.NDSCONFIG_THIS_SESSION"), "no")
      Builtins.y2milestone(
        "inst_oes_postconfig:setting NDSCONFIG_THIS_SESSION to no."
      )

      @stop_ndsd = false # used later for special dsfw upgrade case.
      @start_dsfw_upgrade = Builtins.tolower(
        NovellUtils.ReadOption("START_DSFW_UPGRADE")
      )

      @execute_cmd = ""
      @rval = -1
      @service_configured = Builtins.tolower(
        NovellUtils.ReadOption("SERVICES_CONFIGURED")
      )
      if @service_configured != "yes"
        if Stage.normal == false
          # Stop ndsd - since going from runlevel 3 to runlevel 5 will restart it
          # Otherwise messages on the console will state that it could not be
          # started since it was already running.
          Builtins.y2milestone(
            "inst_oes_postconfig:stopping novell-cifs and ndsd if running."
          )
#          if NovellPatterns.IsPatternInstalled("novell-ncs")
#            Service.Stop("novell-ncs")
#          end
          if NovellPatterns.IsPatternInstalled("novell-cifs")
            Service.Stop("novell-cifs")
          end

          if NovellPatterns.IsPatternInstalled("novell-iprint")
            Service.Stop("novell-ipsmd")
          end

          if @start_dsfw_upgrade != "yes"
            Service.Stop("ndsd")
            Service.Start("ndsd")
          else
            @stop_ndsd = true
          end

#          if NovellPatterns.IsPatternInstalled("novell-ncs")
#            if Service.Enabled("novell-ncs")
#              Service.Start("novell-ncs")
#	    end
#          end
          if NovellPatterns.IsPatternInstalled("novell-cifs")
            if Service.Enabled("novell-cifs")
              Service.Start("novell-cifs")
            end
          end

          if NovellPatterns.IsPatternInstalled("novell-iprint")
            Service.Start("novell-ipsmd")
          end
        end

        # Allow ports 80 (HTTP), 443 (HTTPS), and 5989 (WBEM) through the firewall
        NovellUtils.SetFirewallPort(80, :tcp, true)
        NovellUtils.SetFirewallPort(443, :tcp, true)
        NovellUtils.SetFirewallPort(5989, :tcp, true)

        SCR.Write(path(".sysconfig.novell-common.SERVICES_CONFIGURED"), "yes")
      end

      # Start red-carpet daemon (RCD)
      if Package.Installed("rcd")
        Service.Enable("rcd")
        Service.Start("rcd")
        Builtins.y2milestone(
          "inst_oes_postconfig:started and enabled red-carpet daemon (rcd"
        )
      end

      #Temporary fix to stop openwbem
      if Package.Installed("openwbem")
        Service.Enable("owcimomd")
        Service.Stop("owcimomd")

        Builtins.y2milestone("Temporary fix : stopping  owcimomd")
        @execute_cmd = "/sbin/chkconfig owcimomd off"
        Builtins.y2milestone("eDir:Executing %1", @execute_cmd)
        @rval = Convert.to_integer(
          SCR.Execute(path(".target.bash"), @execute_cmd)
        )
        if @rval != 0
          Builtins.y2error("Temp fix - eDir:chkconfig owcimomd off failed")
        end
      end

      # Start SFCS - always installed
      Service.Enable("sblim-sfcb")
      Service.Stop("sblim-sfcb")
      Builtins.y2milestone(
        "inst_oes_postconfig:stopping sblim-sfcb in preparation for restart."
      )
      Service.Start("sblim-sfcb")
      Builtins.y2milestone("inst_oes_postconfig:started and enabled sblim-sfcb")

      if Package.Installed("novell-nrm")
        @package = "microfocus-nagios"
        @service = "nagios"
        if Package.Installed(@package)
          Service.Enable(@service)
          if Service.Status(@service) != 0
            Service.Start(@service)
            Builtins.y2milestone(
              "inst_oes_postconfig:started and enabled %1",
              @service
            )
          end
        end
        @nrm_started = Builtins.tolower(NovellUtils.ReadOption("NRM_STARTED"))
        if @nrm_started != "yes"
          # Start NRM - always installed
          Service.Enable("novell-httpstkd")
          Service.Start("novell-httpstkd")
          Builtins.y2milestone(
            "inst_oes_postconfig:started and enabled novell-httpstkd"
          )
          # Open ports 8008 or 8009 if NRM is installed
          NovellUtils.SetFirewallPort(8008, :tcp, true)
          NovellUtils.SetFirewallPort(8009, :tcp, true)
          SCR.Write(path(".sysconfig.novell-common.NRM_STARTED"), "yes")
        end
      end

      # Disabling cups if novell-iprint pattern is selected
      if NovellPatterns.IsPatternInstalled("novell-iprint")
        if Package.Installed("cups")
          @cmd2 = "/usr/sbin/rccups stop"
          Builtins.y2milestone(
            "inst_oes_postconfig: stop rccups command: %1",
            @cmd2
          )
          @mymap = Convert.to_map(
            SCR.Execute(path(".target.bash_output"), @cmd2)
          )
          if Ops.get_integer(@mymap, "exit", -1) != 0
            Builtins.y2milestone(
              "inst_oes_postconfig: stop rccups failed: %1 using",
              @mymap
            )
          end

          @cmd2 = "/sbin/insserv -r cups"
          Builtins.y2milestone(
            "inst_oes_postconfig: disable rccups command: %1",
            @cmd2
          )
          @mymap = Convert.to_map(
            SCR.Execute(path(".target.bash_output"), @cmd2)
          )
          if Ops.get_integer(@mymap, "exit", -1) != 0
            Builtins.y2milestone(
              "inst_oes_postconfig: disable rccups failed: %1 using",
              @mymap
            )
          end
        end
      end
      #Opening ports for CIS pattern
      if NovellPatterns.IsPatternInstalled("cloud-integrated-storage")

        NovellUtils.SetFirewallPort(8105, :tcp, true)
        NovellUtils.SetFirewallPort(24224, :tcp, true)

        Builtins.y2milestone("Opened 8105 and 24224 ports")

        cmd = "grep cisConfigSuccess /etc/opt/novell/cis/status.json |cut \-d\"\:\" \-f2 | cut \-d\"\,\" -f1"
        retmap = Convert.to_map(
          SCR.Execute(path(".target.bash_output"), cmd)
        )
        ret = NovellUtils.CleanUpString(Ops.get_string(retmap,"stdout",""))
        Builtins.y2milestone("Checking cisConfigSuccess value at /etc/opt/novell/cis/status.json : %1",ret)
        if ret == "false"
          cmd = "/opt/novell/cis/bin/activate_configuration.sh"
          Builtins.y2milestone("inst_oes_postconfig:executing %1", cmd)
          retmap = Convert.to_map(
            SCR.Execute(path(".target.bash_output"), cmd)
          )
          Builtins.y2milestone("inst_oes_postconfig:output is %1", retmap)
        end
        # script to configure cis port during upgrade
        if Mode.update || Mode.autoupgrade
          cmd = "/opt/novell/cis/bin/cis_susefirewall2firewalld.sh"
          Builtins.y2milestone("inst_oes_postconfig:executing %1", cmd)
          retmap = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
          Builtins.y2milestone("inst_oes_postconfig:output is %1", retmap)
        end
      end

#       Removing iFolder files during upgrade
      ifolder_files= ["/etc/apache2/conf.d/simias.conf","/etc/apache2/conf.d/ifolder_admin.conf","/etc/apache2/conf.d/ifolder_web.conf","/etc/opt/novell/httpd/conf.d/base.conf","/opt/novell/ifolder3"]
       Builtins.y2milestone("Removing ifolder files during upgrade")
       Builtins.foreach(ifolder_files) do |file|
         if FileUtils.Exists(file)
          ret = Convert.to_integer(
            SCR.Execute(path(".target.bash"), Builtins.sformat("rm -rf %1",file))
          )
          Builtins.y2milestone("executed rm %1 and return value is %2",file,ret)
          end
       end
       ret = Convert.to_integer(
          SCR.Execute(path(".target.bash"), Builtins.sformat("rm -f /etc/sysconfig/novell/ifldr3_*"))
          )
       Builtins.y2milestone("executed rm /etc/sysconfig/novell/ifldr3_* and return value is %1",ret)
       ret = Convert.to_integer(
          SCR.Execute(path(".target.bash"), Builtins.sformat("unlink /etc/opt/novell/httpd/conf.d/base.conf"))
          )
       Builtins.y2milestone("executed rm /etc/opt/novell/httpd/conf.d/base.conf and return value is %1",ret)

       #Calling iprint_update_apache script
       if NovellPatterns.IsPatternInstalled("novell-iprint")
         cmd = "/opt/novell/iprint/bin/iprint_update_apache"
         Builtins.y2milestone("inst_oes_postconfig: Executing: %1", cmd)
         rval = Convert.to_integer(SCR.Execute(path(".target.bash"), cmd))
         if rval != 0
           Builtins.y2milestone(
             "inst_oes_postconfig:iprint_update_apachescript returned error: %1",
             rval
           )
         end
       end

      # Call the welcome page setup script to register all installed products.
      @cmd = "/etc/opt/novell/httpd/scripts/wpincludes.pl rebuild"
      Builtins.y2milestone("inst_oes_postconfig: Executing: %1", @cmd)
      @rval = Convert.to_integer(SCR.Execute(path(".target.bash"), @cmd))
      if @rval == 0
        Builtins.y2milestone(
          "inst_oes_postconfig:welcome page wpincludes.pl script called successfully."
        )
        # Fix for bug #678402
        if Service.Status("apache2") == 0
          Service.Enable("apache2")
          if Stage.normal
            Service.Reload("apache2")
            Builtins.y2milestone("inst_oes_postconfig:reloaded apache2 ")
          end
        else
          Service.Enable("apache2")
          Service.Start("apache2")
          Builtins.y2milestone("inst_oes_postconfig:started apache2")
        end
      else
        Builtins.y2milestone(
          "inst_oes_postconfig:welcome page wpincludes.pl script returned error: %1",
          @rval
        )
      end

      services=["novell-xregd","novell-xsrvd"]

      if Mode.update || Mode.autoupgrade
        Builtins.foreach(services) do |service|
          NovellUtils.SystemdStartorRestartService(service,"yes")
        end
      end

      if Mode.installation || Mode.normal 
        Builtins.foreach(services) do |service|
          NovellUtils.SystemdStartorRestartService(service,"no")
        end
      end

      #Start XTIER
      @start_xtier = Builtins.tolower(NovellUtils.ReadOption("START_XTIER"))
      Builtins.y2milestone(
        "inst_oes_postconfig:start xtier is %1",
        @start_xtier
      )
      if @start_xtier == "yes"
        Service.Enable("novell-xregd")
        Service.Enable("novell-xsrvd")
        if Stage.normal
          Service.Restart("novell-xregd")
          Service.Restart("novell-xsrvd")
          Builtins.y2milestone(
            "inst_oes_postconfig:started novell-xregd and novell-xsrvd"
          )
        else
          Service.Start("novell-xregd")
          Service.Start("novell-xsrvd")
          Builtins.y2milestone(
            "inst_oes_postconfig:restarted novell-xregd and novell-xsrvd"
          )
        end
        SCR.Write(path(".sysconfig.novell-common.START_XTIER"), "no")
      end

      # Start novfsd
      if Package.Installed("novell-novfsd")
        if Mode.update || Mode.autoupgrade
          NovellUtils.SystemdStartorRestartService("novfsd","yes")
        end

        if Mode.installation || Mode.normal
          NovellUtils.SystemdStartorRestartService("novfsd","no")
        end
      end

      #Start DNS
      @start_dns = Builtins.tolower(NovellUtils.ReadOption("START_DNS"))
      Builtins.y2milestone("inst_oes_postconfig:start dns is %1", @start_dns)
      if @start_dns == "yes"
        Service.Enable("novell-named")
        if Stage.normal
          Service.Restart("novell-named")
          Builtins.y2milestone("inst_oes_postconfig:restarted novell-named")
        else
          Service.Start("novell-named")
          Builtins.y2milestone("inst_oes_postconfig:started novell-named")
        end
      end
      if @start_dns == "yes" || NovellPatterns.IsPatternInstalled("novell-dhcp")
        SCR.Write(path(".sysconfig.novell-common.START_DNS"), "no")
        if Service.Status("boot.apparmor") == 0
          Service.Enable("boot.apparmor")
          if Stage.normal
            Service.Reload("boot.apparmor")
            Builtins.y2milestone("inst_oes_postconfig:restarted apparmor")
          end
        else
          Builtins.y2milestone(
            "inst_oes_postconfig:apparmor not running,profiles will not be reloaded"
          )
        end
      end
      #Start SMS
      @start_sms = Builtins.tolower(NovellUtils.ReadOption("START_SMS"))
      if @start_sms == "yes"
        Service.Enable("novell-smdrd")
        if Stage.normal
          Service.Restart("novell-smdrd")
          Builtins.y2milestone(
            "inst_oes_postconfig:restarted and enabled novell-smdrd"
          )
        else
          Service.Start("novell-smdrd")
          Builtins.y2milestone(
            "inst_oes_postconfig:started and enabled novell-smdrd"
          )
        end

        @execute_cmd = "/opt/novell/sms/bin/smsconfig -l tsafs"
        @rval = Convert.to_integer(
          SCR.Execute(path(".target.bash"), @execute_cmd)
        )
        Builtins.y2milestone("inst_oes_postconfig: executed: %1", @execute_cmd)
        SCR.Write(path(".sysconfig.novell-common.START_SMS"), "no")
      end
      #Start eDir
      @start_edir = Builtins.tolower(NovellUtils.ReadOption("START_EDIR"))
      if @start_edir == "yes"
        if Stage.normal
          Service.Stop("ndsd")
          Service.Start("ndsd")
          Service.Stop("novell-cifs")
          Service.Start("novell-cifs")
          Builtins.y2milestone("inst_oes_postconfig:restarted edir")
        else
          Service.Start("ndsd")
          Service.Stop("novell-cifs")
          Service.Start("novell-cifs")
          Builtins.y2milestone("inst_oes_postconfig:started edir")
        end
        SCR.Write(path(".sysconfig.novell-common.START_EDIR"), "no")
      end


      @reStartTomcat = false
      Builtins.y2milestone(
        "inst_oes_postconfig:removing tomcat cache and restarting tomcat if upgrade mode. Upgrade Mode is: %1",
        Mode.update
      )
      if Mode.update
        @cmd = "rm -r /var/opt/novell/tomcat/work/Catalina/localhost/nps/org"
        Builtins.y2milestone("inst_oes_postconfig: Executing: %1", @cmd)
        @rval = Convert.to_integer(SCR.Execute(path(".target.bash"), @cmd))
        @reStartTomcat = true
      end

      # Start tomcat 
        Service.Enable("novell-tomcat")
        Service.Enable("apache2")

        Service.Stop("apache2")
        Service.Stop("novell-tomcat")
        Service.Start("novell-tomcat")
        Service.Start("apache2")

        Builtins.y2milestone(
          "inst_oes:restarted and enabled apache2 and novell-tomcat"
        )

        SCR.Write(path(".sysconfig.novell-common.START_APACHE_TOMCAT"), "no")
        SCR.Write(path(".sysconfig.novell-common.START_APACHE"), "no")

      # Start Apache
      @start_apache = Builtins.tolower(NovellUtils.ReadOption("START_APACHE"))
      if @start_apache == "yes"
        Service.Enable("apache2")
        if Stage.normal
          Service.Restart("apache2")
          Builtins.y2milestone(
            "inst_oes_postconfig:restarted and enabled apache2"
          )
        else
          Service.Stop("apache2")
          Builtins.sleep(3000)
          Service.Start("apache2")
          Builtins.y2milestone(
            "inst_oes_postconfig:stopped, started and enabled apache2"
          )
        end
        SCR.Write(path(".sysconfig.novell-common.START_APACHE"), "no")
      end

      # Quickfinder setup call that needs to run after tomcat apache restart (does nothing if quickfinder is not installed).
      Builtins.sleep(5000) #sleep 5 seconds
      #cmd = "/var/lib/qfsearch/bin/updatetomcat.sh";
      #y2milestone("inst_oes_postconfig: Executing: %1", cmd);
      #rval = (integer)SCR::Execute(.target.bash, cmd);
      #y2milestone("inst_oes_postconfig: Return from updateTomcat is: %1", rval);

      Builtins.y2milestone(
        "inst_oes_postconfig: enabling (chkconfig) 'pure-ftpd' if 'novell-ftp' pattern installed."
      )
      if NovellPatterns.IsPatternInstalled("novell-ftp")
        Service.Enable("pure-ftpd")
        NovellUtils.SetFirewallPort(20, :both, true)
        NovellUtils.SetFirewallPort(21, :both, true)
        if Service.Status("pure-ftpd") != 0
          Service.Start("pure-ftpd")
          Builtins.y2milestone(
            "inst_oes_postconfig:started and enabled pure-ftpd"
          )
        end
      end


      Builtins.y2milestone("inst_oes_postconfig: restarting slpd.")
      @cmd = "/usr/sbin/rcslpd restart"
      Builtins.y2milestone("inst_oes_postconfig:executing: %1", @cmd)
      @rval = Convert.to_integer(SCR.Execute(path(".target.bash"), @cmd))

      NovellUtils.ResetEdirAskUserFlag # Remove edir AskUser flag file
      SCR.Execute(
        path(".target.bash"),
        "rm -f /opt/novell/oes-install/edirretryfailure"
      ) # Get rid edir failure flag

      #Start DSFW upgrade script if required
      Builtins.y2milestone(
        "inst_oes_postconfig:start dsfw upg is %1",
        @start_dsfw_upgrade
      )
      if @start_dsfw_upgrade == "yes"
        @cmd2 = "LD_LIBRARY_PATH=/opt/novell/eDirectory/eDir-exclusive/lib64/:$LD_LIBRARY_PATH perl /opt/novell/xad/sbin/upgrade_dsfw.pl"
        Builtins.y2milestone("inst_oes_postconfig:executing %1", @cmd2)

        adminPassword = Ops.get_string(NovellLDAP.Settings,"CONFIG_LDAP_ADMIN_PASSWORD","")

        @retmap = Convert.to_map(
        SCR.Execute(
          path(".target.bash_output"),
          Ops.add(Ops.add(Ops.add("ADM_PASSWD='", adminPassword), "' "), @cmd2)))

        Builtins.y2milestone(
          "inst_oes_postconfig:exit code: %1",
          Ops.get_integer(@retmap, "exit", -1)
        )
        Builtins.y2milestone(
          "inst_oes_postconfig:stdout: %1",
          Ops.get_string(@retmap, "stdout", "")
        )
        Builtins.y2milestone(
          "inst_oes_postconfig:stderr: %1",
          Ops.get_string(@retmap, "srderr", "")
        )

        @exit = Ops.get_integer(@retmap, "exit", -1)
        if @exit != 0
          if @exit == 2
            Popup.Message(NovellUtils.dsfwUpgradeMsg_2)
          else
            NovellUtils.PopupErrorDetailsWithLogFile(
              NovellUtils.dsfwUpgradeMsg,
              @retmap,
              Ops.get_string(@retmap, "cmd", ""),
              "/var/opt/novell/xad/log/ndsdcinit.log"
            )
          end
        end

        SCR.Write(path(".sysconfig.novell-common.START_DSFW_UPGRADE"), "no")
      end

      if @stop_ndsd
        Service.Stop("ndsd") # delayed until after dsfw upgrade
        Service.Start("ndsd")
      end

      if(NovellUtils.is_cis_agent_enabled)
        Service.Restart("oes-cis-agent")
        Builtins.y2milestone("oes-cis-agent restarted")
      end

      # reinitialize the firewall system 
      old_mode = Mode.mode
      begin
        Mode.SetMode("normal")
        Builtins.y2milestone("Setting mode to %1",Mode.mode)
        NovellUtils.ReloadFirewall
      ensure
        Mode.SetMode(old_mode)
        Builtins.y2milestone("Setting mode to %1",Mode.mode)
      end

      ## CIS Data scale pattern activation script
      version_file = "/etc/opt/novell/cis-scale/VERSION"
      if NovellPatterns.IsPatternInstalled("cloud-integrated-storage-data-scale") && !FileUtils::Exists(version_file)
	cmd = Builtins.sformat("/opt/novell/cis/bin/activate_cis_scale.sh")
        Builtins.y2milestone("OesCis-Scale:Running /opt/novell/cis/bin/activate_cis_scale.sh")
        val = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
        exit = Ops.get_integer(val, "exit", -1)
        stdout = Ops.get_string(val, "stdout", "")
        Builtins.y2milestone(
 	 "OesCis-Scale:Activating CIS scale service returned:%1, with stdout:\n%2",
         val, stdout )
      end

      # For some reason ndsd does not restart on the jump to the final runlevel at the end of an AutoYaST install.
      if Mode.autoinst
        Builtins.y2milestone(
          "inst_oes_postconfig:starting ndsd during AutoYaST install."
        )
        Service.Start("ndsd")
      end
	  
      #Enabling and restarting logrotate.timer post the upgrade

      service = "logrotate.timer"
      if !Service.Enabled(service)
        Builtins.y2milestone("%1 is not eanbled",service)
        Service.Enable(service)
        Builtins.y2milestone("%1 is enabled",service)
        Service.Restart(service)
        Builtins.y2milestone("%1 restarted",service)
        Service.Restart("logrotate.service")
      end

      # autoyast install / autoyast upgrade is removing the lines, adding again if it does'nt exists
      # Adding TLS_CACERTDIR, TLS_CERT and TLS_KEY to /etc/openldap/ldap.conf file

      NovellUtils.AddtoOpenLdapConfFile

      if Builtins.setenv("LDAPTLS_REQCERT", "")
        Builtins.y2milestone("LDAPTLS_REQCERT value is unset")
      else
        Builtins.y2milestone("Failed to unset TLSLDAP_REQCERT")
      end

      NovellUtils.RemoveKernelBootParam("vga") if Stage.stage == "continue"
     #During upgrade if ncs is running then we need to start bcc service
      if Mode.update || Mode.autoupgrade
        if NovellPatterns.IsPatternInstalled("novell-bcc")
          Builtins.y2milestone("inst_oes_postconfig:novell-bcc pattern is installed")
          if Service.Active("novell-ncs") 
            Builtins.y2milestone("inst_oes_postconfig:novell-ncs service is running")
            if Service.Enabled("novell-bcc")
              Builtins.y2milestone("inst_oes_postconfig:novell-bcc is enabled")
              Service.Restart("novell-bcc")
              if Service.Active("novell-bcc") 
                Builtins.y2milestone("inst_oes_postconfig:novell-bcc service is running")
              else
               Builtins.y2milestone("inst_oes_postconfig:novell-bcc service is not running")
              end
            else
              Builtins.y2milestone("inst_oes_postconfig:novell-bcc service is disabled")
            end
          else
            Builtins.y2milestone("inst_oes_postconfig:novell-ncs service is not running")
          end
        end
      end

      Builtins.y2milestone("Setting default filesystem to ext4.")
      SCR.Write(path(".sysconfig.storage.DEFAULT_FS"), "ext4")

      # Copying installation.xml from OES media to /etc/opt/novell/oes-installation.xml

#      NovellUtils.CopyControlFile

      Popup.ClearFeedback

      @ret 

      # EOF
    end
  end
end

Yast::InstOesPostconfigClient.new.main
