# encoding: utf-8

# -------------------------------------------------------------------------------
# Copyright 2024 Open Text.
#
# The only warranties for products and services of Open Text and its
# affiliates and licensors (“Open Text”) are as may be set forth in the
# express warranty statements accompanying such products and services.
# Nothing herein should be construed as constituting an additional
# warranty. Open Text shall not be liable for technical or editorial
# errors or omissions contained herein. The information contained herein
# is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains
# confidential information and a valid license is required for possession,
# use or copying. If this work is provided to the U.S. Government,
# consistent with FAR 12.211 and 12.212, Commercial Computer Software,
# Computer Software Documentation, and Technical Data for Commercial Items
# are licensed to the U.S. Government under vendor's standard commercial
# license.
# -------------------------------------------------------------------------------
# -------------------------------------------------------------------------------
# (C) Copyright 2019 - 2021 Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates
# and licensors (“Micro Focus”) are as may be set forth in the express warranty
# statements accompanying such products and services. Nothing herein should be
# construed as constituting an additional warranty. Micro Focus shall not be
# liable for technical or editorial errors or omissions contained herein. The
# information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential
# information and a valid license is required for possession, use or copying. If
# this work is provided to the U.S. Government, consistent with FAR 12.211 and
# 12.212, Commercial Computer Software, Computer Software Documentation, and
# Technical Data for Commercial Items  are licensed to the U.S. Government under
# vendor's standard commercial license.
# -------------------------------------------------------------------------------

# File:        clients/inst_oes_preconfig.ycp
# Package:     novell-common
# Summary:     Pre-Configure items
# Authors:     Blair Merrell <bmerrell@novell.com>
#
# $Id: inst_oes_preconfig.ycp,v 1.1.2.18 2007/03/15 22:10:11 bmerrell Exp $
module Yast
  class InstOesPreconfigClient < Client
    def main
      Yast.import "UI"

      textdomain "novell-base"

      Yast.import "Package"
      Yast.import "Progress"
      Yast.import "Service"
      Yast.import "NovellUtils"
      Yast.import "FileUtils"
      Yast.import "NovellPatterns"
      Yast.import "Wizard"
      Yast.import "Label"
      Yast.import "Report"
      Yast.import "NovellAddonInstall"
      Yast.import "Mode"
      Yast.import "Stage"
      @ret = :auto

      @args = Convert.to_map(WFM.Args(0))
      return @ret if Ops.get_boolean(@args, "going_back", false) == true

      SCR.Write(path(".sysconfig.novell-common.NDSCONFIG_THIS_SESSION"), "no")
      Builtins.y2milestone(
        "inst_oes_preconfig:setting NDSCONFIG_THIS_SESSION to no."
      )

      # Maintain the firewalld service as SuSEfirewall2 status before upgrade(in SLES12SP*)
      if Mode.update || Mode.autoupgrade
        oldFirewallFile = "/usr/lib/systemd/system/SuSEfirewall2.service"
        backupFile = "/usr/lib/systemd/system/SuSEfirewall2.service.bak"
        if FileUtils.Exists(oldFirewallFile)
          Builtins.y2milestone("inst_oes_preconfig: SuSEfirewall2 service file is present")
          NovellUtils.MigrateFirewallPort
          if !Service.Enabled("SuSEfirewall2")
            Service.Stop("firewalld")
            Service.Disable("firewalld")
          else
            Service.Stop("SuSEfirewall2")
            Service.Disable("SuSEfirewall2")
            Service.Enable("firewalld")
            Service.Start("firewalld")
          end
          # move this file as. bak so that service will not be usable
          cmd = Builtins.sformat(
            "mv %1 %2",
            oldFirewallFile,
            backupFile
          )
          rval = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
          Builtins.y2milestone("inst_oes_preconfig: executing cmd: %1 and rval: %2",cmd,rval)
          if Ops.get_integer(rval, "exit", -1) != 0
            Builtins.y2milestone("inst_oes_preconfig: Moving of SuSEfirewall2 service file failed.")
          end
        end
      end

      # for disabling ipv6
      # searching net.ipv6.conf.all.disable_ipv6 = 1 in /etc/sysctl.conf
      @cmd = "grep  'net.ipv6.conf.all.disable_ipv6.*=.*1' /etc/sysctl.conf > /dev/null"
      @rval = Convert.to_map(SCR.Execute(path(".target.bash_output"), @cmd))
        Builtins.y2milestone(" searching net.ipv6.conf.all.disable_ipv6 = 1 in /etc/sysctl.conf:
          executing cmd: %1  rval: %2",@cmd,@rval)
      # if not found adding net.ipv6.conf.all.disable_ipv6 = 1  in /etc/sysctl.conf
      if Ops.get_integer(@rval, "exit", -1) != 0
        @cmd2 = "sed -i '$anet.ipv6.conf.all.disable_ipv6 = 1' /etc/sysctl.conf"
        @rval2 = Convert.to_map(SCR.Execute(path(".target.bash_output"), @cmd2))
        Builtins.y2milestone("inst_oes_postconfig: executing cmd: %1  rval: %2",@cmd2,@rval2)
      end

      #1092205
      #changing value of inet_protocols in /etc/postfix/main.cf
      @cmd = "grep -i 'inet_protocols.*ipv4' /etc/postfix/main.cf > /dev/null"
      @rval = Convert.to_map(SCR.Execute(path(".target.bash_output"), @cmd))
        Builtins.y2milestone(" searching inet_protocols = ipv4 in  /etc/postfix/main.cf:
          executing cmd: %1  rval: %2",@cmd,@rval)
      if Ops.get_integer(@rval, "exit", -1) != 0
        @cmd2 = "sed -i 's/inet_protocols =.*/inet_protocols = ipv4/' /etc/postfix/main.cf"
        @rval2 = Convert.to_map(SCR.Execute(path(".target.bash_output"), @cmd2))
        Builtins.y2milestone("inst_oes_preconfig: executing cmd: %1  rval: %2",@cmd2,@rval2)
      end
      #changing value of myhostname in /etc/postfix/main.cf
      @cmd = Builtins.sformat("grep -i '^myhostname.*%1' /etc/postfix/main.cf > /dev/null", NovellUtils.GetDNSname)
      @rval = Convert.to_map(SCR.Execute(path(".target.bash_output"), @cmd))
        Builtins.y2milestone(" searching myhostname in /etc/postfix/main.cf:
          executing cmd: %1  rval: %2",@cmd,@rval)
      if Ops.get_integer(@rval, "exit", -1) != 0
        @cmd2 = Builtins.sformat("sed -i 's/^myhostname =.*/myhostname = %1/' /etc/postfix/main.cf", NovellUtils.GetDNSname)
        @rval2 = Convert.to_map(SCR.Execute(path(".target.bash_output"), @cmd2))
        Builtins.y2milestone("inst_oes_preconfig: executing cmd: %1  rval: %2",@cmd2,@rval2)
      end
      #Restarting Postfix service
      Service.Restart("postfix")
      if Service.Active("postfix")
        Builtins.y2milestone("inst_oes_preconfig: restarted postfix service")
      else
        Builtins.y2milestone("inst_oes_preconfig: unable to start postfix service")
      end


      # Adding TLS_CACERTDIR, TLS_CERT and TLS_KEY to /etc/openldap/ldap.conf file

      NovellUtils.AddtoOpenLdapConfFile

      if Builtins.setenv("LDAPTLS_REQCERT", "never")
        Builtins.y2milestone("LDAPTLS_REQCERT is set")
      else
        Builtins.y2milestone("failed to set LDAPTLS_REQCERT")
      end

      @xad_pattern = Convert.to_string(
        SCR.Read(path(".sysconfig.novell.xad.PATTERN_NAME"))
      )
      @xad_packages_installed = NovellPatterns.IsPatternInstalled(@xad_pattern)
      @val = Convert.to_string(
        SCR.Read(path(".sysconfig.edirectory.SERVICE_CONFIGURED"))
      )

      #1105182
      if NovellPatterns.IsPatternInstalled("novell-edirectory")
        if Mode.mode == "installation" || Mode.mode == "normal"
          if !@xad_packages_installed
            TypicalOrCustomScreenForEdir()
          elsif @xad_packages_installed &&  NovellPatterns.IsPatternInstalled("novell-premigration-server")
	          TypicalOrCustomScreenForEdir()
          end
        end
      end

      NovellUtils.ModifyingSfcb

      if Mode.update || Mode.autoupgrade
        Service.Stop("novell-tomcat")
        Builtins.y2milestone("Stopping novell-tomcat service during upgrade to avoid java core")
      end

      # Starting haveged daemon

      if Service.Status("haveged") != 0
        Service.Enable("haveged")
        Service.Start("haveged")
        Builtins.y2milestone("inst_oes_postconfig:started haveged")
      end

      NovellUtils.ResetEdirAskUserFlag # Get rid of any residual flag file on the system.
      SCR.Execute(
        path(".target.bash"),
        "rm -f /opt/novell/oes-install/edirretryfailure"
      ) # Get rid edir failure flag

      SCR.Write(path(".sysconfig.novell-common.ABORT_OES_INSTALL"), "no")
      SCR.Write(path(".sysconfig.novell-common"), nil) # flush the cache and write to disk

      @ret

      # EOF
    end

    #1105182
    def TypicalOrCustomScreenForEdir
      if @val == "no"
        @contents = HVSquash(
          VBox(
            HBox(
              Frame(
                _("OES Configuration:"),
                 MarginBox(
                  1.5,
                  0.2,
                  RadioButtonGroup(
                  Id(:edir_quick_install),
                   VBox(
                     Left(
                        RadioButton(
                         Id(0),
                          Opt(:notify),
                          _(
                            "Typical Configuration (requires minimal input)"
                          )
                        )
                      ),
                      Left(
                       RadioButton(
                         Id(1),
                         Opt(:notify),
                          _(
                           "Custom  Configuration (requires input for all options)"
                          )
                       )
                      )
                   )
                 )
                )
             )
           )
         )
       )
         @helps = _(
                "<p><b><big>Typical Configuration</big></b> </p>A two-click express installation with minimal user inputs. This method collects only essential information to proceed with the OES configuration and uses default values for most options. In case you want to modify the default configuration parameters, in the OES install summary screen, click the respective links and modify them.</p>"
         )
        @helps = Ops.add(
          @helps,
          _(
            "<p><b><big>Custom Configuration</p></b></big> <p>This method of OES configuration requires inputs for all parameters.</p> "
           )
        )
         Wizard.SetContentsButtons(
          _("Open Enterprise Server Configuration"),
           @contents,
           @helps,
           Label.BackButton,
           Label.NextButton
         )
        UI.SetFocus(Id(:next))
        UI.ChangeWidget(Id(:edir_quick_install), :CurrentButton, 0)
         while true
           @ret1 = Convert.to_symbol(UI.UserInput)
           @install_type = Convert.to_integer(
           UI.QueryWidget(Id(:edir_quick_install), :CurrentButton)
           )
           # abort?
           if @ret1 == :abort || @ret1 == :cancel
             return :abort
             break
           elsif @ret1 == :next
             NovellUtils.edir_quick_install = true if @install_type == 0
             break
           elsif @install_type == 0 && @ret1 != :next && @ret1 != :back
             Report.Warning(
               _(
                 "A two-click express installation with minimal user inputs.\n" +
                 "This method collects only essential information to proceed with\n" +
                 "the OES configuration. In case you want to modify the default\n" +
                 "configuration parameters, in the OES install summary screen,\n" +
                 "click the respective links and modify them.\n"
               )
             )
           elsif @ret1 == :back
             if Stage.stage == "continue"
               @ret1 = Convert.to_symbol(WFM.CallFunction("lan"))
             end
             return :back
             break
           else
             Builtins.y2error("unexpected retcode: %1", @ret)
             next
           end
         end
       end
    end
  end
end

Yast::InstOesPreconfigClient.new.main
