# encoding: utf-8

# Copyright (c) 2020 Micro Focus or one of its affiliates.
# File:        clients/inst_oes_autoupdate.ycp
# Package:     novell-common
# Summary:     Pre-Configure items
# Authors:     Suresh K. Hosamani <hsuresh@novell.com>
#
# $Id: inst_oes_autoupdate.ycp,v 1.0 2011/10/20 22:10:11 hsuresh Exp $

module Yast
  class InstOesSambaRemovalClient < Client
    def main

      textdomain "novell-base"

      Yast.import "Package"
      Yast.import "Progress"
      Yast.import "Service"
      Yast.import "Mode"
      Yast.import "NovellUtils"
      @ret = :auto


      @args = Convert.to_map(WFM.Args(0))
      return @ret if Ops.get_boolean(@args, "going_back", false) == true

      Packages.Init(true)
      if true
        PackageCallbacks.SetConvertDBCallbacks

        Pkg.TargetInit(Installation.destdir, false)

      end

      Builtins.y2milestone("inst_oes_samba_removal: Checking whether novell-samba pattern is already installed, during upgrade will remove novell-samba pattern and install novell-cifs pattern based on the option selection") 
      @package_properties = Pkg.ResolvableProperties("novell-samba-config", :package, "")
      Builtins.foreach(@package_properties) do |p|
        if Ops.get(p, "status") == :selected || Ops.get(p, "status") == :installed || Ops.get_symbol(p, "status") == :removed
          goon = Popup.AnyQuestion3(_("Novell Samba Deprecated"),
          _("On upgrading this OES Server, Novell Samba will be removed from\nyour server. The data that was accessed from the Samba shares\ncan now only be accessed through CIFS shares.\n
Click Remove Samba to remove Novell Samba.\n
Click Replace Samba with Novell CIFS to remove Novell Samba
and install Novell CIFS.\n
Click Abort to cancel upgrade.
"),
          _("&Remove Samba"),
          _("Replace Samba with Novell &CIFS"),
          _("&Abort"),
          :focus_yes
          ) 
          if goon == :retry
            NovellUtils.CifsPatternInstalled =false 
            @ret = :abort
          elsif goon == :no
            Pkg.ResolvableInstall("novell-cifs", :pattern)
            Pkg.ResolvableInstall("novell-cifs", :package)
            NovellUtils.CifsPatternInstalled = true
            Builtins.y2milestone("novell-cifs is selected for installation")
          end	                 
        end
      end
    @ret
    end
  end
end

Yast::InstOesSambaRemovalClient.new.main
