# encoding: utf-8

# File:        clients/inst_oes_sw_select.ycp
# Module:      Installation/Configuration of Open Enterprise Server (OES)
# Summary:     Dialog for selecting OES products to install
# Authors:     Michael G. Fritch <mgfritch@novell.com>
#
# $Id: inst_oes_sw_select.ycp 31352 2006-06-06 23:53:13Z mgfritch $
module Yast
  class InstOesSwSelectClient < Client
    def main
      Yast.import "UI"
      Yast.import "Pkg"

      textdomain "novell-base"

      Builtins.y2milestone("----------------------------------------")
      Builtins.y2milestone("inst_oes_sw_select started")

      Yast.import "NovellPatterns"
      Yast.import "ProductControl"
      Yast.import "Report"
      Yast.import "Wizard"




      #/////////////////////////////////////////////////////////////////////////
      # MAIN
      #/////////////////////////////////////////////////////////////////////////

      @ret = :auto
      @current_control_file = ProductControl.custom_control_file
      Builtins.y2milestone(
        "inst_oes_sw_select: Saving control file: %1",
        @current_control_file
      )

      # To get the resolver to do a full run (detect conflicting packages, etc.), we must have a "marker" file at a specific location (see bugzilla 351593).
      # integer ccode = (integer)SCR::Execute(.target.bash, "touch /var/lib/YaST2/force_full_solver_run");
      # y2milestone("inst_oes_sw_select: creating '/var/lib/YaST2/force_full_solver_run': %1", ccode);

      # start package management....
      @ret = patternSelection
      Builtins.y2milestone(
        "inst_oes_sw_select: patternSelection returned: %1",
        @ret
      )

      # Start the SlideShow and intall any selected packages...
      if @ret == :accept || @ret == :next
        if NovellPatterns.CommitChanges == false
          @ret = :abort

          # if NovellPatterns::dsfw_install_abort is true will abort the installation.

          return @ret if NovellPatterns.dsfw_install_abort

          #Report.Error(_("An error occured while installing packages")) # FIXME: need better error message
        end
        @ret = :next
      elsif @ret == :cancel
        # FIXME: do we want a popup confirming the abort?
        @ret = :abort
      end

      # Restore the saved control file
      Builtins.y2milestone(
        "ProductControl::custom_control_file = %1",
        ProductControl.custom_control_file
      )
      Builtins.y2milestone(
        "inst_oes_sw_select: Restoring control file: %1",
        @current_control_file
      )
      ProductControl.custom_control_file = @current_control_file
      if !ProductControl.Init
        # popup error message
        # %1 = absolute pathname to a product control file (i.e. '/usr/share/YaST2/control/oes_install.xml')
        Report.Error(
          Builtins.sformat(
            _("Control file %1 was not found."),
            ProductControl.custom_control_file
          )
        )
        @ret = :abort
      end

      Builtins.y2milestone("inst_oes_sw_select: returned: %1", @ret)
      @ret 

      # EOF
    end

    # Start the detailed package selection. If 'mode' is non-nil, it will be
    # passed as an option to the PackageSelector widget.
    #
    # Returns `accept or `cancel .
    #
    def detailedSelection(mode)
      Pkg.SaveState

      # Open empty dialog for instant feedback

      UI.OpenDialog(
        Opt(:defaultsize),
        ReplacePoint(Id(:rep), Label(_("Reading package database...")))
      )

      # This will take a while: Detailed package data are retrieved
      # while the package manager is initialized
      UI.ReplaceWidget(
        :rep,
        mode == nil ?
          PackageSelector(Id(:packages), "") :
          PackageSelector(Id(:packages), Opt(mode), "")
      )

      result = Convert.to_symbol(UI.RunPkgSelection(Id(:packages)))
      UI.CloseDialog
      Builtins.y2milestone("Package selector returned  %1", result)

      if result == :accept
        Pkg.ClearSaveState
      else
        Pkg.RestoreState(false)
      end

      result
    end


    # Start the pattern selection dialog. If the UI does not support the
    # PatternSelector, start the detailed selection with "selections" as the
    # initial view.
    #
    def patternSelection
      Builtins.y2milestone(
        "UI::HasSpecialWidget(`PatternSelector ) = %1",
        UI.HasSpecialWidget(:PatternSelector)
      )
      Builtins.y2milestone(
        "UI::WizardCommand(`Ping() ) = %1",
        UI.WizardCommand(term(:Ping))
      )

      if !UI.HasSpecialWidget(:PatternSelector) ||
          UI.WizardCommand(term(:Ping)) != true
        Builtins.y2milestone("Starting detailedSelection...")
        return detailedSelection(nil) # Fallback: detailed selection
      end

      Builtins.y2milestone("Starting patternSelector...")

      # help text: post install software selection 1/8
      help_text = _(
        "<p>By changing the action on the patterns shown in each category, you determine the role this computer will serve in your network, such as a desktop, server, firewall or collaboration device.</p>"
      ) +
        # help text: post install software selection 2/8
        _(
          "<p>The categories listed and the patterns preselected for installation will vary according to the product you are installing or adding to the computer.</p>"
        ) +
        # help text: post install software selection 3/8
        _(
          "<p>To view a description of a pattern in the right pane, click a pattern.</p>"
        ) +
        # help text: post install software selection 4/8
        _(
          "<p>To change the installation status of a pattern, click its associated icon on the left.</p>"
        ) +
        # help text: post install software selection 5/8
        _(
          "<p>To view the possible status icons and their associated meanings, right-click a pattern.</p>"
        ) +
        # help text: post install software selection 6/8
        _(
          "<p>To view or change the installation status of individual packages in a selected pattern, click <b>Details</b>.</p>"
        ) +
        # help text: post install software selection 7/8
        _(
          "<p><b>Important:</b> Be sure to leave free disk space for system use. Full disk partitions can degrade system performance and even cause serious problems. </p>"
        ) +
        # help text: post install software selection 8/8
        _(
          "<p><b>Disk Usage</b> shows the disk space that will remain after the selected software has been installed.</p>"
        )


      Wizard.OpenAcceptDialog

      Wizard.SetContents(
        # Dialog title
        _("Software Selection"),
        PatternSelector(Id(:patterns)),
        help_text,
        true, # has_back
        true
      ) # has_next

      Wizard.SetDesktopIcon("oes-install")

      result = nil
      begin
        result = Convert.to_symbol(UI.RunPkgSelection(Id(:patterns)))
        Builtins.y2milestone("Pattern selector returned %1", result)

        if result == :details
          result = detailedSelection(nil)

          if result == :cancel
            # don't get all the way out - the user might just have
            # been scared of the gory details.
            result = nil
          end
        end
      end until result == :cancel || result == :accept

      Wizard.CloseDialog

      result
    end
  end
end

Yast::InstOesSwSelectClient.new.main
