# encoding: utf-8

# File:	modules/EdirUtils.ycp
# Package:	novell-common
# Summary:	Edir utility routines
# Authors:	Stephen T. Bunch <sbunch@novell.com>
#
# $Id: EdirUtils.ycp,v 1.1.2.26 2005/10/24 16:07:51 rhaslam Exp $
#
require "yast"

module Yast
  class EdirUtilsClass < Module
    def main

      textdomain "novell-base"

      Yast.import "Popup"
      Yast.import "Report"
      Yast.import "NovellUtils"
      Yast.import "NovellValidators"
      Yast.import "ValidationHelpers"
      Yast.import "String"
    end

    def EdirConfigProblemPrompt(product)
      if Popup.YesNo(
          Builtins.sformat(
            _(
              "It appears that eDirectory has not been successfully configured.  Configuration of %1\n" +
                "may fail if you continue. Once eDirectory is properly configured, %2 can be configured later.\n" +
                "Configure %3 later?"
            ),
            product,
            product,
            product
          )
        )
        Builtins.y2milestone(
          "EdirUtils.EdirConfigProblemPrompt:user has chosen to skip write of %1",
          product
        )
        return true
      end
      Builtins.y2milestone(
        "EdirUtils.EdirConfigProblemPrompt:user has chosen to NOT skip write of %1",
        product
      )
      false
    end

    # isLDAPdn(string dn)
    #
    # determine if the dn presented in in LDAP formet
    #
    # returns ture if it is in LDAP format
    # returns false otherwise
    def isLDAPdn(dn)
      NovellUtils.isLDAP(dn)
    end

    # GetLeafNameNDS(string ndsName)
    #
    # Get the leaf portion of an X500 name
    # presented NDS format
    def GetLeafNameNDS(ndsName)
      tmp = ValidationHelpers.toNds(ndsName)
      tmp = NovellUtils.Typeful2Typeless(tmp)
      tmp = String.Replace(tmp, "\\.", "*")
      # Remove type information
      if Builtins.findfirstof(tmp, ".") == nil
        tmp = String.Replace(tmp, "*", "\\.")
        return tmp
      else
        result = Builtins.substring(tmp, 0, Builtins.findfirstof(tmp, "."))
        result = String.Replace(result, "*", "\\.")
        return result
      end
    end

    # GetLeafNameLDAP(string ldapName)
    #
    # Get the leaf portion of an X500 name
    # presented LDAP format
    def GetLeafNameLDAP(ldapName)
      ndsName = ValidationHelpers.toNds(ldapName)
      tmp = NovellUtils.Typeful2Typeless(ndsName)
      tmp = ValidationHelpers.toLdap(tmp)
      tmp = String.Replace(tmp, "\\,", "*")
      if Builtins.findfirstof(tmp, ",") == nil
        tmp = String.Replace(tmp, "*", "\\,")
        return tmp
      else
        result = Builtins.substring(tmp, 0, Builtins.findfirstof(tmp, ","))
        result = String.Replace(result, "*", "\\,")
        return result
      end
    end


    # Since we do not want to write the password in clear text, we will write the password
    # to the temp dir so that only root is allowed to see.  It is autodeleted by the system.
    # The python ncs code will then read from this file and get the data.
    # Now that we have the system tmp directory, write it to a file at a well known location.




    # Get the GID for a group in edirectory.
    # @return true if modified
    def GetGIDforGroup(ipaddress, port, admin, adminPassword, groupContext, groupName)
      # The following data works on an xad system with the domain name indicated.
      #  string ipaddress = "151.155.188.135";
      #  string port = "636";
      #  string admin = "CN=Administrator,CN=Users,dc=cimlab,dc=provo,dc=novell,dc=com"; // NOTE: this will need to be converted to LDAP format by the NovellUtils...
      #  string groupContext = "cn=Users,dc=cimlab,dc=provo,dc=novell,dc=com"; // NOTE: convert to LDAP using NovellUtils
      #  string groupName = "cn=Domain Users";

      gid = ""

      # Ask system for a special temporary directory
      tmpdir = Convert.to_string(SCR.Read(path(".target.tmpdir")))
      if tmpdir == nil || tmpdir == ""
        Builtins.y2security(
          "Request for temporary directory fauled, Using /tmp directoryas a last resort!"
        )
        tmpdir = "/tmp"
      end


      # Now write the password file to the protected tmp dir.
      pwdFile = Ops.add(tmpdir, "/qxz45190") # FIXME: this is very bad practice...there are better ways to create a temp dir.
      SCR.Write(path(".target.string"), pwdFile, adminPassword)
      SCR.Execute(path(".target.bash"), Ops.add("/bin/chmod 400 ", pwdFile))

      attr = "gidnumber"

      cmd = Builtins.sformat(
        "/usr/bin/ldapsearch -l 30 -H ldaps://%1:%2 -D '%3' -y %4 -x -s one -LLL -b '%5' '%6' '%7'",
        ipaddress,
        port,
        admin,
        pwdFile,
        groupContext,
        groupName,
        attr
      )

      Builtins.y2milestone("EdirUtils:Executing command %1", cmd)
      rmap = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
      Builtins.y2milestone(
        "EdirUtils:exit:%1",
        Ops.get_integer(rmap, "exit", -1)
      )

      if Ops.get_integer(rmap, "exit", -1) == 0
        out = Ops.get_string(rmap, "stdout", "")
        Builtins.y2milestone("EdirUtils:out:%1", out)
        split = Builtins.splitstring(out, "\n")
        Builtins.y2milestone("EdirUtils:split:%1", split)
        gidStr = ""
        Builtins.foreach(split) do |tmpStr|
          gidStr = tmpStr if Builtins.issubstring(tmpStr, "gidnumber:")
        end
        split2 = []
        if gidStr != ""
          split2 = Builtins.splitstring(gidStr, " ")
          Builtins.y2milestone("EdirUtils:id:%1", Ops.get(split2, 0, ""))
          Builtins.y2milestone("EdirUtils:gid:%1", Ops.get(split2, 1, ""))
          gid = Ops.get(split2, 1, "")
        end
      else
        Builtins.y2milestone(
          "EdirUtils:Could not get GID for group:err=%1",
          Ops.get_integer(rmap, "exit", -1)
        )
      end

      gid
    end






    # DeleteDirectoryObject(
    # string admin,
    # string password,
    # string ldapServer,
    # string securePort,
    # string object
    # )
    # Returns:
    #             0 if successfull
    #         non 0 otherwise
    #
    def DeleteDirectoryObject(admin, password, ldapServer, securePort, object)
      admin = NovellUtils.Nds2Ldap(admin) if !isLDAPdn(admin)
      object = NovellUtils.Nds2Ldap(object) if !isLDAPdn(object)

      # Ask system for a special temporary directory
      tmpdir = Convert.to_string(SCR.Read(path(".target.tmpdir")))
      if tmpdir == nil || tmpdir == ""
        Builtins.y2security(
          "Request for temporary directory fauled, Using /tmp directoryas a last resort!"
        )
        tmpdir = "/tmp"
      end

      # Now write the password file to the protected tmp dir.
      pwdFile = Ops.add(tmpdir, "/x107423")
      SCR.Write(path(".target.string"), pwdFile, password)
      SCR.Execute(path(".target.bash"), Ops.add("/bin/chmod 400 ", pwdFile))

      # Formulate the ldapdelete command.
      # -H specify the URI (IPaddress and Port)
      # -D user to login as
      # -y file containing user password
      # -x use simple authentication
      # followed by the thing to delete
      logcmd = Builtins.sformat(
        "ldapdelete -H ldaps://%1:%2 -D '%3' -x '%4'",
        ldapServer,
        securePort,
        admin,
        object
      )
      cmd = Builtins.sformat(
        "ldapdelete -H ldaps://%1:%2 -D '%3' -y %4 -x '%5'",
        ldapServer,
        securePort,
        admin,
        pwdFile,
        object
      )
      Builtins.y2milestone("Deleting directory object %1", object)
      Builtins.y2milestone("Executing command %1", logcmd)
      rmap = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
      Builtins.y2debug("rmap = %1", rmap)
      retval = Ops.get_integer(rmap, "exit", -1)
      if retval != 0
        Builtins.y2error(
          "Execution of ldapdelete for object %1 failed, error returned = %2",
          object,
          Ops.get_integer(rmap, "exit", -1)
        )
        return retval
      else
        return 0
      end
    end


    # Determines if the specified object exists in the directory using
    # ldapsearch,
    #
    # @param [String] admin_dn string admin FDN
    # @param [String] passwd string admin password
    # @param [String] ldapServer string ldap server address
    # @param [Fixnum] securePort integer LDAPS port
    # @param [String] object string object we are preforming the ldapsearch for
    # @param [String] objectClass string objectClass of the object we are using ldapsearch for
    #
    # @return [Boolean] true if the specified object is found
    def DoesDirectoryObjectExist(admin_dn, passwd, ldapServer, securePort, object, objectClass)
      admin = NovellUtils.Nds2Ldap(admin_dn)
      ipaddress = ldapServer
      port = securePort
      index = Builtins.search(NovellUtils.Nds2Ldap(object), ",")
      checkClass = true

      context = NovellUtils.Nds2Ldap(NovellUtils.GetContextNDS(object))
      cn = Builtins.substring(object, 0, index)

      # if object class is entry string; turn off checkClass
      checkClass = false if objectClass == ""
      admin = NovellUtils.Nds2Ldap(admin) if !isLDAPdn(admin)
      # Ask system for a special temporary directory
      tmpdir = Convert.to_string(SCR.Read(path(".target.tmpdir")))
      if tmpdir == nil || tmpdir == ""
        Builtins.y2security(
          "Request for temporary directory fauled, Using /tmp directory as a last resort!"
        )
        tmpdir = "/tmp"
      end

      # Now write the password file to the protected tmp dir.
      pwdFile = Ops.add(tmpdir, "/qxz45190") # FIXME: this is very bad practice...there are better ways to create a temp dir.
      SCR.Write(path(".target.string"), pwdFile, passwd)
      SCR.Execute(path(".target.bash"), Ops.add("/bin/chmod 400 ", pwdFile))
      # Formulate the ldapsearch command.
      # -l 30 (allow 15 seconds for completion)
      # -H specify the URI (IPaddress and Port)
      # -D user to login as
      # -x use simple authentication
      # -y password in file for user
      # -w password for user - replace -y for testing
      # -s how deep to search from starting container
      # -LLL filter output to get just the results in a simple format
      # -b container in which to start the search
      # followed by the thing to search for

      logcmd = Builtins.sformat(
        "/usr/bin/ldapsearch -l 30 -H ldaps://%1:%2 -D '%3' -y %4 -x -s one -LLL -b '%5' '%6'",
        ipaddress,
        port,
        admin,
        "******",
        context,
        cn
      )

      cmd = Builtins.sformat(
        "/usr/bin/ldapsearch -l 30 -H ldaps://%1:%2 -D '%3' -y %4 -x -s one -LLL -b '%5' '%6'",
        ipaddress,
        port,
        admin,
        pwdFile,
        context,
        cn
      )

      Builtins.y2milestone(
        "Checking existance of object %1 of type %2",
        object,
        objectClass
      )
      Builtins.y2milestone("Executing command %1", logcmd)
      rmap = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
      if Ops.get_integer(rmap, "exit", -1) != 0
        Builtins.y2milestone(
          "Execution of ldap_search for object %1 failed, error returned = %2\n%3",
          object,
          Ops.get_integer(rmap, "exit", -1),
          rmap
        )
        return false
      end

      if Ops.get_string(rmap, "stdout", "failed") == nil ||
          Ops.get_string(rmap, "stdout", "failed") == ""
        Builtins.y2milestone(
          "The call to find object %1 failed.\n%2",
          object,
          rmap
        )
        return false
      end

      rmap_stdout = Ops.get_string(rmap, "stdout", "")
      idx = Builtins.search(rmap_stdout, "\n")
      if idx != nil # Only look on the first line of the output.
        rmap_stdout = Builtins.substring(rmap_stdout, 0, idx)
      end
      # Make sure we have not found a bogus object such as a comment
      if Builtins.regexpmatch(rmap_stdout, "^[ \t]*#")
        Builtins.y2milestone(
          "The the call to find object %1 failed. Comment found in return:\n%2",
          object,
          Ops.get_string(rmap, "stdout", "")
        )
        return false
      end

      if !checkClass
        Builtins.y2milestone("object found.  return true")
        return true
      else
        output = Ops.get_string(rmap, "stdout", "Bogus output")
        Builtins.y2milestone("stdout from command:\n%1", output)
        Builtins.y2milestone("Checking object type for class: %1", objectClass)
        testString = Builtins.sformat("objectClass: %1", objectClass)
        index = Builtins.search(output, testString)
        if index == nil
          Builtins.y2milestone(
            "object of the wrong class.  return false\n%1",
            output
          )
          return false # object of the wrong class
        else
          Builtins.y2milestone(
            "object found and of the correct class.  return true"
          )
          return true # object found and of the correct class
        end
      end
    end

    def AreXADPackagesInstalled
      ret = false

      cmd = "rpm -q novell-xad-dcerpc"
      Builtins.y2milestone(
        "EdirUtils.AreXADPackagesInstalled:executing:%1",
        cmd
      )
      d = Convert.to_integer(SCR.Execute(path(".target.bash"), cmd))
      ret = true if d == 0

      Builtins.y2milestone("EdirUtils.AreXADPackagesInstalled returned:%1", ret)
      ret
    end

    publish :function => :EdirConfigProblemPrompt, :type => "boolean (string)"
    publish :function => :isLDAPdn, :type => "boolean (string)"
    publish :function => :GetLeafNameNDS, :type => "string (string)"
    publish :function => :GetLeafNameLDAP, :type => "string (string)"
    publish :function => :GetGIDforGroup, :type => "string (string, string, string, string, string, string)"
    publish :function => :DeleteDirectoryObject, :type => "integer (string, string, string, integer, string)"
    publish :function => :DoesDirectoryObjectExist, :type => "boolean (string, string, string, integer, string, string)"
    publish :function => :AreXADPackagesInstalled, :type => "boolean ()"
  end

  EdirUtils = EdirUtilsClass.new
  EdirUtils.main
end
