# encoding: utf-8

# File:	modules/NovellLdapBrowser.ycp
# Package:	novell-common
# Summary:	Novell LDAP Popup
# Authors:	Jiri Suchomel <jsuchome@suse.cz>
#		Michael G. Fritch <mgfritch@novell.com>
#
# $Id: NovellLdapBrowser.ycp,v 1.1.2.46 2005/10/04 20:38:19 mgfritch Exp $
#
require "yast"

module Yast
  class NovellLdapBrowserClass < Module
    def main
      Yast.import "UI"

      textdomain "novell-base"

      Yast.import "Label"
      Yast.import "Ldap"
      Yast.import "NovellUtils"
    end

    #import "Edirectory";

    # result = `ldapsearch -LLL -b <search_context> -s one -H ldaps://<ip-address> -x -D <ldap-admin-name> -w <password> \"<e.g. cn=user>\" 2>&1`
    #
    # ldapsearch -b 'T=MGFRITCH4_TREE' -H 'ldaps://137.65.47.96:636' -x -D 'cn=admin,o=novell' -w novell 'objectclass=ndsContainer*' |grep 'dn:'

    # Bind to an LDAP server
    #
    # @param [String] server LDAP server DNS or IP address
    # @param [Fixnum] port LDAP port
    # @param [Boolean] useTLS use TLS
    # @param [String] adminDN admin DN in LDAP (comma) format (i.e. 'cn=admin,o=novell')
    # @param [String] password admin password
    # @return [Boolean] true on success
    def bind2(server, port, useTLS, adminDN, password)
      # this code does not seem to work with a secure connection with sles 11.  A new bind has been written below this method.

      # initialization: Execute (.ldap,$["hostname": <host>, "port": <port>, "use_tls": "no"|"yes"|"try" ] )
      args = {
        "hostname" => server,
        "port"     => port,
        #		"version"	: Ldap::ldap_v2 ? 2 : 3, //FIXME: is this needed???
        "use_tls"  => useTLS ? "yes" : "no"
      } # valid parameters: "yes", "no", "try"
      init = Convert.to_boolean(SCR.Execute(path(".ldap"), args))
      if init == nil || !init
        Ldap.LDAPErrorMessage("init", Ldap.LDAPError) # Display popup error message
        return false
      end

      # bind: Execute(.ldap.bind, $[ "bind_dn": binddn, "bindpw": bindpw] )
      # for anonymous acess, call bind with empty map
      args = { "bind_dn" => adminDN, "bind_pw" => password }
      bound = Convert.to_boolean(SCR.Execute(path(".ldap.bind"), args))
      if bound == nil || !bound
        Ldap.LDAPErrorMessage("bind", Ldap.LDAPError) # Display popup error message
        return false
      end

      true
    end

    # Bind to an LDAP server
    #
    # @param [String] server LDAP server DNS or IP address
    # @param [Fixnum] port LDAP port (currently not used)
    # @param [Boolean] useTLS use TLS
    # @param [String] adminDN admin DN in LDAP (comma) format (i.e. 'cn=admin,o=novell')
    # @param [String] password admin password
    # @return [Boolean] true on success
    def bind(server, port, useTLS, adminDN, password)
      Ldap.server = server
      Builtins.y2milestone("NovellLdapBrowser.bind:server is %1", Ldap.server)
      Ldap.bind_dn = adminDN
      Builtins.y2milestone("NovellLdapBrowser.bind:bind_dn is %1", Ldap.bind_dn)
      Ldap.bind_pass = password
      #    y2milestone("NovellLdapBrowser.bind:bind_pass is %1", Ldap::bind_pass);
      Ldap.ldap_tls = useTLS
      Builtins.y2milestone(
        "NovellLdapBrowser.bind:ldap_tls = %1",
        Ldap.ldap_tls
      )

      #    string error = Ldap::LDAPInit ();
      #    string error = Ldap::LDAPInitWithTLSCheck ($[]);  // for SLES 11

      args = {
        "hostname"     => server,
        "use_tls"      => useTLS ? "yes" : "no",
        "require_cert" => "never"
      }

      error_map = Ldap.LDAPInitArgs(args)
      error = Ops.get_string(error_map, "msg", "")

      if error != ""
        Ldap.LDAPErrorMessage("init", error)
        return false
      end

      error = Ldap.LDAPBind(Ldap.bind_pass)
      if error != ""
        Ldap.LDAPErrorMessage("bind", error)
        return false
      end

      error = Ldap.InitSchema
      if error != ""
        Ldap.LDAPErrorMessage("schema", error)
        return false
      end

      true
    end


    # Unbind from a previous bind to an LDAP server
    #
    # @return [Boolean] true on success
    #global boolean unbind() {
    # unbind: Execute(.ldap.unbind)
    #	boolean unbound = (boolean) SCR::Execute(.ldap.unbind);
    #	if (unbound == nil || !unbound) {
    #//		Ldap::LDAPErrorMessage ("unbind", Ldap::LDAPError ()); // FIXME: Ldap::LDAPErrorMessage does not contain an unbind error message.  Do we need to create one of our own?
    #		return false;
    #	}

    #	return true;
    #}


    # Popup for browsing LDAP tree and selecting the DN
    # WARNING: we expect that LDAP connection is already correctly initialized !
    #
    # @param [String] baseDN the starting point (base of tree); if empty string is given, the search for available bases will be done automatically
    # @param [String] searchFilter ldapsearch searchFilter  If 'searchFilter == ""' then 'searchFilter = "objectclass=*"'
    # @param [String] title translatable title to dispaly on the Popup dialog
    # @return DN of selected item, empty string when canceled
    def BrowseTree(baseDN, searchFilter, title)
      # get display mode
      display_info = UI.GetDisplayInfo
      textmode = Ops.get_boolean(display_info, "TextMode", true)

      # map of already read subtrees
      dns = {}
      # selected DN (return value)
      current_dn = ""

      contents = HBox(
        VSpacing(20),
        VBox(
          HSpacing(70),
          VSpacing(0.2),
          HBox(
            HSpacing(),
            ReplacePoint(Id(:reptree), Tree(Id(:tree), title, [])),
            ReplacePoint(Id(:repbuttons), Empty()),
            HSpacing()
          ),
          HBox(
            HSpacing(1.5),
            PushButton(Id(:ok), Opt(:key_F10), Label.OKButton),
            PushButton(Id(:cancel), Opt(:key_F9), Label.CancelButton),
            # pushbutton label
            textmode ?
              # button label
              Right(PushButton(Id(:open), Opt(:key_F6), _("&Open"))) :
              Empty(),
            HSpacing(1.5)
          ),
          VSpacing(0.2)
        )
      )

      UI.OpenDialog(Opt(:decorated), contents)
      items = []
      out = Convert.convert(
        SCR.Read(
          path(".ldap.search"),
          {
            "base_dn"      => baseDN,
            "filter"       => searchFilter,
            "scope"        => 1,
            "dn_only"      => true,
            "not_found_ok" => true
          }
        ),
        :from => "any",
        :to   => "list <string>"
      )
      out = Builtins.sort(out)
      items = Builtins.maplist(out) do |dn|
        Ops.set(dns, dn, false)
        Item(dn, false, [])
      end if Ops.greater_than(
        Builtins.size(out),
        0
      )

      if Ops.greater_than(Builtins.size(items), 0)
        UI.ReplaceWidget(
          Id(:reptree),
          textmode ?
            Tree(Id(:tree), title, items) :
            Tree(Id(:tree), Opt(:notify), title, items)
        )
      elsif baseDN == ""
        bases = Convert.to_list(
          SCR.Read(
            path(".ldap.search"),
            { "base_dn" => "", "scope" => 0, "attrs" => ["namingcontexts"] }
          )
        )
        if Ops.greater_than(Builtins.size(bases), 0)
          items = Builtins.maplist(
            Ops.get_list(bases, [0, "namingcontexts"], [])
          ) { |dn| Item(dn, false, []) }
        end
        if Ops.greater_than(Builtins.size(items), 0)
          UI.ReplaceWidget(
            Id(:reptree),
            textmode ?
              Tree(Id(:tree), title, items) :
              Tree(Id(:tree), Opt(:notify), title, items)
          )
        end
      end

      UI.SetFocus(Id(:tree)) if textmode

      subdns = []
      open_items = {}

      update_items = lambda do |its|
        its = deep_copy(its)
        Builtins.maplist(its) do |it|
          dn = Ops.get_string(it, 0, "")
          next Item(dn, true, Builtins.maplist(subdns) { |k| Item(k, false, []) }) if dn == current_dn
          last = Ops.subtract(Builtins.size(it), 1)
          next deep_copy(it) if Builtins.size(Ops.get_list(it, last, [])) == 0
          # `OpenItems doesn't work in ncurses...
          open = Builtins.haskey(open_items, dn) && !textmode
          Item(dn, open, update_items.call(Ops.get_list(it, last, [])))
        end
      end

      retval = baseDN
      while true
        ret = UI.UserInput
        if ret == :tree || ret == :open
          current_dn = Convert.to_string(
            UI.QueryWidget(Id(:tree), :CurrentItem)
          )
          if !Ops.get(dns, current_dn, false)
            subdns = Convert.convert(
              SCR.Read(
                path(".ldap.search"),
                {
                  "base_dn"      => current_dn,
                  "filter"       => searchFilter,
                  "scope"        => 1,
                  "dn_only"      => true,
                  "not_found_ok" => true
                }
              ),
              :from => "any",
              :to   => "list <string>"
            )
            Ops.set(dns, current_dn, true)
            subdns = Builtins.sort(subdns)
            if Ops.greater_than(Builtins.size(subdns), 0)
              open_items = Convert.to_map(UI.QueryWidget(:tree, :OpenItems))
              items = update_items.call(items)
              UI.ReplaceWidget(
                Id(:reptree),
                textmode ?
                  Tree(Id(:tree), title, items) :
                  Tree(Id(:tree), Opt(:notify), title, items)
              )
              UI.ChangeWidget(Id(:tree), :CurrentItem, current_dn)
              open_items = {}
            end
          end
          UI.SetFocus(Id(:tree)) if textmode
        end
        if ret == :cancel
          retval = ""
          break
        end
        if ret == :ok
          dn = Convert.to_string(UI.QueryWidget(Id(:tree), :CurrentItem))
          if dn != nil
            retval = dn
          else
            retval = current_dn
          end
          break
        end
      end
      UI.CloseDialog
      retval
    end

    # Bind to an LDAP server and display a popup for browsing LDAP tree and selecting the DN
    #
    # @param [String] server LDAP server DNS or IP address
    # @param [Fixnum] port LDAP port
    # @param [Boolean] useTLS use TLS
    # @param [String] adminDN admin DN in LDAP (comma) format (i.e. 'cn=admin,o=novell')
    # @param [String] password admin password
    # @param [String] baseDN the starting point (base of tree); if empty string is given, the search for available bases will be done automatically
    # @param [String] searchFilter ldapsearch searchFilter  If 'searchFilter == ""' then 'searchFilter = "objectclass=*"'
    # @param [String] title translatable title to dispaly on the Popup dialog
    # @return DN of selected item, empty string when canceled
    def LdapBrowser(server, port, useTLS, adminDN, password, baseDN, searchFilter, title)
      ret = ""
      server_and_port = Ops.add(Ops.add("ldap://", server), ":1389")

      bound = false

      if !Ldap.ldap_initialized
        if NovellUtils.ServerIsDsfwServer(server)
          Builtins.y2milestone(
            "NovellLdapBrowser.LdapBrowser:called with server:port '%1', useTLS '%2', adminDN '%3', password '%4', baseDN '%5', searchFilter '%6', title '%7'",
            server_and_port,
            useTLS,
            adminDN,
            "****",
            baseDN,
            searchFilter,
            title
          )
          bound = bind(server_and_port, port, useTLS, adminDN, password)
        else
          Builtins.y2milestone(
            "NovellLdapBrowser.LdapBrowser:called with server '%1', port '%2', useTLS '%3', adminDN '%4', password '%5', baseDN '%6', searchFilter '%7', title '%8'",
            server,
            port,
            useTLS,
            adminDN,
            "****",
            baseDN,
            searchFilter,
            title
          )

          bound = bind(server, port, useTLS, adminDN, password)
        end
        return "" if bound == false
      end

      ret = BrowseTree(baseDN, searchFilter, title)

      # not needed anymore --kills ldap in sles 11
      #	unbind();

      Builtins.y2milestone(
        "NovellLdapBrowser::LdapBrowser(%1, %2, %3, %4, *****, %5, %6, %7) returned: %8",
        server,
        port,
        useTLS,
        adminDN,
        baseDN,
        searchFilter,
        title,
        ret
      )
      ret
    end

    publish :function => :bind2, :type => "boolean (string, integer, boolean, string, string)"
    publish :function => :bind, :type => "boolean (string, integer, boolean, string, string)"
    publish :function => :BrowseTree, :type => "string (string, string, string)"
    publish :function => :LdapBrowser, :type => "string (string, integer, boolean, string, string, string, string, string)"
  end

  NovellLdapBrowser = NovellLdapBrowserClass.new
  NovellLdapBrowser.main
end
