# encoding: utf-8

# -------------------------------------------------------------------------------
# Copyright 2023 - 2024 Open Text.
#
# The only warranties for products and services of Open Text and its
# affiliates and licensors (“Open Text”) are as may be set forth in the
# express warranty statements accompanying such products and services.
# Nothing herein should be construed as constituting an additional
# warranty. Open Text shall not be liable for technical or editorial
# errors or omissions contained herein. The information contained herein
# is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains
# confidential information and a valid license is required for possession,
# use or copying. If this work is provided to the U.S. Government,
# consistent with FAR 12.211 and 12.212, Commercial Computer Software,
# Computer Software Documentation, and Technical Data for Commercial Items
# are licensed to the U.S. Government under vendor's standard commercial
# license.
# -------------------------------------------------------------------------------
# -------------------------------------------------------------------------------
# (C) Copyright 2019 - 2023 Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates
# and licensors (“Micro Focus”) are as may be set forth in the express warranty
# statements accompanying such products and services. Nothing herein should be
# construed as constituting an additional warranty. Micro Focus shall not be
# liable for technical or editorial errors or omissions contained herein. The
# information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential
# information and a valid license is required for possession, use or copying. If
# this work is provided to the U.S. Government, consistent with FAR 12.211 and
# 12.212, Commercial Computer Software, Computer Software Documentation, and
# Technical Data for Commercial Items  are licensed to the U.S. Government under
# vendor's standard commercial license.
# -------------------------------------------------------------------------------

# File:	modules/NovellUtils.ycp
# Package:	novell-common
# Summary:	Novell utility routines
# Authors:	Stephen T. Bunch <sbunch@novell.com>
#
# $Id: NovellUtils.ycp,v 1.1.2.46 2005/10/04 20:38:19 mclark Exp $
require "yast"
require "fileutils"
require "y2firewall/firewalld"

module Yast
  class NovellUtilsClass < Module
    def main
      Yast.import "Pkg"
      Yast.import "UI"

      textdomain "novell-base"

      Yast.import "Map"
      Yast.import "IP"
      Yast.import "Popup"
      Yast.import "Report"
      Yast.import "NetworkInterfaces"
      Yast.import "Address"
      Yast.import "Mode"
      Yast.import "Stage"
      Yast.import "Label"
      Yast.import "String"
      Yast.import "Product"
      Yast.import "Progress"
      Yast.import "ValidationHelpers"
      Yast.import "NovellAddonInstall"
      Yast.import "NovellLDAP"
      Yast.import "NovellPatterns"
      Yast.import "FileUtils"
      Yast.import "Arch"
      Yast.import "Bootloader"

      @CifsPatternInstalled = false
      @is_cis_agent_enabled = false
      @edir_quick_install = false
      @passwordMessage = _("Replace this text with the real password")
      @noConfigMessage = _(
        "This product does not require configuration for upgrade."
      )
      @dsfwUpgradeMsg = _("Error calling DSFW upgrade script.")
      @dsfwUpgradeMsg_2 = _(
        "The DNS SRV records are not correct. This may result in failure of further ADC\n" +
          "installations. For details on resolving this issue, refer the Troubleshooting\n" +
          "chapter in the DSfW Administration guide."
      )
      @dsfwMixedModeUpgardeMsg = _("All Domain Controllers must have the same version of OES.\nAfter upgrading this Domain Controller, ensure to upgrade\nall the other Domain Controllers in this domain and in\nother domains to the same version of OES."
      )
      @removeServiceProxyMsg = _("It is recommended to use a Common Proxy User for the OES services.\nUsing Service-specific Proxy Users will not be supported from the next OES release.")
      @adminPasswordExpireMsg = _("Online migration is paused as the administrator password has expired.\nReset the password and click 'OK' to continue with the process.")
      @domainPasswordExpireMsg = _("Online migration is aborted as the domain administrator password has expired.\nReset the password and click 'OK' to restart the process.")
      @channelUpgrade_error = _("Configuration is paused due to an authentication failure. To continue with the configuration, click 'OK'.")
      @proxyScriptPath = "/opt/novell/proxymgmt/bin/"
      @channelUpgrade_abort = false
      @channelUpgrade = false

      @scr_conf_path = path(".sysconfig.novell-common")
    end

    def ReadOption(optname)
      Builtins.y2milestone("NovellUtils::ReadOptions")
      optvalue = Convert.to_string(
        SCR.Read(Ops.add(@scr_conf_path, Builtins.topath(optname)))
      )

      optvalue = "" if optvalue == nil

      optvalue
    end

    #1119141
    # If Multiple Pem file are present
    # Delete the multiple file
    def DeleteMultiPemFile
      multipemPath = [ "/etc/etc/pki/trust/anchors", "/etc/pki/trust/anchors/anchors", "/etc/pki/trust/anchors", "/var/lib/ca-certificates/openssl", "/var/lib/ca-certificates/pem", "/etc/pki/trust", "/etc/ssl/certs.rpmsave" ]
      multipemPath.each do |f|
        multipemFile = Ops.add(f, "/registration-server.pem")
        if FileUtils::Exists("#{multipemFile}")
          Builtins.y2milestone("Deleting #{multipemFile} because of multiple registration-server.pem available")
          SCR.Execute(
            path(".target.bash"),
            Builtins.sformat("rm #{multipemFile}")
          )
        end
      end

      SCR.Execute(path(".target.bash"), "/usr/sbin/update-ca-certificates")
    end

    def ExtendSchema(sch_file)
      return_value = true
      if FileUtils.Exists(sch_file)
        Yast.import "Edirectory"
        ndssch_file = "/opt/novell/eDirectory/bin/ndssch"
        schemaLogPath = "/var/opt/novell/eDirectory/log/oes_schema.log"
        password = Ops.get_string(
          NovellLDAP.Settings,
          "CONFIG_LDAP_ADMIN_PASSWORD",
          ""
        )
        local_server = Ops.get(NovellLDAP.GetLocalServers(nil), 0, {})
        local_host = Ops.get_string(local_server, "CONFIG_LDAP_SERVER_IP", "")
        local_port = Ops.get_integer(local_server, "CONFIG_LDAP_SECURE_PORT", 636)

        replica_server = Ops.get_string(Edirectory.EdirSettings, "CONFIG_EDIR_REPLICA_SERVER", "")
        if replica_server == "" || replica_server == nil
          replica_server = SCR.Read(path(".sysconfig.edirectory.CONFIG_EDIR_REPLICA_SERVER"))
        end
        remote_server = NovellLDAP.GetAddressServer(
          replica_server,
          nil
        )
        remote_host = Ops.get_string(remote_server, "CONFIG_LDAP_SERVER_IP", "")
        remote_port = Ops.get_integer(
          remote_server,
          "CONFIG_LDAP_SECURE_PORT",
          636
        )

        remote_host = "" if remote_host == local_host # don't fall back to the same address.

        admin = NovellUtils.Ldap2Nds(
          Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_ADMIN_CONTEXT", "")
        )
        if Convert.to_string(SCR.Read(path(".sysconfig.novell.xad.SERVICE_CONFIGURED"))) == "yes"
          admin = NovellUtils.Ldap2Nds(
            Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_ADMIN_CONTEXT", "")
          )
          if Convert.to_string(SCR.Read(path(".sysconfig.edirectory.CONFIG_EDIR_TREE_TYPE"))) == "existing"
            admin = NovellUtils.Ldap2Nds(
              Ops.get_string(NovellLDAP.Settings,"XAD_TREE_ADMIN_CONTEXT", "")
            )
            password = Ops.get_string(
              NovellLDAP.Settings,
              "XAD_TREE_ADMIN_PASSWORD",
              ""
            )
            if password == "" || password == nil
              NovellLDAPUI.XadTreeAdminPasswordDialog
            end
          end
        end

        if password == "" || password == nil
          NovellLDAPUI.AdminPasswordDialog
        end

        cmd = Ops.add(
          Ops.add(
            Ops.add(
              Ops.add(
                Ops.add(
                  Ops.add(
                    Ops.add(
                      Ops.add(Ops.add(ndssch_file, " -h "), local_host),
                      " -F '"
                    ),
                    schemaLogPath
                  ),
                  "' '"
                ),
                admin
              ),
              "'"
            ),
            " "
          ),
          sch_file
        )

        Builtins.y2milestone("NovellUtils :Executing %1",cmd)

        rmap = Convert.to_map(
          SCR.Execute(
            path(".target.bash_output"),
            Ops.add(Ops.add(Ops.add("ADM_PASSWD='", password), "' "), cmd)
          )
        )

        if Ops.get_integer(rmap, "exit", -1) != 0
          if remote_host != ""
            cmd = Ops.add(
              Ops.add(
                Ops.add(
                  Ops.add(
                    Ops.add(
                      Ops.add(
                        Ops.add(
                          Ops.add(Ops.add(ndssch_file, " -h "), remote_host),
                          " -F '"
                        ),
                        schemaLogPath
                      ),
                      "' '"
                    ),
                    admin
                  ),
                  "'"
                ),
                " "
              ),
              sch_file
            )

            Builtins.y2milestone(
              "NovellUtils : Retry of schema extension with fallback IP address:Executing %1",
              cmd
            )

            rmap = Convert.to_map(
              SCR.Execute(
                path(".target.bash_output"),
                Ops.add(Ops.add(Ops.add("ADM_PASSWD='", password), "' "), cmd)
              )
            )
          end

          if Ops.get_integer(rmap, "exit", -1) != 0
            Builtins.y2milestone(
              "NovellUtils : extend password policy:failed extending schema: %1 using file %2",
              Ops.get_integer(rmap, "exit", -1),
              sch_file
            )
            Builtins.y2milestone("Failed to apply the schema file: %1", sch_file)
            return_value = false
          end
        end
      else
        return_value = false
        Builtins.y2milestone("Schema File : %1 does not exist.",sch_file)
      end

      return_value
    end

    # LUM or NCS is configured without local edir
    def IsLumNcsConfigured
      lum_service_configured = Builtins.tolower(Convert.to_string(SCR.Read(path(".sysconfig.novell-lum.SERVICE_CONFIGURED"))))
      ncs_service_configured = Builtins.tolower(Convert.to_string(SCR.Read(path(".sysconfig.ncs.SERVICE_CONFIGURED"))))
      edir_service_configured = Builtins.tolower(Convert.to_string(SCR.Read(path(".sysconfig.edirectory.SERVICE_CONFIGURED"))))

      if (lum_service_configured == "yes" || ncs_service_configured == "yes") && edir_service_configured == "no"
        return true
      else
        return false
      end
    end

    # Update Proxy Credentials for LUM and NCS
    def LumNcsCommonProxyCredUpdate
      local_server = Ops.get(NovellLDAP.GetLocalServers(nil), 0, {})
      sPort = Ops.get_integer(local_server, "CONFIG_LDAP_SECURE_PORT", 636)
      ip = Ops.get_string(local_server, "CONFIG_LDAP_SERVER_IP", "")
      admin_fdn = NovellUtils.Nds2Ldap(
      Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_ADMIN_CONTEXT", ""))
      pwd = Ops.get_string(NovellLDAP.Settings,"CONFIG_LDAP_ADMIN_PASSWORD","" )
      commonProxyName = RetrieveProxyCredentials("username")
      commonProxyPassword = RetrieveProxyCredentials("password")

      if Builtins.tolower(Convert.to_string(SCR.Read(path(".sysconfig.novell-lum.SERVICE_CONFIGURED")))) == "yes"
        cmd = Builtins.sformat(
          "OES_LUM_DATA='%1' /var/lib/novell-lum/lum_update_proxy_cred.sh '%2'",
          commonProxyPassword,
          commonProxyName
        )
        logcmd = Builtins.sformat(
          "/var/lib/novell-lum/lum_update_proxy_cred.sh '%1'",
          commonProxyName
        )
        Builtins.y2milestone(
          "NovellUtils.LumOrNcsCommonProxyUpdate:executing: %1",
          logcmd
        )
        val = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
        exitVal = Ops.get_integer(val, "exit", -1)
        stdout = Ops.get_string(val, "stdout", "")
        Builtins.y2milestone(
          "NovellUtils.LumOrNcsCommonProxyUpdate:returned:%1, with stdout:\n%2",
          val,
          stdout
        )
        if exitVal == 0
          Builtins.y2milestone(
            "NovellUtils.LumOrNcsCommonProxyUpdate: lum_update_proxy_cred passed"
          )
          SCR.Write(path(".sysconfig.novell-lum.CONFIG_LUM_PROXY_USER"),commonProxyName)
          Service.Restart("namcd")
        end
      elsif Builtins.tolower(Convert.to_string(SCR.Read(path(".sysconfig.ncs.SERVICE_CONFIGURED")))) == "yes"
        cmd = Builtins.sformat(
          "OES_NCS_DATA='%1' OES_ADMIN_DATA='%2' /opt/novell/ncs/bin/ncs_proxy_rights_assign '%3' '%4' %5 %6 1",
          commonProxyPassword,
          pwd,
          commonProxyName,
          ip,
          sPort
        )
        logcmd = Builtins.sformat(
          "/opt/novell/ncs/bin/ncs_proxy_rights_assign '%1' '%2' %3 %4 1",
          commonProxyName,
          ip,
          sPort
        )
        Builtins.y2milestone(
          "NovellUtils.LumOrNcsCommonProxyUpdate:executing: %1",
          logcmd
        )
        val = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
        exitVal = Ops.get_integer(val, "exit", -1)
        stdout = Ops.get_string(val, "stdout", "")
        Builtins.y2milestone(
          "NovellUtils.LumOrNcsCommonProxyUpdate:returned:%1, with stdout:\n%2",
          val,
          stdout
        )
        if exitVal == 0
          cmd1 = Builtins.sformat(
            "/opt/novell/ncs/bin/ncs_update_proxy_cred '%1'",
            commonProxyName
          )
          Builtins.y2milestone(
            "NovellUtils.LumOrNcsCommonProxyUpdate:executing: %1",
            cmd1
          )
          val1 = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
          exitVal1 = Ops.get_integer(val, "exit", -1)
          stdout1 = Ops.get_string(val, "stdout", "")
          Builtins.y2milestone(
            "NovellUtils.LumOrNcsCommonProxyUpdate:returned:%1, with stdout:\n%2",
            val1,
            stdout1
          )
          if exitVal1 == 0
             Builtins.y2milestone(
               "NovellUtils.LumOrNcsCommonProxyUpdate: ncs_update_proxy_cred passed"
             )
             SCR.Write(path(".sysconfig.ncs.CONFIG_NCS_PROXY_USER"),commonProxyName)
          end
        else
          Builtins.y2milestone(
            "NovellUtils.LumOrNcsCommonProxyUpdate : NCS has Failed the rights assignment"
          )
        end
      end
    end

    def FindUpgradeIsFrom
      oes_upgrade_from = ""
      option = "UPGRADE_FROM"
      cfg_file = "/mnt/etc/sysconfig/novell/oes-ldap"
      (ver, rel) = Misc.CustomSysconfigRead("VERSION", "", "/mnt/etc/novell-release").split(".")
      oes_upgrade_from = "OES#{ver}"
      oes_upgrade_from += "SP#{rel}" if rel.to_i > 0
      if oes_upgrade_from != ""
        cmd = Builtins.sformat(
          '/usr/bin/sed -i -e \'/^#\?\(\s*%1\s*=\s*\).*/{s//\1"%2"/;:a;n;ba;q}\' -e \'$a%1="%2"\' %3',
          option,
          oes_upgrade_from,
          cfg_file
        )
        rval = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
        Builtins.y2milestone(
          "NovellUtils.FindUpgradeIsFrom: executing cmd: %1  rval: %2",
          cmd,
          rval
        )
      else
        return false
      end
      return true
    end

    # To check upgrade is from
    def UpgradeIsFrom
      upgrade_from = ""
      cfgPath = "/etc/sysconfig/novell/oes-ldap"
      cfgfile_path = Ops.add(
        path(".etc.sysconfig.novell") + path(".value"),
        Builtins.topath(cfgPath)
      )
      Builtins.y2milestone(
        "NovellUtils.get.UPGRADE_FROM:cfgfile_path %1",
        cfgfile_path
      )
      # Read in the global ldap config file settings
      upgrade_from = ReadOptionFromSysconfig(cfgfile_path, "UPGRADE_FROM")
      Builtins.y2milestone(
        "Get UPGRADE_FROM from ldap conf file:%1",
         upgrade_from
      )
      return upgrade_from
    end

    # During upgrade get the sysconfig value for module
    def GetOldSysconfigMap(module_name)
      old_Settings = {}
      upgradeInfo =  {}
      upgrade_from = UpgradeIsFrom()
      if !Builtins.contains(upgrade_from, "OES2023")
        if NovellAddonInstall.IsProductUpgrade(upgrade_from, module_name) || (module_name == "novell-dns" && NovellPatterns.IsPatternInstalled("novell-domain-services"))
          upgradeInfo = NovellAddonInstall.GetProductUpgradeInfo(upgrade_from, module_name)
          old_Settings = NovellAddonInstall.GetSysconfigMap(
            Ops.get_string(upgradeInfo, "config_file", "")
          )
          Builtins.y2milestone("%1: oldsettings: %2", module_name, old_Settings)
        end
      elsif !Builtins.contains(upgrade_from, "OES11") && !Builtins.contains(upgrade_from, "OES2015") && !Builtins.contains(upgrade_from, "OES2018")
        if NovellAddonInstall.IsProductUpgradeOES(module_name) || (module_name == "novell-dns" && NovellPatterns.IsPatternInstalled("novell-domain-services"))
          upgradeInfo = NovellAddonInstall.GetProductUpgradeInfoOES(module_name)
          old_Settings = NovellAddonInstall.GetSysconfigMap(
            Ops.get_string(upgradeInfo, "config_file", "")
          )
          Builtins.y2milestone("%1: oldsettings: %2", module_name, old_Settings)
        end
      end
      deep_copy(old_Settings)
    end

    # To check the system is booted in secureboot or not
    def IsSecureBoot
      secure_boot = Misc::SysconfigRead(path(".sysconfig.bootloader.SECURE_BOOT"),"no")
      Builtins.y2milestone("Value of SECURE_BOOT in bootloader sysconfig file is : %1 ",secure_boot)
      if secure_boot == "yes"
        return true
      else
        return false
      end
    end

    # To import the oes certificate in secureboot
    # First get the OES certificate path
    # Then test the cert is imported or not by mokutil test
    # If not use mokutil import to import the key
    def ImportOesCertificate
      target_dir = "/etc/uefi/certs/"
      files = Convert.to_list(SCR.Read(path(".target.dir"), target_dir))
      Builtins.foreach(files) do |f|
        cert_file = Ops.add(target_dir, f)
        cmd = Builtins.sformat("openssl x509 -noout -subject -in %1 -inform der",cert_file)
        ret = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
        stdout = Ops.get_string(ret, "stdout", "")
        if Builtins.contains(stdout, "Micro Focus")
          rmap = Convert.to_map(SCR.Execute(path(".target.bash_output"),
                Builtins.sformat("/usr/bin/mokutil --test %1",cert_file)))
          if Ops.get_integer(rmap, "exit", -1) == 0
            Builtins.y2milestone("%1 is not enrolled",cert_file)
            cmd = Builtins.sformat("/usr/bin/mokutil --import %1 --root-pw",cert_file)
            Builtins.y2milestone("Executing: %1 to import the OES certificate ",cmd)
            ret = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
            if Ops.get_integer(ret, "exit", -1) == 0
              importCompleteMsg = Builtins.sformat(_( "<p>The Micro Focus Secure Boot certificates are imported.</p>\n" +
                                  "<p>Reboot the system and in the <b>Shim UEFI Key Management</b> screen, press any key (in 10 seconds) to enroll the key\n" +
                                  "and follow the steps to add the certificates in the Machine Owner Keys (MOK) database.\n" +
                                  "For more information on MOK, see <b>'Deploying OES 23.4 in a UEFI Secure Boot Environment'</b>\n" +
                                  "in the <b>OES Installation Guide</b>.</p>"))
              if Mode.autoinst || Mode.autoupgrade
                Popup.anyTimedMessageTypeInternal("", importCompleteMsg, 20, true, 60, 10)
              else
                Popup.anyMessageInternalRich("", importCompleteMsg, 60, 10)
              end
            end
          end
        end
      end
    end

    def ModifyingSfcb

      if UpgradeIsFrom() == "" || UpgradeIsFrom() == nil
        sfcb_option = [ "sslKeyFilePath", "sslCertificateFilePath", "sslCertList", "sslClientTrustStore" ]
        sfcb_option.each do |value|
          value.eql?("sslKeyFilePath") ? option_value = "\\/etc\\/ssl\\/servercerts\\/serverkey.pem" : option_value = "\\/etc\\/ssl\\/servercerts\\/servercert.pem"
        @cmd = Builtins.sformat('/usr/bin/sed -i -e \'/^#\?\(\s*%1\s*:\s*\).*/{s//\1%2/;:a;n;ba;q}\' -e \'$a%1: %2\' /etc/sfcb/sfcb.cfg',value, option_value)
        @rval = Convert.to_integer(SCR.Execute(path(".target.bash"), @cmd))
        Builtins.y2milestone(
          "NovellUtils.ModifyingSfcb:CertificatePaths: executing cmd: %1  rval: %2",
          @cmd,
          @rval
        )
        end
      end

      # searching /opt/novell/lib64/cmpi in /etc/sfcb/sfcb.cfg
      @cmd = "grep  'providerDirs:.*/opt/novell/lib64/cmpi /opt/novell/lib64/sfcb/cmpi' /etc/sfcb/sfcb.cfg > /dev/null"
      @rval = Convert.to_integer(SCR.Execute(path(".target.bash"), @cmd))
      # if not found adding /opt/novell/lib64/cmpi in /etc/sfcb/sfcb.cfg
      if @rval != 0
        @cmd2 = "sed -i 's/providerDirs:.*/& \\/opt\\/novell\\/lib64\\/cmpi \\/opt\\/novell\\/lib64\\/sfcb\\/cmpi/' /etc/sfcb/sfcb.cfg"
        @rval2 = Convert.to_integer(SCR.Execute(path(".target.bash"), @cmd2))
        Builtins.y2milestone(
          "NovellUtils.ModifyingSfcb : executing cmd: %1  rval: %2",
           @cmd2,
           @rval2
        )
      end

      # To change the default values of providerSampleInterval and providerTimeoutInterval variables
      sfcb_option = [ "providerSampleInterval", "providerTimeoutInterval" ]
      sfcb_option.each do |value|
        @cmd = Builtins.sformat("/usr/bin/sed -i 's/\\(#%1:.*\\|%1:.*\\)/%1: 5/' /etc/sfcb/sfcb.cfg",value)
        @rval = Convert.to_integer(SCR.Execute(path(".target.bash"), @cmd))
        Builtins.y2milestone(
          "NovellUtils.ModifyingSfcb: executing cmd: %1  rval: %2",
          @cmd,
          @rval
        )
      end

      # To change the default value of sslNoSSLv3 and sslNoTLSv1 variable so that SSLv3 and TLSv1
      # will be disabled during install/upgrade.
      # To understand the fix visit below link
      # https://stackoverflow.com/a/49852337/15466346
      # Note : TLS1.2 will be enabled by default in sfcb
      sfcb_option = [ "sslNoSSLv3", "sslNoTLSv1", "sslNoTLSv1_1" ]
      option_value = true
      sfcb_option.each do |value|
        if value.eql?("sslNoTLSv1_1")
          option_value = false
        end
        @cmd = Builtins.sformat('/usr/bin/sed -i -e \'/^#\?\(\s*%1\s*:\s*\).*/{s//\1%2/;:a;n;ba;q}\' -e \'$a%1: %2\' /etc/sfcb/sfcb.cfg',value, option_value)
        @rval = Convert.to_integer(SCR.Execute(path(".target.bash"), @cmd))
        Builtins.y2milestone(
          "NovellUtils.ModifyingSfcb: executing cmd: %1  rval: %2",
          @cmd,
          @rval
        )
      end
    end

    #1161386
    #NDSPasswordCheck is used to check whether the password is expired or not
    def NDSPasswordCheck(adminPassword)
      ldap_admin_context = Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_ADMIN_CONTEXT", "")
      Builtins.y2milestone(
        "NovellUtils.GetAdminUser:admin user from ldap map:%1",
        ldap_admin_context
      )
      if ldap_admin_context == nil || ldap_admin_context == ""
        cfgPath = "/etc/sysconfig/novell/oes-ldap"
        cfgfile_path = Ops.add(
          path(".etc.sysconfig.novell") + path(".value"),
          Builtins.topath(cfgPath)
        )
        Builtins.y2milestone(
          "NovellUtils.getAdminUser:cfgfile_path %1",
          cfgfile_path
        )

        # Read in the global ldap config file settings
        ldap_admin_context = ReadOptionFromSysconfig(cfgfile_path, "CONFIG_LDAP_ADMIN_CONTEXT")
        Builtins.y2milestone(
          "GetAdminUser:admin user from ldap conf file:%1",
          ldap_admin_context
        )
      end

      ldap_admin_context = Ldap2Nds(ldap_admin_context)
      cmd = Builtins.sformat("/opt/novell/eDirectory/bin/ndslogin %1",ldap_admin_context)
      Builtins.y2milestone("Command to Execute : ndslogin %1 ",ldap_admin_context)
      ret = Convert.to_map(
        SCR.Execute(path(".target.bash_output"),
        Ops.add(Ops.add(Ops.add("ADM_PASSWD='", adminPassword), "' "), cmd))
      )
      Builtins.y2milestone("NovellUtils ndslogin :  Return value is %1",ret)
      exitValue = Ops.get_integer(ret, "exit", -1)
      if exitValue != 0
        stdout = Ops.get_string(ret, "stdout", "")
        regx = /.*\((.*?)\)/
        stdout = stdout.slice(regx, 1)
        Builtins.y2milestone("NovellUtils ndslogin : Error code for ndslogin is %1 ",stdout)
         if stdout == "-223"
           Report.Error(@adminPasswordExpireMsg)
         end
        return stdout
      end
     return exitValue
    end

    # Read a setting (option) from a specified config file
    # Removes any extra quotes around values and avoids passing a nil value
    #
    # @param [Yast::Path] configpath scr path to the config file
    # @param [String] optname string key of the setting(option) we are reading
    # @return [String] value of the specfied setting
    def ReadOptionFromSysconfig(configpath, optname)
      optvalue = Convert.to_string(
        SCR.Read(Ops.add(configpath, Builtins.topath(optname)))
      )

      if optvalue == nil
        optvalue = "" # avoid nil values
      elsif Builtins.regexpmatch(optvalue, "'.*'")
        # remove the single quotes
        tmpoptvalue = Builtins.regexpsub(optvalue, "'(.*)'", "\\1")
        optvalue = tmpoptvalue if optvalue != nil || optvalue != ""
      elsif Builtins.regexpmatch(optvalue, "\".*\"")
        # remove the double quotes
        tmpoptvalue = Builtins.regexpsub(optvalue, "\"(.*)\"", "\\1")
        optvalue = tmpoptvalue if optvalue != nil || optvalue != ""
      end

      optvalue
    end


    #***********************************************************
    # String manipulation routines
    #***********************************************************

    # Strip leading "stripChars"
    # @param string to modify
    # @param string chars to strip
    # @return [String]
    def StripLeading(str, stripChars)
      while Ops.greater_or_equal(Builtins.size(str), 1)
        check = Builtins.substring(str, 0, 1)
        if Builtins.findfirstof(stripChars, check) != nil
          str = Builtins.substring(str, 1)
        else
          break
        end
      end
      str
    end

    # Strip trailing "stripChars"
    # @param string to modify
    # @param string chars to strip
    # @return [String]
    def StripTrailing(str, stripChars)
      while Ops.greater_or_equal(Builtins.size(str), 1)
        check = Builtins.substring(str, Ops.subtract(Builtins.size(str), 1), 1)
        if Builtins.findfirstof(stripChars, check) != nil
          str = Builtins.substring(str, 0, Ops.subtract(Builtins.size(str), 1))
        else
          break
        end
      end
      str
    end

    # Strip leading and trailing "stripChars"
    # @param string to modify
    # @param string chars to strip
    # @return [String]
    def Strip(str, stripChars)
      str = StripLeading(str, stripChars)
      str = StripTrailing(str, stripChars)
      str
    end

    # Clean up a string - remove \n, \r, and spaces.
    # @param string
    # @return [String]
    def CleanUpString(str)
      return str if str == nil || str == ""

      workStr = Strip(str, " \n\t ")

      # Since the strip command above does not seem to remove LF chars...
      idx = Builtins.search(workStr, "\n")
      workStr = Builtins.substring(workStr, 0, idx) if idx != nil

      workStr
    end

    # If a line returned by ldapsearch is over a certain length, ldapsearch breaks the line
    # up with a LF Space before continuing.  This method combines the pieces into one line.
    #
    # For Example, this:
    #
    # dn: cn=lnclust,ou=LINUXCLUSTER,ou=DATACENTER,ou=CSQA,ou=BLR,o=SP3REL,l=KAR,c=I
    #  N
    #
    # is converted to this:
    #
    # dn: cn=lnclust,ou=LINUXCLUSTER,ou=DATACENTER,ou=CSQA,ou=BLR,o=SP3REL,l=KAR,c=IN
    #
    # @param string
    # @return [String]
    def CombineLdapSearchReturn(str)
      Builtins.y2milestone(
        "NovellUtils.CombineLdapSearchReturn:string on entry '%1'",
        str
      )
      return "" if str == nil || str == ""

      workstr = str

      workstr = StripLeading(workstr, "\n")
      workstr = StripTrailing(workstr, "\n")

      idx = Builtins.search(workstr, "\n")
      if idx != nil
        cnt = 1
        partslist = Builtins.splitstring(workstr, "\n")
        Builtins.foreach(partslist) do |part|
          if cnt == 1
            workstr = part
          else
            workstr = Ops.add(
              workstr,
              Builtins.substring(part, 1, Builtins.size(part))
            )
          end # get rid of the space after the LF
          cnt = Ops.add(cnt, 1)
        end
      end
      Builtins.y2milestone(
        "NovellUtils.CombineLdapSearchReturn:string on exit '%1'",
        workstr
      )
      workstr
    end


    #***********************************************************
    # Network routines
    #***********************************************************

    #  Function:  map GetAllStaticInterfaces()
    #
    #  Returns a map of all network interfaces with
    #  static IP addresses
    #
    #  The returned map is structued as follows:
    #
    #  $[IP_Address:Hardware_Address]
    #
    # @return [Hash]
    def GetAllStaticInterfaces
      deviceActive = true

      #read current settings for network
      NetworkInterfaces.CleanCacheRead

      #generate the map: static IP -> device
      ip2device = {}
      devs = NetworkInterfaces.Locate("BOOTPROTO", "static")
      Builtins.y2milestone(
        "NovellUtils.GetAllStaticInterfaces:devices are: %1",
        devs
      )
      Builtins.foreach(devs) do |dev|
        deviceActive = NetworkInterfaces.IsConnected(dev)
        Builtins.y2milestone(
          "NovellUtils.GetAllStaticInterfaces:current device is: %1, active status is %2",
          dev,
          deviceActive
        )
        if deviceActive # Don't use interfaces that are down (bad for ndsconfig).
          ip = NetworkInterfaces.GetValue(dev, "IPADDR")
          Builtins.y2milestone(
            "NovellUtils.GetAllStaticInterfaces:ip is '%1'",
            ip
          )
          if IP.Check4(ip)
            if ip != nil && ip != ""
              # vmWare returns a static ip of "0.0.0.0", so we must skip this one
              Ops.set(ip2device, ip, dev) if ip != nil && ip != "" && ip  != "0.0.0.0" && ip != "127.0.0.1"
            end
          end
        end
      end

      Builtins.y2milestone(
        "NovellUtils.GetAllStaticInterfaces:mapping of IP addresses and network devices: %1",
        ip2device
      )
      deep_copy(ip2device)
    end

    # Function: global string GetIPAddr()
    #
    # This function gets the IP address of the 1st
    # interface with a static address
    #
    # If there are no such addresses then return empty string
    #
    # @return [String] IP address
    def GetIPAddr
      ip = ""
      staticInterfaces = GetAllStaticInterfaces()

      addrList = Map.Keys(staticInterfaces)
      count = Builtins.size(addrList)
      i = 0
      while Ops.less_than(i, count)
        ip = Ops.get_string(addrList, 0, "")
        if ip != ""
          # We use Address here rather than calling
          # NovellValidators to avoid cyclic dependency
          # between NovellUtils and NovellValidators
          if Address.Check4(ip)
            Builtins.y2debug("First static IP address %1", ip)
            return ip
          end
        end
        i = Ops.add(i, 1)
      end
      Builtins.y2milestone("First static IP address %1", ip)
      ""
    end


    # Returns a list of all static IP addresses found on the system
    # @return [Array]
    def GetAllStaticIPAddrs
      staticInterfaces = GetAllStaticInterfaces()
      addrList = Map.Keys(staticInterfaces)

      deep_copy(addrList)
    end

    # Function: MapIPAddrToDevice(string ip)
    #
    # Given an IP address return the device to which
    # the address is bound
    #
    # @param string IP address
    # @return [String] device
    def MapIPAddrToDevice(ip)
      cmd = Builtins.sformat(
        "ip -family inet -oneline addr show to %1 | cut -d \" \" -f2",
        ip
      )
      ret = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
      if Ops.get_integer(ret, "exit", -1) == 0
        device = Ops.get_string(ret, "stdout", "")
        device = Strip(device, " \t\n")
        Builtins.y2milestone(
          "The ip address %1 mapped to the device \"%2\"",
          ip,
          device
        )
        return device
      else
        return ""
      end
    end

    # Return the hostname of the local machine
    # @return [String] host name
    def GetHostname
      rmap = Convert.to_map(
        SCR.Execute(path(".target.bash_output"), "hostname")
      )
      if Ops.get_integer(rmap, "exit", -1) == 0
        hostname = Ops.get_string(rmap, "stdout", "")
        hostname = Strip(hostname, " \n\t ")

        # Since the strip command above does not seem to remove LF chars...
        idx = Builtins.search(hostname, "\n")
        hostname = Builtins.substring(hostname, 0, idx) if idx != nil

        return hostname
      else
        return ""
      end
    end

    # Return the long hostname (fqdn) of the local machine
    # from SLES defined api if it does not return correct value
    # call hostname -f
    def GetHostnameFQDN
      Yast.import "Hostname"
      Yast.import "DNS"
      longHostname = ""
      dnsHostname = Yast::DNS.hostname
      currentDomain = Hostname.CurrentDomain
      Builtins.y2debug("Value of dnsHostname is %1 and currentDomain is %2", dnsHostname, currentDomain)
      if dnsHostname == "" || currentDomain == "" || Builtins.contains(dnsHostname, currentDomain)
        longHostname = GetDNSname()
      else
        longHostname = Yast::Hostname.MergeFQ(dnsHostname, currentDomain)
      end
      Builtins.y2milestone("NovellUtils::GetHostnameFQDN returned: %1", longHostname)
      return longHostname
    end

    # Return the domain name of the local machine
    # @return [String] domain name
    def GetDomainName
      rmap = Convert.to_map(
        SCR.Execute(path(".target.bash_output"), "hostname -d")
      )
      if Ops.get_integer(rmap, "exit", -1) == 0
        domain = Ops.get_string(rmap, "stdout", "")
        domain = Strip(domain, " \n\t ")

        # Since the strip command above does not seem to remove LF chars...
        idx = Builtins.search(domain, "\n")
        domain = Builtins.substring(domain, 0, idx) if idx != nil

        return domain
      else
        return ""
      end
    end


    # Return the DNS name of the local machine
    # It seems that this is the DNS name of eth0
    # in the case of multiple NICs.
    # @return [String] DNS name
    def GetDNSname
      rmap = Convert.to_map(
        SCR.Execute(path(".target.bash_output"), "hostname -f")
      )
      if Ops.get_integer(rmap, "exit", -1) == 0
        hostname = Ops.get_string(rmap, "stdout", "")
        hostname = Strip(hostname, " \n\t ")

        # Since the strip command above does not seem to remove LF chars...
        idx = Builtins.search(hostname, "\n")
        hostname = Builtins.substring(hostname, 0, idx) if idx != nil

        return hostname
      else
        return ""
      end
    end


    # Check to see if the eDirectory daemon is running
    # FIXME: this method should be moved into the Edirectory module!
    # @return [Boolean]
    def IseDirRunning
      running = false

      cmd = "ps -ef | grep ndsd | grep -v grep | wc -l | tr -d \"\\n\""
      Builtins.y2milestone("Executing: %1", cmd)
      ret = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
      if Ops.get_integer(ret, "exit", -1) == 0
        output = Ops.get_string(ret, "stdout", "-1")
        if Ops.greater_than(Builtins.tointeger(output), 0)
          # eDirectory is RUNNING
          running = true
        end
      end
      Builtins.y2milestone("NovellUtils::IseDirRunning() returned: %1", running)
      running
    end


    # See if eDirectory is installed locally(or will be) on this machine and is running.
    # FIXME: this method should be moved into the Edirectory module!
    def IsEdirectoryInstalledLocally
      ret = false
      if Ops.get(NovellAddonInstall.products_to_configure, "edirectory", true)
        ret = true
        Builtins.y2milestone(
          "NovellUtils::IsEdirectoryInstalledLocally() - eDirectory is selected for configuration"
        )
      else
        Builtins.y2warning(
          "NovellUtils::IsEdirectoryInstalledLocally() - eDirectory is NOT selected for configuration"
        )
        ret = IseDirRunning()
      end

      Builtins.y2milestone(
        "NovellUtils::IsEdirectoryInstalledLocally() returned: %1",
        ret
      )
      ret
    end


    # Determine what the default and fallback IP addresses should be for a product.
    # @param sting IP address returned by the ldap dialogs
    # @return [Hash]
    def GetDefaultAndFallbackIPAddresses(ldapIP)
      results = {}

      # Start with a default for the default.
      Ops.set(results, "CONFIG_DEFAULT_IP", ldapIP)
      Ops.set(results, "CONFIG_FALLBACK_IP", "")

      # Decide what the default ldap address should be. If we are joining a tree, but edir is local
      # then the default will be the local IP.
      eDirLocal = IsEdirectoryInstalledLocally()
      localIP = GetIPAddr()
      ldapRemote = true

      ldapRemote = false if ldapIP == localIP

      if eDirLocal == true
        if ldapRemote == true
          Ops.set(results, "CONFIG_DEFAULT_IP", localIP)
          Ops.set(results, "CONFIG_FALLBACK_IP", ldapIP)
        else
          Ops.set(results, "CONFIG_DEFAULT_IP", localIP)
          Ops.set(results, "CONFIG_FALLBACK_IP", "")
        end
      else
        Ops.set(results, "CONFIG_DEFAULT_IP", ldapIP)
        Ops.set(results, "CONFIG_FALLBACK_IP", "")
      end

      Builtins.y2milestone("NovellUtils:eDirLocal: %1", eDirLocal)
      Builtins.y2milestone("NovellUtils:ldapRemote: %1", ldapRemote)
      Builtins.y2milestone("NovellUtils:localIP: %1", localIP)
      Builtins.y2milestone("NovellUtils:ldapIP: %1", ldapIP)
      Builtins.y2milestone("NovellUtils:results: %1", results)

      deep_copy(results)
    end


    # This function will either open or close a port
    # in the firewall
    #
    # Returns true on success
    #
    # @param [Fixnum] port to modify
    # @param [Fixnum] port type - `tcp, `udp, or `both
    # @param boolean status to set -- either true or false
    #
    # @return [Boolean] true on success
    def SetFirewallPort(port, port_type, enable)
      firewalld = Y2Firewall::Firewalld.instance
      Builtins.y2milestone(
        "Setting firewall port %1 (%2) to %3",
        port,
        port_type,
        enable
      )
      retVal = false
      port_types = []
      if port_type == :tcp || port_type == :both
        port_types = Builtins.add(port_types, "tcp")
      end
      if port_type == :udp || port_type == :both
        port_types = Builtins.add(port_types, "udp")
      end
      progress_orig = Progress.set(false)
      read_fw_config = false
      if Mode.autoinst
        Mode.SetMode("installation")
        read_fw_config = firewalld.read
        Mode.SetMode("autoinstallation")
      else
        read_fw_config = firewalld.read
      end
      Builtins.y2milestone("Reading firewall configuration returned: %1 ", read_fw_config)
      defaultZone = firewalld.api.default_zone
      Builtins.y2milestone("Default zone in firewall is: %1", defaultZone)
      if read_fw_config
        Builtins.foreach(port_types) do |type|
          port_id = Builtins.tostring(port)
          retVal = firewalld.api.add_port(defaultZone, "#{port_id}/#{type}")
          Builtins.y2milestone(
            "NovellUtils:SetFirewallPort  type: %1  port: %2  return: %3",
            type,
            port_id,
            retVal
          )
        end

        retVal = firewalld.write if retVal
      end
      Progress.set(progress_orig)
      retVal
    end

    # Returns true on success
    def ReloadFirewall
      firewalld = Y2Firewall::Firewalld.instance
      retVal = firewalld.api.reload
      Builtins.y2milestone(
        "NovellUtils:ReloadFirewall return: %1",
        retVal
      )
      retVal
    end

    def GetServerContext(treeName, adminName, adminPassword, ipAddress, ldap_port, ldap_secure_port)
      ret = ""

      cmd = Builtins.sformat(
        "/opt/novell/oes-install/util/ndsGetServerContext.sh  -t %1 -a %2 -s %3 -L %4 -l %5",
        treeName,
        adminName,
        ipAddress,
        ldap_port,
        ldap_secure_port
      )
      Builtins.y2milestone("NovellUtils.GetServerContext:executing:%1", cmd)

      cmd = Ops.add(Ops.add(Ops.add("ADM_PASSWD=", adminPassword), " "), cmd)

      val = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
      exit = Ops.get_integer(val, "exit", -1)
      Builtins.y2milestone("NovellUtils.GetServerContext:return code:%1", val)
      if exit == 0
        ret = Ops.get_string(val, "stdout", "")
        ret = "" if ret == nil

        # remove the line feed from the stdout string
        ret = StripTrailing(ret, "\n ")

        Builtins.y2milestone(
          "NovellUtils.GetServerContext:context returned:%1",
          ret
        )
      else
        if exit == 1
          Builtins.y2milestone(
            "NovellUtils.GetServerContext:Server is not reachable using the given ip address:%1",
            ipAddress
          )
        elsif exit == 2
          Builtins.y2milestone("NovellUtils.GetServerContext:Parameter error.")
        else
          Builtins.y2milestone(
            "NovellUtils.GetServerContext:Unknown error:%1",
            exit
          )
        end
      end

      ret
    end

    #***********************************************************
    # LDAP/NDAP manipulation routines
    #***********************************************************

    # Nds2Ldap(string) Convert string from NDS syntax to LDAP syntax
    # MUST take special care to deal with escaped periods "\\."
    # LDAP accepts plain periods.
    # @param string
    # @return [String]
    def Nds2Ldap(nds)
      ldap = ""

      return nds if nds == "[root]"

      ldap = ValidationHelpers.toLdap(nds)

      # now replace any nds style ("s") state attributes with ldap style ("st") state attributes
      split = Builtins.splitstring(ldap, ",")
      subsplit = []

      indx = 0
      foundState = false
      Builtins.foreach(split) do |testLine|
        if Builtins.issubstring(Builtins.tolower(testLine), "s=")
          subsplit = Builtins.splitstring(testLine, "=")

          Ops.set(split, indx, Ops.add("st=", Ops.get(subsplit, 1, "")))
          foundState = true
        end
        indx = Ops.add(indx, 1)
      end

      ldap = Builtins.mergestring(split, ",") if foundState

      ldap
    end

    #***********************************************************
    # LDAP/NDAP manipulation routines
    #***********************************************************

    # escape the '.' for extend schema util
    # @param string
    # @return [String]
    def esdot(dn)
      ValidationHelpers.esdot(dn)
    end


    # Ldap2Nds(string) Convert string from LDAP syntax to NDS syntax
    # The "." must become "\\."
    # @param string
    # @return [String]
    def Ldap2Nds(ldap)
      nds = ""

      return ldap if ldap == "[root]"

      nds = ValidationHelpers.toNds(ldap)

      # now replace any ldap style ("st") state attributes with nds style ("s") state attributes
      split = Builtins.splitstring(nds, ".")
      subsplit = []

      indx = 0
      foundState = false
      Builtins.foreach(split) do |testLine|
        if Builtins.issubstring(Builtins.tolower(testLine), "st=")
          subsplit = Builtins.splitstring(testLine, "=")

          Ops.set(split, indx, Ops.add("s=", Ops.get(subsplit, 1, "")))
          foundState = true
        end
        indx = Ops.add(indx, 1)
      end

      nds = Builtins.mergestring(split, ".") if foundState

      nds
    end


    # Checks if the string is in LDAP form
    # @param string
    # @return [Boolean]
    def isLDAP(ldap)
      ldap == ValidationHelpers.toLdap(ldap)
    end


    # Checks if the string is in NDS (NDAP typed) form
    # @param string
    # @return [Boolean]
    def isNDS(nds)
      nds == ValidationHelpers.toNds(nds)
    end


    # Typeful2Typeless (string ndsName)
    #
    # Convest an X500 name in NDS format
    # to a typeless version*
    #
    # @param string nds name
    # @return [String]
    def Typeful2Typeless(ndsName)
      ndsName = Ldap2Nds(ndsName) # make sure given paramater really is NDAP format
      tmp = ndsName
      return tmp if Builtins.search(tmp, "=") == nil
      tmp = String.Replace(tmp, "\\.", "+")
      stmp = Builtins.splitstring(tmp, ".")
      rval = ""
      Builtins.foreach(stmp) do |seg|
        idx = Builtins.search(seg, "=")
        if idx != nil
          rval = Ops.add(rval, ".") if rval != nil && rval != ""
          rval = Ops.add(rval, Builtins.substring(seg, Ops.add(idx, 1)))
        end
      end
      rval = String.Replace(rval, "+", "\\.")
      rval
    end


    # Return the context containing the name given
    # In NDAP strings we must hadle "\."
    # @param string nds name
    # @return [String] nds context
    def GetContextNDS(name)
      tmp = ValidationHelpers.toNds(name)
      tmp = String.Replace(tmp, "\\.", "+")
      if tmp != nil && Builtins.findfirstof(tmp, ".") != nil
        tmp = Builtins.substring(
          tmp,
          Ops.add(Builtins.findfirstof(tmp, "."), 1)
        )
        tmp = String.Replace(tmp, "+", "\\.")
        return tmp
      end
      ""
    end


    # Return the context containing the name given in LDAP
    # There are no "\\." in legal LDAP names
    # @param string ldap name
    # @return [String] ldap context
    def GetContextLDAP(name)
      tmp = ValidationHelpers.toLdap(name)
      tmp = String.Replace(tmp, "\\,", "+")
      if tmp != nil && Builtins.findfirstof(tmp, ",") != nil
        tmp = Builtins.substring(
          tmp,
          Ops.add(Builtins.findfirstof(tmp, ","), 1)
        )
        tmp = String.Replace(tmp, "+", "\\,")
        return tmp
      end
      ""
    end


    # Return the context containing the name given
    # @param string
    # @return [String]
    def GetContext(name)
      if isLDAP(name)
        name = GetContextLDAP(name)
      else
        name = GetContextNDS(name)
      end
      name
    end


    # Return the top context for a NDS name
    # @param string nds name
    # @param string nds context
    def GetTopContextNDS(name)
      topContext = GetContextNDS(name)
      prevContext = name
      while topContext != ""
        prevContext = topContext
        topContext = GetContextNDS(topContext)
      end
      prevContext
    end


    # Returns a list contining the CN and context seperated from the FDN (e.g. "cn=admin,o=novell" --> $["admin", "novell"])
    # @param [String] fdn
    # @return [Array]
    def ExtractCN(fdn)
      fdn = Nds2Ldap(fdn)
      i = Builtins.search(fdn, ",")

      return [] if i == nil || Ops.less_than(i, 4) #extractCN failed return empty list

      cn = Builtins.substring(fdn, 3, Ops.subtract(i, 3)) #skip over 'cn='
      context = Builtins.substring(fdn, Ops.add(i, 1))
      [cn, context]
    end


    # Attempt to return the FDN for this node by using /usr/bin/ndsstat
    # @return [String]
    def GetServerFDN
      # check if we can get a FDN for this node
      rmap = Convert.to_map(
        SCR.Execute(
          path(".target.bash_output"),
          "/opt/novell/eDirectory/bin/ndsstat | grep \"^Server Name:\""
        )
      )
      if Ops.get_integer(rmap, "exit", -1) == 0
        fdn = Ops.get_string(rmap, "stdout", "")

        _begin = Builtins.search(fdn, ".") # skip over leading '.'
        return "" if _begin == nil || Ops.less_than(_begin, 13) # length of "Sever Name: "
        _end = Builtins.search(fdn, ".T=")
        return "" if _end == nil || Ops.less_or_equal(_end, _begin)

        fdn = Builtins.substring(
          fdn,
          Ops.add(_begin, 1),
          Ops.subtract(_end, Ops.add(_begin, 1))
        )
        fdn = Strip(fdn, "\n \t ")
        return fdn
      end

      ""
    end



    #***********************************************************
    # Misc routines
    #***********************************************************

    # WARNING: NovellUtils::IsSelectionInstalled() is deprecated. Please use NovellPatterns::IsPatternInstalled() instead.
    def IsSelectionInstalled(selection_name)
      Builtins.y2warning(
        "WARNING: NovellUtils::IsSelectionInstalled() is deprecated. Please use NovellPatterns::IsPatternInstalled() instead."
      )
      Yast.import "NovellPatterns"
      NovellPatterns.IsPatternInstalled(selection_name)
    end

    # Is the currently installed product desktop or server based?
    # @return [Boolean] true if server product
    def IsServerProduct
      Builtins.issubstring(Builtins.tolower(Product.name), "server")
    end

    # Get the user ID of a given user (in string format).
    # @param string username
    # @return [String] UID
    def Get_UID(user)
      if user == nil || user == ""
        Builtins.y2milestone("NovellUtils:Get_UID:Empty parameter.")
        return ""
      end

      execute_cmd = Ops.add(
        Ops.add("grep ^", user),
        ": /etc/passwd | cut -d':' -f3"
      )
      rmap = Convert.to_map(
        SCR.Execute(path(".target.bash_output"), execute_cmd)
      )
      if Ops.get_integer(rmap, "exit", -1) != 0
        Builtins.y2milestone(
          "NovellUtils:Get_UID:Error getting user ID for user %1",
          user
        )
        return ""
      end
      CleanUpString(Ops.get_string(rmap, "stdout", ""))
    end


    # Get the group ID of a given group (in string format).
    # @param [String] group name
    # @return [String] GID
    def Get_GID(group)
      if group == nil || group == ""
        Builtins.y2milestone("NovellUtils:Get_GID:Empty parameter.")
        return ""
      end

      execute_cmd = Ops.add(
        Ops.add("grep ^", group),
        ": /etc/group | cut -d':' -f3"
      )
      rmap = Convert.to_map(
        SCR.Execute(path(".target.bash_output"), execute_cmd)
      )
      if Ops.get_integer(rmap, "exit", -1) != 0
        Builtins.y2milestone(
          "NovellUtils:Get_GID:Error getting group ID for group %1",
          group
        )
        return ""
      end

      CleanUpString(Ops.get_string(rmap, "stdout", ""))
    end

    # Get the group IDs of a given user (in string format).
    # @param [String] user name
    # @return [Array<String>] of GIDs
    def Get_User_GIDs(user)
      if user == nil || user == ""
        Builtins.y2milestone("NovellUtils:Get_User_GIDs:Empty parameter.")
        return []
      end

      execute_cmd = Builtins.sformat("id -G '%1'", user)
      rmap = Convert.to_map(
        SCR.Execute(path(".target.bash_output"), execute_cmd)
      )
      if Ops.get_integer(rmap, "exit", -1) != 0
        Builtins.y2milestone(
          "NovellUtils::Get_User_GIDs:Error getting group IDs for user %1",
          user
        )
        return []
      end

      tmpstr = CleanUpString(Ops.get_string(rmap, "stdout", ""))
      ret = Builtins.splitstring(tmpstr, " \t")
      if ret == nil
        ret = []
      else
        ret = Builtins.filter(ret) { |s| s != nil && s != "" }
      end
      Builtins.y2milestone("NovellUtils::Get_User_GIDs returned: %1", ret)
      deep_copy(ret)
    end


    # Check and see if LUM installed and configured.
    # FIXME: maybe this should be moved into the LUM module as part of the status utility??
    # @return [Boolean]
    def LUMAvailable
      Yast.import "NovellPatterns"
      lum_configured = false
      service_configured = Convert.to_string(
        SCR.Read(path(".sysconfig.novell-lum.SERVICE_CONFIGURED"))
      )
      service_configured = Builtins.tolower(service_configured)
      lum_configured = true if service_configured == "yes"

      if lum_configured ||
          NovellPatterns.IsPatternInstalled("novell-lum") &&
            Ops.get(
              NovellAddonInstall.products_to_configure,
              "novell-lum",
              true
            )
        return true
      else
        return false
      end
    end




    # Routine: StoreTrustedCertificateInJavaKeystore
    # Parameters:
    #    certificateFile - Full path specification to the file
    #                             containing the certificate
    #    keystore - Full path to the keystore in which to store the file
    #    alias    - name by which the certificate should be known
    #
    # This enables SSL validation of the certificate path for authentication
    # using secure LDAP or secure HTTP.
    #
    # ONLY ROOT CERTIFICATE should be stored in the root certificate store
    # (lots of people still like to ignore this edict).
    #
    # @param [String] certificateFile - Full path specification to the file containing the certificate
    # @param [String] keystore - Full path to the keystore in which to store the file
    # @param [String] alias    - name by which the certificate should be known
    # @return [Boolean]
    def StoreTrustedCertificateInJavaKeystore(certificateFile, keystore, _alias)
      certStored = false
      execute_cmd = ""
      rval = -1

      # Location of the Java used by this application
      #    string lib = "lib";  // this is the 32 bit version

      #    if( Arch::x86_64 ())
      #        lib = "lib64";   // this is the 64 bit path

      # use the sym link in /usr/bin provided by update alternatives.
      _JAVA_HOME = "/usr"

      # keystore file location
      _ROOT_CERT_STORE = keystore

      # The absolute path to the expected location of the certificate
      _CERT_FILE = certificateFile

      # Make sure that the path to the keystore exists
      pathToKeystore = ""
      pathEnd = Builtins.findlastof(keystore, "/")
      if pathEnd != nil
        pathToKeystore = Builtins.substring(keystore, 0, pathEnd)
      end
      done = Convert.to_integer(
        SCR.Execute(
          path(".target.bash"),
          Builtins.sformat("mkdir -p %1", pathToKeystore)
        )
      )
      if done == 0
        Builtins.y2milestone(
          "Created path to keystore file = %1",
          pathToKeystore
        )
      else
        Builtins.y2error(
          "Failed to create path to keystore file = %1",
          pathToKeystore
        )
      end

      sizeofCert = Convert.to_integer(
        SCR.Read(path(".target.size"), _CERT_FILE)
      )
      sizeofAlias = Builtins.size(_alias)

      if Ops.greater_than(sizeofCert, 0) && Ops.greater_than(sizeofAlias, 0)
        # Attempt to remove any existing certificate by the same alias
        # if this is not done THEN the import will FAIL
        #
        # Remove certificate from the specified keystore
        # using the alias provided
        execute_cmd = Ops.add(
          Ops.add(
            Ops.add(
              Ops.add(
                Ops.add(
                  Ops.add(Ops.add(_JAVA_HOME, "/bin/keytool"), " -keystore "),
                  _ROOT_CERT_STORE
                ),
                " -delete "
              ),
              " -alias "
            ),
            _alias
          ),
          " -storepass changeit "
        )

        rval = Convert.to_integer(
          SCR.Execute(path(".target.bash"), execute_cmd)
        )
        if rval == 0
          Builtins.y2milestone(
            "Previous copy of the %1 certificate was removed from the Java keystore",
            _alias
          )
        end
        # add the certificate to the specified keystore
        execute_cmd = Ops.add(
          Ops.add(
            Ops.add(
              Ops.add(
                Ops.add(
                  Ops.add(
                    Ops.add(
                      Ops.add(
                        Ops.add(
                          Ops.add(_JAVA_HOME, "/bin/keytool"),
                          " -keystore "
                        ),
                        _ROOT_CERT_STORE
                      ),
                      " -import "
                    ),
                    " -alias "
                  ),
                  _alias
                ),
                " -file "
              ),
              _CERT_FILE
            ),
            " -storepass changeit "
          ),
          " -noprompt "
        )

        rval = Convert.to_integer(
          SCR.Execute(path(".target.bash"), execute_cmd)
        )
        if rval != 0
          # TODO - fix this message
          Report.Warning(
            _(
              "Unable to add the tree self-signed certificate to the root certificate store"
            )
          )
        else
          certStored = true
        end
      else
        if !Ops.greater_than(sizeofCert, 0)
          Builtins.y2milestone(
            "Unable to store the certificate %1 in the keystore because the the certificate file was not found.",
            _CERT_FILE
          )
          certStored = false
        end
        if !Ops.greater_than(sizeofAlias, 0)
          Builtins.y2milestone(
            "Unable to store the certificate %1 in the keystore because no alias for the certificate was provided by the caller",
            _CERT_FILE
          )
        end
      end

      if certStored
        # Make www the group owner of the ROOT_CERT_STORE
        execute_cmd = Ops.add("chgrp " + " www ", _ROOT_CERT_STORE)
        rval = Convert.to_integer(
          SCR.Execute(path(".target.bash"), execute_cmd)
        )
        if rval != 0
          Report.Warning(
            _(
              "Unable to change the group owner of the root certificate store to www"
            )
          )
        end

        # change group permissions the ROOT_CERT_STORE to give write access to www
        execute_cmd = Ops.add("chmod " + " g+w ", _ROOT_CERT_STORE)
        rval = Convert.to_integer(
          SCR.Execute(path(".target.bash"), execute_cmd)
        )
        if rval != 0
          Report.Warning(
            _(
              "Unable to change the group permissions on the root certificate store for write access by www"
            )
          )
        end
      end
      certStored
    end

    # Displays a formatted popup error message, with a details button.
    # @param [String] errorMessage string - a simple message describing the error
    # @param [Hash] out map - output of the executed command
    # @param [String] cmd string - the executed command
    # @return [void]
    def PopupErrorDetails(errorMessage, out, cmd)
      out = deep_copy(out)
      Builtins.y2milestone(
        "NovellUtils::PopupErrorDetails(%1, %2, %3)",
        errorMessage,
        out,
        cmd
      )
      message = _("Undefined Error")
      message = errorMessage if errorMessage != nil && errorMessage != ""
      Popup.ClearFeedback
      if !Popup.AnyQuestion(
          Label.ErrorMsg,
          message,
          Label.OKButton,
          _("Details"),
          :focus
        )
        stderr = Ops.get_string(out, "stderr", "")
        stderr = "" if stderr == nil
        stdout = Ops.get_string(out, "stdout", "")
        stdout = "" if stdout == nil
        description = Builtins.sformat(
          "<pre>command: %1 \n %2 %3</pre>",
          cmd,
          stderr,
          stdout
        )
        UI.OpenDialog(
          Opt(:decorated),
          HBox(
            VSpacing(25),
            VBox(
              HSpacing(100),
              # popup window header
              Heading(message),
              VSpacing(0.5),
              RichText(description),
              VSpacing(1.0),
              # push button label
              PushButton(Id(:ok), Opt(:default, :key_F10), Label.OKButton)
            )
          )
        )

        UI.SetFocus(Id(:ok))
        UI.UserInput
        UI.CloseDialog
      end

      nil
    end

    # Displays a formatted popup error message, with a details button.
    # @param [String] errorMessage string - a simple message describing the error
    # @param out map - output of the executed command
    # @param cmd string - the executed command
    # @return [void]
    def PopupErrorDetailsText(errorMessage, details)
      Builtins.y2milestone(
        "NovellUtils::PopupErrorDetailsText(%1, %2)",
        errorMessage,
        details
      )
      message = _("Undefined Error")
      message = errorMessage if errorMessage != nil && errorMessage != ""
      Popup.ClearFeedback
      if !Popup.AnyQuestion(
          Label.ErrorMsg,
          message,
          Label.OKButton,
          _("Details"),
          :focus
        )
        UI.OpenDialog(
          Opt(:decorated),
          HBox(
            VSpacing(10),
            VBox(
              HSpacing(75),
              # popup window header
              Heading(message),
              VSpacing(0.5),
              RichText(details),
              VSpacing(1.0),
              # push button label
              PushButton(Id(:ok), Opt(:default, :key_F10), Label.OKButton)
            )
          )
        )

        UI.SetFocus(Id(:ok))
        UI.UserInput
        UI.CloseDialog
      end

      nil
    end

    def PopupErrorDetailsWithLogFile(errorMessage, out, cmd, logfilePath)
      out = deep_copy(out)
      logData = ""
      if Ops.greater_than(SCR.Read(path(".target.size"), logfilePath), 0)
        logData = Convert.to_string(
          SCR.Read(path(".target.string"), logfilePath)
        )
      end

      if logData != nil && logData != ""
        Ops.set(out, "stdout", Ops.add("\n", logData))
      end

      PopupErrorDetails(errorMessage, out, Ops.get_string(out, "cmd", ""))

      nil
    end

    # Check if a given SCR agent is mounted and accessable
    #
    # FIXME: This is a bad hack!!!  Must talked with SUSE YaST team to solve the problem
    #
    # @param [Yast::Path] agent_path YCP path the SCR agent
    # @param [Yast::Path] additional_path optional path to appened to the agent_path for the SCR::Read. NOTE: If additonal_path == nil then will try SCR::Dir(agent_path) which only works well for Sysconfig agents
    # @return [Boolean] true if the SCR Sysconfig agent is mounted
    def IsAgentMounted(agent_path, additional_path)
      agent_mounted = false

      # Check if agent exists
      if additional_path == nil
        val = SCR.Dir(agent_path)
        Builtins.y2debug("SCR::Dir(%1) returned: %2", agent_path, val)
        if val != nil && Ops.greater_than(Builtins.size(val), 0)
          agent_mounted = true
          Builtins.y2milestone("SCR agent %1 is mounted", agent_path)
        end
      else
        read_path = Ops.add(agent_path, additional_path)
        val = Convert.to_string(SCR.Read(read_path))
        Builtins.y2debug("SCR::Read(%1) returned: %2", read_path, val)
        if val != nil && val != ""
          agent_mounted = true
          Builtins.y2milestone("SCR agent %1 is mounted", agent_path)
        end
      end


      if !agent_mounted
        Builtins.y2warning("SCR agent %1 is not mounted!", agent_path)
      end

      agent_mounted
    end


    # Ensure a given SCR agent is mounted and accessable
    # If the agent is not mounted, try to mount it using SCR::RegisterAgent
    #
    # FIXME: This is a bad hack!!!  Must talked with SUSE YaST team to solve the problem
    #
    # @param [String] agent_cfg string name of the SCR agent cfg file (relative path from the scrconf dir)
    # @param [Yast::Path] agent_path YCP path to test the SCR agent
    # @param [Yast::Path] additional_path optional path to appened to the agent_path for the SCR::Read. NOTE: If additonal_path == nil then will try SCR::Dir(agent_path) which only works well for Sysconfig agents
    # @return [Boolean] true if the SCR agent is mounted
    def EnsureAgentMounted(agent_cfg, agent_path, additional_path)
      # make sure the agent isn't already mounted
      return true if IsAgentMounted(agent_path, additional_path)

      y2update_cfg = Builtins.sformat("/y2update/scrconf/%1", agent_cfg) # filename path to the scr agent cfg in the y2update directory
      scrconf_cfg = Builtins.sformat("/usr/share/YaST2/scrconf/%1", agent_cfg) # filename path to the scr agent cfg in scrconf directory

      if Ops.greater_or_equal(SCR.Read(path(".target.size"), y2update_cfg), 0)
        #try and mount the agent from the y2update directory
        Builtins.y2milestone(
          "Trying to mount SCR agent %1 using %2",
          agent_path,
          y2update_cfg
        )
        SCR.RegisterAgent(agent_path, y2update_cfg)
        if IsAgentMounted(agent_path, additional_path)
          return true # agent mounted!
        end
      end

      # try and mount the agent from the scrconf directory
      Builtins.y2milestone(
        "Trying to mount SCR agent %1 using %2",
        agent_path,
        scrconf_cfg
      )
      SCR.RegisterAgent(agent_path, scrconf_cfg)

      IsAgentMounted(agent_path, additional_path)
    end


    # Ensure a given SCR Sysconfig agent is mounted and accessable
    # If the agent is not mounted, try to mount it using SCR::RegisterAgent
    # NOTE: this only works for Syconfig SCR agents!!!!
    #
    # FIXME: This is a bad hack!!!  Must talked with SUSE YaST team to solve the problem
    #
    # @param [String] agent_cfg string name of the SCR agent cfg file (relative path from the scrconf dir)
    # @param [Yast::Path] agent_path YCP path to test the SCR agent
    # @return [Boolean] true if the SCR agent is mounted
    def EnsureSysconfigAgentMounted(agent_cfg, agent_path)
      EnsureAgentMounted(agent_cfg, agent_path, nil)
    end


    # Generate a random password of 11 characters. (standard password policy is 2-12 )
    def GenerateRandomPassword
      pword = ""

      cmd = "head -c 20 < /dev/random | uuencode -m - | tail -n 2 | head -n 1"

      ret = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
      if Ops.get_integer(ret, "exit", -1) == 0
        pword = Ops.get_string(ret, "stdout", "")
        pword = Builtins.substring(
          pword,
          0,
          Ops.subtract(Builtins.size(pword), 1)
        ) # remove line feed
        # limit to 11 characters
        pword = Builtins.substring(pword, 0, 11)
        Builtins.y2milestone(
          "NovellUtils.GenerateRandomPassword:random password generated"
        )
      else
        Builtins.y2milestone(
          "NovellUtils.GenerateRandomPassword:random password generation not successful"
        )
      end
      pword
    end

    #    The following APIs set a file flag that says the user drilled down into edirectory and went through
    #    the eDir GUI a second or more times in the proposal/summary screen.  The flag will trigger all other
    #    installs that use eDir data for defaults to re-read the edir values and recompute their defaults.

    def ResetEdirAskUserFlag
      cmd = "rm -f /opt/novell/oes-install/ediraskuser"
      Builtins.y2milestone(
        "NovellUtils.ResetEdirAskUserFlag:executing: %1",
        cmd
      )
      rval = Convert.to_integer(SCR.Execute(path(".target.bash"), cmd))

      true
    end

    def SetEdirAskUserFlag
      # Start with a fresh file
      ResetEdirAskUserFlag()

      # Start out with a blank file.
      ret = SCR.Write(
        path(".target.string"),
        "/opt/novell/oes-install/ediraskuser",
        ""
      )
      Builtins.y2milestone(
        "NovellUtils.SetEdirAskUserFlag:writing out /opt/novell/oes-install/ediraskuser file."
      )
      ret = SCR.Write(
        path(".target.string"),
        "/opt/novell/oes-install/ediraskuser",
        "Edir Ask User Called:\n"
      )
      if ret != true
        Builtins.y2milestone(
          "NovellUtils.SetEdirAskUserFlag:Failed to write to /opt/novell/oes-install/ediraskuser"
        )
      end

      # Flush the buffer
      SCR.Write(
        path(".target.string"),
        "/opt/novell/oes-install/ediraskuser",
        nil
      )

      true
    end

    def ProductInList(product)
      ret = false

      data_file = "/opt/novell/oes-install/ediraskuser"
      stmp = []

      # Read the file into memory
      productData = Convert.to_string(
        SCR.Read(path(".target.string"), data_file)
      )

      if productData != nil && productData != ""
        # break up into individual lines
        stmp = Builtins.splitstring(productData, "\n")

        Builtins.foreach(stmp) do |prod|
          if prod != nil && prod != ""
            if prod == product
              ret = true
              raise Break
            end
          end
        end
      end

      ret
    end

    def AddProductToList(product)
      Builtins.y2milestone(
        "NovellUtils.AddProductToList:adding product %1 to list.",
        product
      )
      data_file = "/opt/novell/oes-install/ediraskuser"

      # Read the file into memory
      productData = Convert.to_string(
        SCR.Read(path(".target.string"), data_file)
      )

      # Append product to list
      productData = Ops.add(Ops.add(productData, product), "\n")

      # Save data to file
      ResetEdirAskUserFlag() # Start with a fresh file

      ret = SCR.Write(
        path(".target.string"),
        "/opt/novell/oes-install/ediraskuser",
        ""
      )
      ret = SCR.Write(
        path(".target.string"),
        "/opt/novell/oes-install/ediraskuser",
        productData
      )
      if ret != true
        Builtins.y2milestone(
          "NovellUtils.AddProductToList:Failed to write to /opt/novell/oes-install/ediraskuser"
        )
      end

      # Flush the buffer
      SCR.Write(
        path(".target.string"),
        "/opt/novell/oes-install/ediraskuser",
        nil
      )

      true
    end

    def EdirAskUserFlag(product)
      ret = false

      if Ops.greater_than(
          SCR.Read(path(".target.size"), "/opt/novell/oes-install/ediraskuser"),
          0
        )
        if ProductInList(product)
          ret = false
        else
          ret = true
          AddProductToList(product)
        end
        Builtins.y2milestone(
          "NovellUtils.EdirAskUserFlag:product in list: %1",
          ret
        )
      end
      Builtins.y2milestone(
        "NovellUtils.EdirAskUserFlag:flag file exists: %1",
        ret
      )
      ret
    end

    def SystemdStartorRestartService(service,upgrade)
      if upgrade == "yes" 
        if Service.Enabled(service)
          Service.Restart(service)
          if Service.Active(service)
            Builtins.y2milestone("Restarted %1 service",service)
          else
            Builtins.y2milestone("Restarting %l service failed",service)
          end
        else
          Builtins.y2milestone("%1 service is not enabled",service)
        end
      else
        Service.Enable(service)
        status = Service.Start(service)
        Builtins.y2milestone("Starting %1 service status %2",service,status)
      end
    end

    # OES Common Proxy User Methods

    def CreateOESProxyUser(proxyUser, proxyPassword, admin, adminPassword, ip, port, attach_policy)
      cmd = Builtins.sformat(
        "OES_DATA='%1' OES_PROXY_DATA='%2' %3create_common_proxyuser.sh '%4' '%5' %6 %7 %8",
        adminPassword,
        proxyPassword,
        @proxyScriptPath,
        proxyUser,
        admin,
        ip,
        port,
        attach_policy
      )
      log_cmd = Builtins.sformat(
        "%1create_common_proxyuser.sh '%2' '%3' %4 %5 %6",
        @proxyScriptPath,
        proxyUser,
        admin,
        ip,
        port,
        attach_policy
      )
      Builtins.y2milestone(
        "NovellUtils.CreateOESProxyUser:executing: %1",
        log_cmd
      )
      val = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
      exit = Ops.get_integer(val, "exit", -1)
      stdout = Ops.get_string(val, "stdout", "")
      Builtins.y2milestone(
        "NovellUtils.CreateOESProxyUser:returned:%1, with stdout:\n%2",
        val,
        stdout
      )

      exit
    end

    def StoreCredsInOCS(proxyUser, proxyPassword)
      cmd = Builtins.sformat(
        "OES_PROXY_DATA='%1' %2cp_update_proxy_cred.sh '%3'",
        proxyPassword,
        @proxyScriptPath,
        proxyUser
      )
      logCmd = Builtins.sformat(
        "OES_PROXY_DATA='%1' %2cp_update_proxy_cred.sh '%3'",
        "****",
        @proxyScriptPath,
        proxyUser
      )
      Builtins.y2milestone("NovellUtils.StoreCredsInOCS:executing: %1", logCmd)
      val = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
      exit = Ops.get_integer(val, "exit", -1)
      stdout = Ops.get_string(val, "stdout", "")
      Builtins.y2milestone(
        "NovellUtils.StoreCredsInOCS:returned:%1, with stdout:\n%2",
        val,
        stdout
      )

      exit
    end

    def StoreCredsInOCSforUpgrade(proxyUser, proxyPassword,service)
      cmd = Builtins.sformat(
        "OES_PROXY_DATA='%1' %2cp_update_proxy_cred_upgrade.sh '%3' '%4'",
        proxyPassword,
        @proxyScriptPath,
        proxyUser,
	service
      )
      logCmd = Builtins.sformat(
        "OES_PROXY_DATA='%1' %2cp_update_proxy_cred_upgrade.sh '%3' '%4'",
        "****",
        @proxyScriptPath,
        proxyUser,
	service
      )
      Builtins.y2milestone("NovellUtils.StoreCredsInOCSforUpgrade:executing: %1", logCmd)
      val = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
      exit = Ops.get_integer(val, "exit", -1)
      stdout = Ops.get_string(val, "stdout", "")
      Builtins.y2milestone(
        "NovellUtils.StoreCredsInOCS:returned:%1, with stdout:\n%2",
        val,
        stdout
      )

      exit
    end


    def CreateProxyPolicy(admin, adminPassword, ip, port, proxyPassword)
      cmd = Builtins.sformat(
        "OES_DATA='%1' OES_PROXY_DATA='%2' %3create_common_proxypolicy.sh '%4' %5 %6",
        adminPassword,
        proxyPassword,
        @proxyScriptPath,
        admin,
        ip,
        port
      )
      log_cmd = Builtins.sformat(
        "%1create_common_proxypolicy.sh '%2' %3 %4",
        @proxyScriptPath,
        admin,
        ip,
        port
      )
      Builtins.y2milestone(
        "NovellUtils.CreateProxyPolicy:executing: %1",
        log_cmd
      )
      val = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
      exit = Ops.get_integer(val, "exit", -1)
      stdout = Ops.get_string(val, "stdout", "")
      Builtins.y2milestone(
        "NovellUtils.CreateProxyPolicy:returned:%1, with stdout:\n%2",
        val,
        stdout
      )

      exit
    end

    def AttachProxyUserToPolicy(proxyUser, admin, adminPassword, ip, port, proxyPassword)
      cmd = Builtins.sformat(
        "OES_DATA='%1' OES_PROXY_DATA='%2' %3attach_proxyuser_pwpolicy.sh '%4' '%5' %6 %7",
        adminPassword,
        proxyPassword,
        @proxyScriptPath,
        proxyUser,
        admin,
        ip,
        port
      )
      log_cmd = Builtins.sformat(
        "%1attach_proxyuser_pwpolicy.sh '%2' '%3' %4 %5",
        @proxyScriptPath,
        proxyUser,
        admin,
        ip,
        port
      )
      Builtins.y2milestone(
        "NovellUtils.AttachProxyUserToPolicy:executing: %1",
        log_cmd
      )
      val = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
      exit = Ops.get_integer(val, "exit", -1)
      if exit == 2
        Popup.Message(
          _(
            "Unable to attach the proxy policy to common proxy user now.\nPlease complete this task after the installation using iManager"
          )
        )
        Builtins.y2milestone(
          "NovellUtils.AttachProxyUserToPolicy:call returned 2."
        )
        exit = 0
      end
      stdout = Ops.get_string(val, "stdout", "")
      Builtins.y2milestone(
        "NovellUtils.AttachProxyUserToPolicy:returned:%1, with stdout:\n%2",
        val,
        stdout
      )

      exit
    end

    # credType can be "username" or "password"
    def RetrieveProxyCredentials(credType)
      cmd = Builtins.sformat(
        "%1cp_retrieve_proxy_cred %2",
        @proxyScriptPath,
        credType
      )
      Builtins.y2milestone(
        "NovellUtils.RetrieveProxyCredentials:executing: %1",
        cmd
      )
      val = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
      exit = Ops.get_integer(val, "exit", -1)
      Builtins.y2milestone(
        "NovellUtils.RetrieveProxyCredentials:returned:%1",
        exit
      )
      stdout = Ops.get_string(val, "stdout", "")
      stdout = CleanUpString(stdout)

      stdout
    end

    # get a password based on current policy
    def RetrievePolicyPassword(ip, port, admin, adminPassword, proxy_context)
      cmd = Builtins.sformat(
        "%1randpolicypwd %2 %3 1 '%4' '%5' '%6'",
        @proxyScriptPath,
        ip,
        port,
        admin,
        adminPassword,
        proxy_context
      )
      log_cmd = Builtins.sformat(
        "%1randpolicypwd %2 %3 1 '%4' '%5' '%6'",
        @proxyScriptPath,
        ip,
        port,
        admin,
        "****",
        proxy_context
      )
      Builtins.y2milestone(
        "NovellUtils.RetrievePolicyPassword:executing: %1",
        log_cmd
      )
      val = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
      exit = Ops.get_integer(val, "exit", -1)
      stdout = Ops.get_string(val, "stdout", "")

      stdout
    end

    def ProxyAddToCrontab
      retCode = true

      cmd = Builtins.sformat("%1add_to_crontab.sh", @proxyScriptPath)
      Builtins.y2milestone("NovellUtils.ProxyAddToCrontab:executing: %1", cmd)
      val = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))

      exit = Ops.get_integer(val, "exit", -1)
      retCode = false if exit != 0

      Builtins.y2milestone("NovellUtils.ProxyAddToCrontab:returned:%1", val)

      retCode
    end

    # prints Mode:: and Stage:: values to log
    def LogInstallMode
      Builtins.y2milestone("**************************")
      Builtins.y2milestone("Mode::installation(): %1", Mode.installation)
      Builtins.y2milestone("Mode::normal(): %1", Mode.normal)
      Builtins.y2milestone("Stage::normal(): %1", Stage.normal)
      Builtins.y2milestone("Stage::cont(): %1", Stage.cont)
      Builtins.y2milestone("Stage::initial(): %1", Stage.initial)
      Builtins.y2milestone("Stage::firstboot(): %1", Stage.firstboot)
      Builtins.y2milestone("**************************")
      true
    end

    def TranslateLDAPError(returnCode)
      retCodeValues = {}

      # from ldap.h
      Ops.set(retCodeValues, 1, "LDAP_OPERATIONS_ERROR")
      Ops.set(retCodeValues, 2, "LDAP_PROTOCOL_ERROR")
      Ops.set(retCodeValues, 3, "LDAP_TIMELIMIT_EXCEEDED")
      Ops.set(retCodeValues, 4, "LDAP_SIZELIMIT_EXCEEDED")
      Ops.set(retCodeValues, 5, "LDAP_COMPARE_FALSE")
      Ops.set(retCodeValues, 6, "LDAP_COMPARE_TRUE")
      Ops.set(retCodeValues, 7, "LDAP_AUTH_METHOD_NOT_SUPPORTED")
      Ops.set(retCodeValues, 8, "LDAP_STRONG_AUTH_REQUIRED")
      Ops.set(retCodeValues, 9, "LDAP_PARTIAL_RESULTS")
      Ops.set(retCodeValues, 10, "LDAP_REFERRAL")
      Ops.set(retCodeValues, 11, "LDAP_ADMINLIMIT_EXCEEDED")
      Ops.set(retCodeValues, 12, "LDAP_UNAVAILABLE_CRITICAL_EXTENSION")
      Ops.set(retCodeValues, 13, "LDAP_CONFIDENTIALITY_REQUIRED")
      Ops.set(retCodeValues, 14, "LDAP_SASL_BIND_IN_PROGRESS")
      Ops.set(retCodeValues, 16, "LDAP_NO_SUCH_ATTRIBUTE")
      Ops.set(retCodeValues, 17, "LDAP_UNDEFINED_TYPE")
      Ops.set(retCodeValues, 18, "LDAP_INAPPROPRIATE_MATCHING")
      Ops.set(retCodeValues, 19, "LDAP_CONSTRAINT_VIOLATION")
      Ops.set(retCodeValues, 20, "LDAP_TYPE_OR_VALUE_EXISTS")
      Ops.set(retCodeValues, 21, "LDAP_INVALID_SYNTAX")
      Ops.set(retCodeValues, 32, "LDAP_NO_SUCH_OBJECT")
      Ops.set(retCodeValues, 33, "LDAP_ALIAS_PROBLEM")
      Ops.set(retCodeValues, 34, "LDAP_INVALID_DN_SYNTAX")
      Ops.set(retCodeValues, 35, "LDAP_IS_LEAF")
      Ops.set(retCodeValues, 36, "LDAP_ALIAS_DEREF_PROBLEM")
      Ops.set(retCodeValues, 47, "LDAP_X_PROXY_AUTHZ_FAILURE")
      Ops.set(retCodeValues, 48, "LDAP_INAPPROPRIATE_AUTH")
      Ops.set(retCodeValues, 49, "LDAP_INVALID_CREDENTIALS")
      Ops.set(retCodeValues, 50, "LDAP_INSUFFICIENT_ACCESS")
      Ops.set(retCodeValues, 51, "LDAP_BUSY")
      Ops.set(retCodeValues, 52, "LDAP_UNAVAILABLE")
      Ops.set(retCodeValues, 53, "LDAP_UNWILLING_TO_PERFORM")
      Ops.set(retCodeValues, 54, "LDAP_LOOP_DETECT")
      Ops.set(retCodeValues, 64, "LDAP_NAMING_VIOLATION")
      Ops.set(retCodeValues, 65, "LDAP_OBJECT_CLASS_VIOLATION")
      Ops.set(retCodeValues, 66, "LDAP_NOT_ALLOWED_ON_NONLEAF")
      Ops.set(retCodeValues, 67, "LDAP_NOT_ALLOWED_ON_RDN")
      Ops.set(retCodeValues, 68, "LDAP_ALREADY_EXISTS")
      Ops.set(retCodeValues, 69, "LDAP_NO_OBJECT_CLASS_MODS")
      Ops.set(retCodeValues, 70, "LDAP_RESULTS_TOO_LARGE")
      Ops.set(retCodeValues, 71, "LDAP_AFFECTS_MULTIPLE_DSAS")
      Ops.set(retCodeValues, 80, "LDAP_OTHER")
      Ops.set(retCodeValues, 113, "LDAP_CUP_RESOURCES_EXHAUSTED")
      Ops.set(retCodeValues, 114, "LDAP_CUP_SECURITY_VIOLATION")
      Ops.set(retCodeValues, 115, "LDAP_CUP_INVALID_DATA")
      Ops.set(retCodeValues, 116, "LDAP_CUP_UNSUPPORTED_SCHEME")
      Ops.set(retCodeValues, 117, "LDAP_CUP_RELOAD_REQUIRED")
      Ops.set(retCodeValues, 118, "LDAP_CANCELLED")
      Ops.set(retCodeValues, 119, "LDAP_NO_SUCH_OPERATION")
      Ops.set(retCodeValues, 120, "LDAP_TOO_LATE")
      Ops.set(retCodeValues, 121, "LDAP_CANNOT_CANCEL")
      Ops.set(retCodeValues, 122, "LDAP_ASSERTION_FAILED")
      Ops.set(retCodeValues, 123, "LDAP_PROXIED_AUTHORIZATION_DENIED")

      retVal = Ops.get_string(retCodeValues, returnCode, "")
      retVal = "UNKNOWN_ERROR" if retVal == nil || retVal == ""

      retVal
    end
    def ServerIsDsfwServer(ip)
      retCode = false

      cmd = Builtins.sformat(
        "/usr/bin/ldapsearch -x -H ldaps://%1:636 -s base -b \"\" | grep 2.16.840.1.113719.1.513.7.1 > /dev/null",
        ip
      )
      Builtins.y2milestone("Edirectory.ServerIsDsfwServer:executing %1", cmd)
      ret = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
      retCode = true if Ops.get_integer(ret, "exit", -1) == 0 # server is a dsfw server

      Builtins.y2milestone(
        "Edirectory.ServerIsDsfwServer:returned %1",
        Ops.get_integer(ret, "exit", -1)
      )
      retCode
    end


    # Pings an IP address to see if it responds
    def Ping_IP(ip)
      retCode = true

      cmd = Ops.add("ping -c 2 ", ip) # Send 2 packets, then exit.

      ccode = Convert.to_integer(SCR.Execute(path(".target.bash"), cmd))
      retCode = false if ccode != 0

      retCode
    end
    def AddtoOpenLdapConfFile
      Builtins.y2milestone(
        "Executing NovellUtils::AddtoOpenLdapConfFile() function"
      )
      cmd = "grep  'TLS_CACERTDIR.*/etc/ssl/certs' /etc/openldap/ldap.conf > /dev/null"
      rval = Convert.to_integer(SCR.Execute(path(".target.bash"), cmd))
      if rval != 0
        cmd2 = "echo TLS_CACERTDIR /etc/ssl/certs >>  /etc/openldap/ldap.conf"
        rval2 = Convert.to_integer(SCR.Execute(path(".target.bash"), cmd2))
        Builtins.y2milestone(
          "Edirectory: executing cmd: %1  rval: %2",
          cmd2,
          rval2
        )
        if rval2 != 0
          Builtins.y2milestone(
            "Failed to add 'TLS_CACERTDIR /etc/ssl/certs' to /etc/openldap/ldap.conf file"
          )
        end
      end
      cmd = "grep  'TLS_CERT.*/etc/ssl/servercerts/servercert.pem' /etc/openldap/ldap.conf > /dev/null"
      rval = Convert.to_integer(SCR.Execute(path(".target.bash"), cmd))
      if rval != 0
        cmd2 = "echo TLS_CERT /etc/ssl/servercerts/servercert.pem >>  /etc/openldap/ldap.conf"
        rval2 = Convert.to_integer(SCR.Execute(path(".target.bash"), cmd2))
        Builtins.y2milestone(
          "Edirectory: executing cmd: %1  rval: %2",
          cmd2,
          rval2
        )
        if rval2 != 0
          Builtins.y2milestone(
            "Failed to add 'TLS_CERT /etc/ssl/servercerts/servercert.pem' to /etc/openldap/ldap.conf file"
          )
        end
      end
      cmd = "grep  'TLS_KEY.*/etc/ssl/servercerts/serverkey.pem' /etc/openldap/ldap.conf > /dev/null"
      rval = Convert.to_integer(SCR.Execute(path(".target.bash"), cmd))
      if rval != 0
        cmd2 = "echo TLS_KEY /etc/ssl/servercerts/serverkey.pem >>  /etc/openldap/ldap.conf"
        rval2 = Convert.to_integer(SCR.Execute(path(".target.bash"), cmd2))
        Builtins.y2milestone(
          "Edirectory: executing cmd: %1  rval: %2",
          cmd2,
          rval2
        )
        if rval2 != 0
          Builtins.y2milestone(
            "Failed to add 'TLS_KEY /etc/ssl/servercerts/serverkey.pem' to /etc/openldap/ldap.conf file"
          )
        end
      end

      nil
    end
    def ServerIseDirServer(ip)
      if ip == "" || ip == nil
        Report.Error(
          Builtins.sformat(_("eDirectory IP address cannot be empty."))
        )
        return
      end
      Popup.ShowFeedback(
        _("Validating the eDirectory server."),
        _("This may take a while")
      )
      execute_cmd = Ops.add("/opt/novell/eDirectory/bin/ndsstat -h ", ip)
      ret = Convert.to_map(
        SCR.Execute(path(".target.bash_output"), execute_cmd)
      )
      mret = Ops.get_integer(ret, "exit", -1)
      Popup.ClearFeedback
      if mret == 0
        Popup.Message(
          Builtins.sformat(
            _(
              "Validation successful. Proceed with the eDirectory configuration."
            )
          )
        )
      else
        Report.Error(
          Builtins.sformat(
            _(
              "Validation unsuccessful. Ensure that the eDirectory is up and running,\n" +
                "or the eDirectory server IP address that you have entered is a valid one.\n" +
                "Proceed with the installation only after a successful validation."
            )
          )
        )
      end

      nil
    end

    # Function to copy installation.xml from OES media to /etc/opt/novell/oes-installation.xml
    def CopyControlFile
      Builtins.y2milestone("Executing NovellUtils::CopyControlFile()")
      src_id = NovellPatterns.GetProductSourceID(NovellPatterns.oes_addon_name)
      control = Pkg.SourceProvideOptionalFile(src_id, 1, "/installation.xml")
      cached_control_file = "/etc/opt/novell/oes-installation.xml"

      # Taking backup of oes-installation.xml during upgrade or autoupgrade

      if FileUtils.Exists(cached_control_file)
        if Mode.update || Mode.autoupgrade
          ret = Convert.to_integer(
            SCR.Execute(
              path(".target.bash"),
              Builtins.sformat(
                "/bin/mv %1 %2",
                cached_control_file,
                Ops.add(cached_control_file, "_backup")
              )
            )
          )
          if ret != 0
            Builtins.y2milestone(
              "Failed to copy installation.xml from media to %1_backup",
              cached_control_file
            )
          else
            Builtins.y2milestone(
              "Copied installation.xml from OES media to /etc/opt/novell/oes-installation.xml"
            )
          end
        end
      end
      if !FileUtils.Exists(cached_control_file)
        ret = Convert.to_integer(
          SCR.Execute(
            path(".target.bash"),
            Builtins.sformat("/bin/cp %1 %2", control, cached_control_file)
          )
        )
        if ret != 0
          Builtins.y2milestone(
            "Failed to copy installation.xml from media to %1",
            cached_control_file
          )
        else
          Builtins.y2milestone(
            "Copied installation.xml from OES media to /etc/opt/novell/oes-installation.xml"
          )
        end
      end

      nil
    end

    def AddKernelBootParam(param, value)
      Yast.import "Bootloader"

      Builtins.y2milestone("Adding #{param} kernel parameter set to #{value}")
      if Mode.update
        Builtins.y2milestone("Changing SCR to /mnt")
        old_SCR = WFM.SCRGetDefault
        new_SCR = WFM.SCROpen("chroot=/mnt:scr", false)
        WFM.SCRSetDefault(new_SCR)
      end
      Yast::Bootloader.modify_kernel_params(param => value)
      Yast::Bootloader.proposed_cfg_changed = true
      Yast::Bootloader.Write unless Stage.initial
      if Mode.update
        Builtins.y2milestone("Changing SCR back to default")
        WFM.SCRSetDefault(old_SCR)
        WFM.SCRClose(new_SCR)
      end
      nil
    end

    def MovetoCommonProxyDuringUpgrade(service)
      local_server = Ops.get(NovellLDAP.GetLocalServers(nil), 0, {})
      sPort = Ops.get_integer(local_server, "CONFIG_LDAP_SECURE_PORT", 636)
      ip = Ops.get_string(local_server, "CONFIG_LDAP_SERVER_IP", "")
      admin_fdn = NovellUtils.Nds2Ldap(
      Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_ADMIN_CONTEXT", ""))
      pwd = Ops.get_string(NovellLDAP.Settings,"CONFIG_LDAP_ADMIN_PASSWORD","" )
      Builtins.y2milestone("Moving %1 service to common proxy",service)
      cmd = "UPGRADE=1 /opt/novell/proxymgmt/bin/move_to_common_proxy.sh \-d #{admin_fdn} \-i #{ip} \-p #{sPort} \-s #{service}"
      val = Convert.to_map(
      SCR.Execute(
                  path(".target.bash_output"),
                  Builtins.sformat("%1 -w '%2' ", cmd,pwd)
                 ))
      Builtins.y2milestone("Status of moving  %1 service to common proxy : %2",service,val)
    end

    def RemoveKernelBootParam(param)
      Yast.import "Bootloader"

      Builtins.y2milestone("Removing #{param} kernel parameter")
      Yast::Bootloader.modify_kernel_params(param => :missing)
      Yast::Bootloader.proposed_cfg_changed = true
      Yast::Bootloader.Write unless Stage.initial
      nil
    end

    def CloneSettingstoAutoinstXml(pattern)
      isPatternInstalled = true
      exec_cmd = "rpm -q --whatprovides patterns-oes-#{pattern}"
      ret_val = Convert.to_map(SCR.Execute(path(".target.bash_output"), exec_cmd))
      installed =  Ops.get_integer(ret_val, "exit", -1)

      if installed == 0 
        isPatternInstalled = true
        Builtins.y2milestone("%1 pattern is installed, configuration detils will be cloned to autoinst.xml",pattern)
      else
        isPatternInstalled = false
        Builtins.y2milestone("%1 pattern is not installed, configuration detils will not be cloned to autoinst.xml",pattern)
      end
      isPatternInstalled
    end
    def SetupGrubDristributor
      # Setup distributor
      dist = OesVersion()
      Builtins.y2milestone("Setting GRUB distributor to \"#{dist}\"")
      Yast::Bootloader.Read if Stage.normal
      ::Bootloader::BootloaderFactory.current.grub_default.distributor = "#{dist}"
      Yast::Bootloader.Write if Stage.normal
    end

    def OesVersion
      ver = Misc.CustomSysconfigRead("DISPLAY_RELEASE", "", "/etc/novell-release")
      dist = "OES #{ver}"
      Builtins.y2milestone("OES Version is %1", dist)
      return dist
    end

    #Migrate the tcp and udp ports from SuSEfirewall2 to firewalld
    def MigrateFirewallPort
      firewalld = Y2Firewall::Firewalld.instance
      defaultZone = firewalld.api.default_zone
      Builtins.y2milestone("Default zone in firewall is: %1", defaultZone)

      #Read the TCP ports
      tcp_ports = Convert.to_string(
        SCR.Read(path(".sysconfig.SuSEfirewall2.FW_SERVICES_EXT_TCP"))
      )
      Builtins.y2milestone("NovellUtils:MigrateFirewallPort Read the value FW_SERVICES_EXT_TCP = %1", tcp_ports.split)

      #Read the UDP ports
      udp_ports = Convert.to_string(
        SCR.Read(path(".sysconfig.SuSEfirewall2.FW_SERVICES_EXT_UDP"))
      )
      Builtins.y2milestone("NovellUtils:MigrateFirewallPort Read the value FW_SERVICES_EXT_UDP = %1", udp_ports.split)

      #Open the ports in firewalld
      tcp_ports.split.each do |tcp_port|
        retVal = firewalld.api.add_port(defaultZone, "#{tcp_port}/tcp")
        Builtins.y2milestone(
          "NovellUtils:MigrateFirewallPort type: tcp  port: %1  return: %2",
          tcp_port,
          retVal
        )
      end

      udp_ports.split.each do |udp_port|
        retVal = firewalld.api.add_port(defaultZone, "#{udp_port}/udp")
        Builtins.y2milestone(
          "NovellUtils:MigrateFirewallPort type: udp  port: %1  return: %2",
          udp_port,
          retVal
        )
      end
    end

    publish :variable => :CifsPatternInstalled, :type => "boolean"
    publish :variable => :is_cis_agent_enabled, :type => "boolean"
    publish :variable => :edir_quick_install, :type => "boolean"
    publish :variable => :passwordMessage, :type => "string"
    publish :variable => :noConfigMessage, :type => "string"
    publish :variable => :dsfwUpgradeMsg, :type => "string"
    publish :variable => :dsfwUpgradeMsg_2, :type => "string"
    publish :variable => :dsfwMixedModeUpgardeMsg, :type => "string"
    publish :variable => :removeServiceProxyMsg, :type => "string"
    publish :variable => :proxyScriptPath, :type => "string"
    publish :variable => :channelUpgrade_abort, :type => "boolean"
    publish :variable => :channelUpgrade, :type => "boolean"
    publish :variable => :channelUpgrade_error, :type => "string"
    publish :variable => :adminPasswordExpireMsg, :type => "string"
    publish :variable => :domainPasswordExpireMsg, :type => "string"
    publish :function => :ReadOption, :type => "string (string)"
    publish :function => :ReadOptionFromSysconfig, :type => "string (string)"
    publish :function => :NDSPasswordCheck, :type => "string (string)"
    publish :function => :StripLeading, :type => "string (string, string)"
    publish :function => :StripTrailing, :type => "string (string, string)"
    publish :function => :Strip, :type => "string (string, string)"
    publish :function => :CleanUpString, :type => "string (string)"
    publish :function => :CombineLdapSearchReturn, :type => "string (string)"
    publish :function => :GetIPAddr, :type => "string ()"
    publish :function => :GetAllStaticIPAddrs, :type => "list ()"
    publish :function => :MapIPAddrToDevice, :type => "string (string)"
    publish :function => :GetHostname, :type => "string ()"
    publish :function => :GetDomainName, :type => "string ()"
    publish :function => :GetDNSname, :type => "string ()"
    publish :function => :GetHostnameFQDN, :type => "string ()"
    publish :function => :IseDirRunning, :type => "boolean ()"
    publish :function => :IsEdirectoryInstalledLocally, :type => "boolean ()"
    publish :function => :GetDefaultAndFallbackIPAddresses, :type => "map (string)"
    publish :function => :SetFirewallPort, :type => "boolean (integer, symbol, boolean)"
    publish :function => :ReloadFirewall, :type => "boolean ()"
    publish :function => :GetServerContext, :type => "string (string, string, string, string, integer, integer)"
    publish :function => :Nds2Ldap, :type => "string (string)"
    publish :function => :esdot, :type => "string (string)"
    publish :function => :Ldap2Nds, :type => "string (string)"
    publish :function => :isLDAP, :type => "boolean (string)"
    publish :function => :isNDS, :type => "boolean (string)"
    publish :function => :Typeful2Typeless, :type => "string (string)"
    publish :function => :GetContextNDS, :type => "string (string)"
    publish :function => :GetContextLDAP, :type => "string (string)"
    publish :function => :GetContext, :type => "string (string)"
    publish :function => :GetTopContextNDS, :type => "string (string)"
    publish :function => :ExtractCN, :type => "list (string)"
    publish :function => :GetServerFDN, :type => "string ()"
    publish :function => :IsLumNcsConfigured, :type => "boolean ()"
    publish :function => :IsSecureBoot, :type => "boolean ()"
    publish :function => :ImportOesCertificate, :type => "void ()"
    publish :function => :IsSelectionInstalled, :type => "boolean (string)"
    publish :function => :IsServerProduct, :type => "boolean ()"
    publish :function => :Get_UID, :type => "string (string)"
    publish :function => :Get_GID, :type => "string (string)"
    publish :function => :Get_User_GIDs, :type => "list <string> (string)"
    publish :function => :LumNcsCommonProxyCredUpdate, :type => "void ()"
    publish :function => :LUMAvailable, :type => "boolean ()"
    publish :function => :ModifyingSfcb, :type => "void ()"
    publish :function => :StoreTrustedCertificateInJavaKeystore, :type => "boolean (string, string, string)"
    publish :function => :PopupErrorDetails, :type => "void (string, map, string)"
    publish :function => :PopupErrorDetailsText, :type => "void (string, string)"
    publish :function => :PopupErrorDetailsWithLogFile, :type => "void (string, map, string, string)"
    publish :function => :IsAgentMounted, :type => "boolean (path, path)"
    publish :function => :EnsureAgentMounted, :type => "boolean (string, path, path)"
    publish :function => :EnsureSysconfigAgentMounted, :type => "boolean (string, path)"
    publish :function => :GenerateRandomPassword, :type => "string ()"
    publish :function => :ResetEdirAskUserFlag, :type => "boolean ()"
    publish :function => :SetEdirAskUserFlag, :type => "boolean ()"
    publish :function => :ProductInList, :type => "boolean (string)"
    publish :function => :AddProductToList, :type => "boolean (string)"
    publish :function => :EdirAskUserFlag, :type => "boolean (string)"
    publish :function => :SystemdStartorRestartService, :type => "void ()"
    publish :function => :CreateOESProxyUser, :type => "integer (string, string, string, string, string, integer, string)"
    publish :function => :StoreCredsInOCS, :type => "integer (string, string)"
    publish :function => :StoreCredsInOCSforUpgrade, :type => "integer (string, string, string)"
    publish :function => :CreateProxyPolicy, :type => "integer (string, string, string, integer, string)"
    publish :function => :AttachProxyUserToPolicy, :type => "integer (string, string, string, string, integer, string)"
    publish :function => :RetrieveProxyCredentials, :type => "string (string)"
    publish :function => :RetrievePolicyPassword, :type => "string (string, integer, string, string, string)"
    publish :function => :ProxyAddToCrontab, :type => "boolean ()"
    publish :function => :LogInstallMode, :type => "boolean ()"
    publish :function => :TranslateLDAPError, :type => "string (integer)"
    publish :function => :ServerIsDsfwServer, :type => "boolean (string)"
    publish :function => :Ping_IP, :type => "boolean (string)"
    publish :function => :AddtoOpenLdapConfFile, :type => "void ()"
    publish :function => :ServerIseDirServer, :type => "void (string)"
    publish :function => :CopyControlFile, :type => "void ()"
    publish :function => :AddKernelBootParam, :type => "void ()"
    publish :function => :MovetoCommonProxyDuringUpgrade, :type => "void (string)"
    publish :function => :RemoveKernelBootParam, :type => "void ()"
    publish :function => :CloneSettingstoAutoinstXml, :type => "boolean (string)"
    publish :function => :SetupGrubDristributor, :type => "void ()"
    publish :function => :DeleteMultiPemFile, :type => "void ()"
    publish :function => :UpgradeIsFrom, :type => "string ()"
    publish :function => :OesVersion, :type => "string ()"
    publish :function => :GetOldSysconfigMap, :type => "map (string)"
    publish :function => :FindUpgradeIsFrom, :type => "boolean ()"
    publish :function => :MigrateFirewallPort, :type => "void ()"
  end

  NovellUtils = NovellUtilsClass.new
  NovellUtils.main
end
